@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2013 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -34,10 +34,8 @@
 #include "zend_objects.h"
 #include "zend_objects_API.h"
 #include "zend_modules.h"
-
-#ifdef ZEND_MULTIBYTE
+#include "zend_float.h"
 #include "zend_multibyte.h"
-#endif /* ZEND_MULTIBYTE */
 
 /* Define ZTS if you want a thread-safe Zend */
 /*#undef ZTS*/
@@ -68,6 +66,8 @@ typedef struct _zend_declarables {
 } zend_declarables;
 
 typedef struct _zend_vm_stack *zend_vm_stack;
+typedef struct _zend_ini_entry zend_ini_entry;
+
 
 struct _zend_compiler_globals {
 	zend_stack bp_stack;
@@ -101,10 +101,10 @@ struct _zend_compiler_globals {
 
 	HashTable *auto_globals;
 
+	zend_bool parse_error;
 	zend_bool in_compilation;
 	zend_bool short_tags;
 	zend_bool asp_tags;
-	zend_bool allow_call_time_pass_reference;
 
 	zend_declarables declarables;
 
@@ -137,25 +137,25 @@ struct _zend_compiler_globals {
 	zend_bool  in_namespace;
 	zend_bool  has_bracketed_namespaces;
 
-	HashTable *labels;
-	zend_stack labels_stack;
+	zend_compiler_context context;
+	zend_stack context_stack;
 
-#ifdef ZEND_MULTIBYTE
-	zend_encoding **script_encoding_list;
+	/* interned strings */
+	char *interned_strings_start;
+	char *interned_strings_end;
+	char *interned_strings_top;
+	char *interned_strings_snapshot_top;
+
+	HashTable interned_strings;
+
+	const zend_encoding **script_encoding_list;
 	size_t script_encoding_list_size;
+	zend_bool multibyte;
 	zend_bool detect_unicode;
 	zend_bool encoding_declared;
 
-	zend_encoding *internal_encoding;
-
-	/* multibyte utility functions */
-	zend_encoding_detector encoding_detector;
-	zend_encoding_converter encoding_converter;
-	zend_encoding_oddlen encoding_oddlen;
-#endif /* ZEND_MULTIBYTE */
-
 #ifdef ZTS
-	HashTable **static_members;
+	zval ***static_members_table;
 	int last_static_member;
 #endif
 };
@@ -240,6 +240,7 @@ struct _zend_executor_globals {
 
 	HashTable *ini_directives;
 	HashTable *modified_ini_directives;
+	zend_ini_entry *error_reporting_ini_entry;	                
 
 	zend_objects_store objects_store;
 	zval *exception, *prev_exception;
@@ -254,7 +255,12 @@ struct _zend_executor_globals {
 
 	zend_bool active; 
 
-	void *saved_fpu_cw;
+	zend_op *start_op;
+
+	void *saved_fpu_cw_ptr;
+#if XPFPA_HAVE_CW
+	XPFPA_CW_DATATYPE saved_fpu_cw;
+#endif
 
 	void *reserved[ZEND_MAX_RESERVED_RESOURCES];
 };
@@ -292,7 +298,6 @@ struct _zend_php_scanner_globals {
 	int yy_state;
 	zend_stack state_stack;
 	
-#ifdef ZEND_MULTIBYTE
 	/* original (unfiltered) script */
 	unsigned char *script_org;
 	size_t script_org_size;
@@ -304,9 +309,7 @@ struct _zend_php_scanner_globals {
 	/* input/ouput filters */
 	zend_encoding_filter input_filter;
 	zend_encoding_filter output_filter;
-	zend_encoding *script_encoding;
-	zend_encoding *internal_encoding;
-#endif /* ZEND_MULTIBYTE */
+	const zend_encoding *script_encoding;
 };
 
 #endif /* ZEND_GLOBALS_H */
