@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2011 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        | 
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_indent.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id$ */
 
 /* This indenter doesn't really work, it's here for no particular reason. */
 
@@ -64,7 +64,7 @@ ZEND_API void zend_indent()
 	while ((token_type=lex_scan(&token TSRMLS_CC))) {
 		switch (token_type) {
 			case T_INLINE_HTML:
-				zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
+				zend_write((char*)LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 				break;
 			case T_WHITESPACE: {
 					token.type = 0;
@@ -118,16 +118,16 @@ dflt_printout:
 							} else {
 								handle_whitespace(emit_whitespace);
 							}
-							zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
+							zend_write((char*)LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 							break;
 					}
 				} else {
 					handle_whitespace(emit_whitespace);
 					if (in_string) {
-						zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
+						zend_write((char*)LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 						/* a part of a string */
 					} else {
-						zend_write(LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
+						zend_write((char*)LANG_SCNG(yy_text), LANG_SCNG(yy_leng));
 					}
 				}
 				break;
