@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_objects.c 293155 2010-01-05 20:46:53Z sebastian $ */
+/* $Id$ */
 
 #include "zend.h"
 #include "zend_globals.h"
@@ -28,10 +28,9 @@
 
 ZEND_API void zend_object_std_init(zend_object *object, zend_class_entry *ce TSRMLS_DC)
 {
-	ALLOC_HASHTABLE(object->properties);
-	zend_hash_init(object->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-
-	object->ce = ce;	
+	object->ce = ce;
+	object->properties = NULL;
+	object->properties_table = NULL;
 	object->guards = NULL;
 }
 
@@ -39,11 +38,23 @@ ZEND_API void zend_object_std_dtor(zend_object *object TSRMLS_DC)
 {
 	if (object->guards) {
 		zend_hash_destroy(object->guards);
-		FREE_HASHTABLE(object->guards);		
+		FREE_HASHTABLE(object->guards);
 	}
 	if (object->properties) {
 		zend_hash_destroy(object->properties);
 		FREE_HASHTABLE(object->properties);
+		if (object->properties_table) {
+			efree(object->properties_table);
+		}
+	} else if (object->properties_table) {
+		int i;
+
+		for (i = 0; i < object->ce->default_properties_count; i++) {
+			if (object->properties_table[i]) {
+				zval_ptr_dtor(&object->properties_table[i]);
+			}
+		}
+		efree(object->properties_table);
 	}
 }
 
@@ -52,8 +63,9 @@ ZEND_API void zend_objects_destroy_object(zend_object *object, zend_object_handl
 	zend_function *destructor = object ? object->ce->destructor : NULL;
 
 	if (destructor) {
-		zval *obj;
 		zval *old_exception;
+		zval *obj;
+		zend_object_store_bucket *obj_bucket;
 
 		if (destructor->op_array.fn_flags & (ZEND_ACC_PRIVATE|ZEND_ACC_PROTECTED)) {
 			if (destructor->op_array.fn_flags & ZEND_ACC_PRIVATE) {
@@ -62,23 +74,23 @@ ZEND_API void zend_objects_destroy_object(zend_object *object, zend_object_handl
 				if (object->ce != EG(scope)) {
 					zend_class_entry *ce = object->ce;
 
-					zend_error(EG(in_execution) ? E_ERROR : E_WARNING, 
-						"Call to private %s::__destruct() from context '%s'%s", 
-						ce->name, 
-						EG(scope) ? EG(scope)->name : "", 
+					zend_error(EG(in_execution) ? E_ERROR : E_WARNING,
+						"Call to private %s::__destruct() from context '%s'%s",
+						ce->name,
+						EG(scope) ? EG(scope)->name : "",
 						EG(in_execution) ? "" : " during shutdown ignored");
 					return;
 				}
 			} else {
 				/* Ensure that if we're calling a protected function, we're allowed to do so.
 				 */
-				if (!zend_check_protected(destructor->common.scope, EG(scope))) {
+				if (!zend_check_protected(zend_get_function_root_class(destructor), EG(scope))) {
 					zend_class_entry *ce = object->ce;
 
-					zend_error(EG(in_execution) ? E_ERROR : E_WARNING, 
-						"Call to protected %s::__destruct() from context '%s'%s", 
-						ce->name, 
-						EG(scope) ? EG(scope)->name : "", 
+					zend_error(EG(in_execution) ? E_ERROR : E_WARNING,
+						"Call to protected %s::__destruct() from context '%s'%s",
+						ce->name,
+						EG(scope) ? EG(scope)->name : "",
 						EG(in_execution) ? "" : " during shutdown ignored");
 					return;
 				}
@@ -88,30 +100,33 @@ ZEND_API void zend_objects_destroy_object(zend_object *object, zend_object_handl
 		MAKE_STD_ZVAL(obj);
 		Z_TYPE_P(obj) = IS_OBJECT;
 		Z_OBJ_HANDLE_P(obj) = handle;
-		/* TODO: We cannot set proper handlers. */
-		Z_OBJ_HT_P(obj) = &std_object_handlers; 
+		obj_bucket = &EG(objects_store).object_buckets[handle];
+		if (!obj_bucket->bucket.obj.handlers) {
+			obj_bucket->bucket.obj.handlers = &std_object_handlers;
+		}
+		Z_OBJ_HT_P(obj) = obj_bucket->bucket.obj.handlers;
 		zval_copy_ctor(obj);
 
 		/* Make sure that destructors are protected from previously thrown exceptions.
 		 * For example, if an exception was thrown in a function and when the function's
 		 * local variable destruction results in a destructor being called.
 		 */
-		old_exception = EG(exception);
-		EG(exception) = NULL;
+		old_exception = NULL;
+		if (EG(exception)) {
+			if (Z_OBJ_HANDLE_P(EG(exception)) == handle) {
+				zend_error(E_ERROR, "Attempt to destruct pending exception");
+			} else {
+				old_exception = EG(exception);
+				EG(exception) = NULL;
+			}
+		}
 		zend_call_method_with_0_params(&obj, object->ce, &destructor, ZEND_DESTRUCTOR_FUNC_NAME, NULL);
 		if (old_exception) {
 			if (EG(exception)) {
-				zend_class_entry *default_exception_ce = zend_exception_get_default(TSRMLS_C);
-				zval *file = zend_read_property(default_exception_ce, old_exception, "file", sizeof("file")-1, 1 TSRMLS_CC);
-				zval *line = zend_read_property(default_exception_ce, old_exception, "line", sizeof("line")-1, 1 TSRMLS_CC);
-
-				zval_ptr_dtor(&obj);
-				zval_ptr_dtor(&EG(exception));
+				zend_exception_set_previous(EG(exception), old_exception TSRMLS_CC);
+			} else {
 				EG(exception) = old_exception;
-				zend_error(E_ERROR, "Ignoring exception from %s::__destruct() while an exception is already active (Uncaught %s in %s on line %ld)", 
-					object->ce->name, Z_OBJCE_P(old_exception)->name, Z_STRVAL_P(file), Z_LVAL_P(line));
 			}
-			EG(exception) = old_exception;
 		}
 		zval_ptr_dtor(&obj);
 	}
@@ -124,49 +139,68 @@ ZEND_API void zend_objects_free_object_storage(zend_object *object TSRMLS_DC)
 }
 
 ZEND_API zend_object_value zend_objects_new(zend_object **object, zend_class_entry *class_type TSRMLS_DC)
-{	
+{
 	zend_object_value retval;
 
 	*object = emalloc(sizeof(zend_object));
 	(*object)->ce = class_type;
+	(*object)->properties = NULL;
+	(*object)->properties_table = NULL;
+	(*object)->guards = NULL;
 	retval.handle = zend_objects_store_put(*object, (zend_objects_store_dtor_t) zend_objects_destroy_object, (zend_objects_free_object_storage_t) zend_objects_free_object_storage, NULL TSRMLS_CC);
 	retval.handlers = &std_object_handlers;
-	(*object)->guards = NULL;
 	return retval;
 }
 
-ZEND_API zend_object *zend_objects_get_address(zval *zobject TSRMLS_DC)
+ZEND_API zend_object *zend_objects_get_address(const zval *zobject TSRMLS_DC)
 {
 	return (zend_object *)zend_object_store_get_object(zobject TSRMLS_CC);
 }
 
-static void zval_add_ref_or_clone(zval **p)
-{
-	if (Z_TYPE_PP(p) == IS_OBJECT && !PZVAL_IS_REF(*p)) {
-		TSRMLS_FETCH();
-
-		if (Z_OBJ_HANDLER_PP(p, clone_obj) == NULL) {
-			zend_error(E_ERROR, "Trying to clone an uncloneable object of class %s",  Z_OBJCE_PP(p)->name);
-		} else {
-			zval *orig = *p;
-
-			ALLOC_ZVAL(*p);
-			**p = *orig;
-			INIT_PZVAL(*p);
-			(*p)->value.obj = Z_OBJ_HT_PP(p)->clone_obj(orig TSRMLS_CC);
-		}
-	} else {
-		(*p)->refcount++;
-	}
-}
-
 ZEND_API void zend_objects_clone_members(zend_object *new_object, zend_object_value new_obj_val, zend_object *old_object, zend_object_handle handle TSRMLS_DC)
 {
-	if (EG(ze1_compatibility_mode)) {
-		zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref_or_clone, (void *) NULL /* Not used anymore */, sizeof(zval *));
-	} else {
-		zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref, (void *) NULL /* Not used anymore */, sizeof(zval *));
+	int i;
+
+	if (old_object->properties_table) {
+		if (!new_object->properties_table) {
+			new_object->properties_table = emalloc(sizeof(zval*) * old_object->ce->default_properties_count);
+			memset(new_object->properties_table, 0, sizeof(zval*) * old_object->ce->default_properties_count);
+		}
+		for (i = 0; i < old_object->ce->default_properties_count; i++) {
+			if (!new_object->properties) {
+				if (new_object->properties_table[i]) {
+					zval_ptr_dtor(&new_object->properties_table[i]);
+				}
+			}
+			if (!old_object->properties) {
+				new_object->properties_table[i] = old_object->properties_table[i];
+				if (new_object->properties_table[i]) {
+					Z_ADDREF_P(new_object->properties_table[i]);
+				}
+			}
+		}
 	}
+	if (old_object->properties) {
+		if (!new_object->properties) {
+			ALLOC_HASHTABLE(new_object->properties);
+			zend_hash_init(new_object->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
+		}
+		zend_hash_copy(new_object->properties, old_object->properties, (copy_ctor_func_t) zval_add_ref, (void *) NULL /* Not used anymore */, sizeof(zval *));
+		if (old_object->properties_table) {
+			HashPosition pos;
+			zend_property_info *prop_info;
+			for (zend_hash_internal_pointer_reset_ex(&old_object->ce->properties_info, &pos);
+			     zend_hash_get_current_data_ex(&old_object->ce->properties_info, (void**)&prop_info, &pos) == SUCCESS;
+			     zend_hash_move_forward_ex(&old_object->ce->properties_info, &pos)) {
+				if ((prop_info->flags & ZEND_ACC_STATIC) == 0) {
+					if (zend_hash_quick_find(new_object->properties, prop_info->name, prop_info->name_length+1, prop_info->h, (void**)&new_object->properties_table[prop_info->offset]) == FAILURE) {
+						new_object->properties_table[prop_info->offset] = NULL;
+					}
+				}
+			}
+		}
+	}
+
 	if (old_object->ce->clone) {
 		zval *new_obj;
 
@@ -188,14 +222,11 @@ ZEND_API zend_object_value zend_objects_clone_obj(zval *zobject TSRMLS_DC)
 	zend_object *new_object;
 	zend_object_handle handle = Z_OBJ_HANDLE_P(zobject);
 
-	/* assume that create isn't overwritten, so when clone depends on the 
+	/* assume that create isn't overwritten, so when clone depends on the
 	 * overwritten one then it must itself be overwritten */
 	old_object = zend_objects_get_address(zobject TSRMLS_CC);
 	new_obj_val = zend_objects_new(&new_object, old_object->ce TSRMLS_CC);
 
-	ALLOC_HASHTABLE(new_object->properties);
-	zend_hash_init(new_object->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-
 	zend_objects_clone_members(new_object, new_obj_val, old_object, handle TSRMLS_CC);
 
 	return new_obj_val;
