@@ -3,7 +3,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    | Authors: Dmitry Stogov <dmitry@zend.com>                             |
    +----------------------------------------------------------------------+
 
-	 $Id: zend_vm_gen.php 293155 2010-01-05 20:46:53Z sebastian $
+	 $Id$
 */
 
 $header_text = <<< DATA
@@ -24,7 +24,7 @@ $header_text = <<< DATA
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2012 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -82,7 +82,7 @@ $typecode = array(
 );
 
 $op1_type = array(
-	"ANY"    => "opline->op1.op_type",
+	"ANY"    => "opline->op1_type",
 	"TMP"    => "IS_TMP_VAR",
 	"VAR"    => "IS_VAR",
 	"CONST"  => "IS_CONST",
@@ -91,7 +91,7 @@ $op1_type = array(
 );
 
 $op2_type = array(
-	"ANY"    => "opline->op2.op_type",
+	"ANY"    => "opline->op2_type",
 	"TMP"    => "IS_TMP_VAR",
 	"VAR"    => "IS_VAR",
 	"CONST"  => "IS_CONST",
@@ -118,75 +118,75 @@ $op2_free = array(
 );
 
 $op1_get_zval_ptr = array(
-	"ANY"    => "get_zval_ptr(&opline->op1, EX(Ts), &free_op1, \\1)",
-	"TMP"    => "_get_zval_ptr_tmp(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
-	"VAR"    => "_get_zval_ptr_var(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
-	"CONST"  => "&opline->op1.u.constant",
+	"ANY"    => "get_zval_ptr(opline->op1_type, &opline->op1, EX_Ts(), &free_op1, \\1)",
+	"TMP"    => "_get_zval_ptr_tmp(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_var(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
+	"CONST"  => "opline->op1.zv",
 	"UNUSED" => "NULL",
-	"CV"     => "_get_zval_ptr_cv(&opline->op1, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_cv_\\1(EX_CVs(), opline->op1.var TSRMLS_CC)",
 );
 
 $op2_get_zval_ptr = array(
-	"ANY"    => "get_zval_ptr(&opline->op2, EX(Ts), &free_op2, \\1)",
-	"TMP"    => "_get_zval_ptr_tmp(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
-	"VAR"    => "_get_zval_ptr_var(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
-	"CONST"  => "&opline->op2.u.constant",
+	"ANY"    => "get_zval_ptr(opline->op2_type, &opline->op2, EX_Ts(), &free_op2, \\1)",
+	"TMP"    => "_get_zval_ptr_tmp(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_var(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
+	"CONST"  => "opline->op2.zv",
 	"UNUSED" => "NULL",
-	"CV"     => "_get_zval_ptr_cv(&opline->op2, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_cv_\\1(EX_CVs(), opline->op2.var TSRMLS_CC)",
 );
 
 $op1_get_zval_ptr_ptr = array(
-	"ANY"    => "get_zval_ptr_ptr(&opline->op1, EX(Ts), &free_op1, \\1)",
+	"ANY"    => "get_zval_ptr_ptr(opline->op1_type, &opline->op1, EX_Ts(), &free_op1, \\1)",
 	"TMP"    => "NULL",
-	"VAR"    => "_get_zval_ptr_ptr_var(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_ptr_var(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
 	"CONST"  => "NULL",
 	"UNUSED" => "NULL",
-	"CV"     => "_get_zval_ptr_ptr_cv(&opline->op1, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_ptr_cv_\\1(EX_CVs(), opline->op1.var TSRMLS_CC)",
 );
 
 $op2_get_zval_ptr_ptr = array(
-	"ANY"    => "get_zval_ptr_ptr(&opline->op2, EX(Ts), &free_op2, \\1)",
+	"ANY"    => "get_zval_ptr_ptr(opline->op2_type, &opline->op2, EX_Ts(), &free_op2, \\1)",
 	"TMP"    => "NULL",
-	"VAR"    => "_get_zval_ptr_ptr_var(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_ptr_var(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
 	"CONST"  => "NULL",
 	"UNUSED" => "NULL",
-	"CV"     => "_get_zval_ptr_ptr_cv(&opline->op2, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_ptr_cv_\\1(EX_CVs(), opline->op2.var TSRMLS_CC)",
 );
 
 $op1_get_obj_zval_ptr = array(
-	"ANY"    => "get_obj_zval_ptr(&opline->op1, EX(Ts), &free_op1, \\1)",
-	"TMP"    => "_get_zval_ptr_tmp(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
-	"VAR"    => "_get_zval_ptr_var(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
-	"CONST"  => "&opline->op1.u.constant",
+	"ANY"    => "get_obj_zval_ptr(opline->op1_type, &opline->op1, EX_Ts(), &free_op1, \\1)",
+	"TMP"    => "_get_zval_ptr_tmp(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_var(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
+	"CONST"  => "opline->op1.zv",
 	"UNUSED" => "_get_obj_zval_ptr_unused(TSRMLS_C)",
-	"CV"     => "_get_zval_ptr_cv(&opline->op1, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_cv_\\1(EX_CVs(), opline->op1.var TSRMLS_CC)",
 );
 
 $op2_get_obj_zval_ptr = array(
-	"ANY"    => "get_obj_zval_ptr(&opline->op2, EX(Ts), &free_op2, \\1)",
-	"TMP"    => "_get_zval_ptr_tmp(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
-	"VAR"    => "_get_zval_ptr_var(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
-	"CONST"  => "&opline->op2.u.constant",
+	"ANY"    => "get_obj_zval_ptr(opline->op2_type, &opline->op2, EX_Ts(), &free_op2, \\1)",
+	"TMP"    => "_get_zval_ptr_tmp(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_var(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
+	"CONST"  => "opline->op2.zv",
 	"UNUSED" => "_get_obj_zval_ptr_unused(TSRMLS_C)",
-	"CV"     => "_get_zval_ptr_cv(&opline->op2, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_cv_\\1(EX_CVs(), opline->op2.var TSRMLS_CC)",
 );
 
 $op1_get_obj_zval_ptr_ptr = array(
-	"ANY"    => "get_obj_zval_ptr_ptr(&opline->op1, EX(Ts), &free_op1, \\1)",
+	"ANY"    => "get_obj_zval_ptr_ptr(opline->op1_type, &opline->op1, EX_Ts(), &free_op1, \\1)",
 	"TMP"    => "NULL",
-	"VAR"    => "_get_zval_ptr_ptr_var(&opline->op1, EX(Ts), &free_op1 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_ptr_var(opline->op1.var, EX_Ts(), &free_op1 TSRMLS_CC)",
 	"CONST"  => "NULL",
 	"UNUSED" => "_get_obj_zval_ptr_ptr_unused(TSRMLS_C)",
-	"CV"     => "_get_zval_ptr_ptr_cv(&opline->op1, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_ptr_cv_\\1(EX_CVs(), opline->op1.var TSRMLS_CC)",
 );
 
 $op2_get_obj_zval_ptr_ptr = array(
-	"ANY"    => "get_obj_zval_ptr_ptr(&opline->op2, EX(Ts), &free_op2, \\1)",
+	"ANY"    => "get_obj_zval_ptr_ptr(opline->op2_type, &opline->op2, EX_Ts(), &free_op2, \\1)",
 	"TMP"    => "NULL",
-	"VAR"    => "_get_zval_ptr_ptr_var(&opline->op2, EX(Ts), &free_op2 TSRMLS_CC)",
+	"VAR"    => "_get_zval_ptr_ptr_var(opline->op2.var, EX_Ts(), &free_op2 TSRMLS_CC)",
 	"CONST"  => "NULL",
 	"UNUSED" => "_get_obj_zval_ptr_ptr_unused(TSRMLS_C)",
-	"CV"     => "_get_zval_ptr_ptr_cv(&opline->op2, EX(Ts), \\1 TSRMLS_CC)",
+	"CV"     => "_get_zval_ptr_ptr_cv_\\1(EX_CVs(), opline->op2.var TSRMLS_CC)",
 );
 
 $op1_is_tmp_free = array(
@@ -304,7 +304,7 @@ function helper_name($name, $spec, $op1, $op2) {
 }
 
 // Generates code for opcode handler or helper
-function gen_code($f, $spec, $kind, $code, $op1, $op2) {
+function gen_code($f, $spec, $kind, $export, $code, $op1, $op2, $name) {
 	global $op1_type, $op2_type, $op1_get_zval_ptr, $op2_get_zval_ptr,
 		$op1_get_zval_ptr_ptr, $op2_get_zval_ptr_ptr,
 		$op1_get_obj_zval_ptr, $op2_get_obj_zval_ptr,
@@ -343,7 +343,9 @@ function gen_code($f, $spec, $kind, $code, $op1, $op2) {
 			"/ZEND_VM_C_LABEL\(\s*([A-Za-z_]*)\s*\)/m",
 			"/ZEND_VM_C_GOTO\(\s*([A-Za-z_]*)\s*\)/m",
 			"/^#if\s+1\s*\\|\\|.*[^\\\\]$/m",
-			"/^#if\s+0\s*&&.*[^\\\\]$/m"
+			"/^#if\s+0\s*&&.*[^\\\\]$/m",
+			"/^#ifdef\s+ZEND_VM_EXPORT\s*\n/m",
+			"/^#ifndef\s+ZEND_VM_EXPORT\s*\n/m"
 		),
 		array(
 			$op1_type[$op1],
@@ -374,9 +376,14 @@ function gen_code($f, $spec, $kind, $code, $op1, $op2) {
 			"goto \\1".(($spec && $kind != ZEND_VM_KIND_CALL)?("_SPEC".$prefix[$op1].$prefix[$op2]):""),
 			"#if 1",
 			"#if 0",
+			$export?"#if 1\n":"#if 0\n",
+			$export?"#if 0\n":"#if 1\n"
 		),
 		$code);
 
+	if (0 && strpos($code, '{') === 0) {
+		$code = "{\n\tfprintf(stderr, \"$name\\n\");\n" . substr($code, 1);
+	}
 	// Updating code according to selected threading model
 	switch($kind) {
 		case ZEND_VM_KIND_CALL:
@@ -404,7 +411,7 @@ function gen_code($f, $spec, $kind, $code, $op1, $op2) {
 					"/ZEND_VM_DISPATCH_TO_HELPER_EX\(\s*([A-Za-z_]*)\s*,\s*([A-Za-z_]*)\s*,\s*(.*)\s*\);/me",
 				),
 				array(
-					"&execute_data",
+					"execute_data",
 					"goto \\1".($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_LABEL",
 					"'goto '.helper_name('\\1',$spec,'$op1','$op2')",
 					"'\\2 = \\3; goto '.helper_name('\\1',$spec,'$op1','$op2').';'",
@@ -420,7 +427,7 @@ function gen_code($f, $spec, $kind, $code, $op1, $op2) {
 					"/ZEND_VM_DISPATCH_TO_HELPER_EX\(\s*([A-Za-z_]*)\s*,\s*([A-Za-z_]*)\s*,\s*(.*)\s*\);/me",
 				),
 				array(
-					"&execute_data",
+					"execute_data",
 					"goto \\1".($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_HANDLER",
 					"'goto '.helper_name('\\1',$spec,'$op1','$op2')",
 					"'\\2 = \\3; goto '.helper_name('\\1',$spec,'$op1','$op2').';'",
@@ -481,7 +488,7 @@ function gen_handler($f, $spec, $kind, $name, $op1, $op2, $use, $code, $lineno)
 	// Generate opcode handler's entry point according to selected threading model
 	switch($kind) {
 		case ZEND_VM_KIND_CALL:
-			out($f,"static int ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_HANDLER(ZEND_OPCODE_HANDLER_ARGS)\n");
+			out($f,"static int ZEND_FASTCALL  ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_HANDLER(ZEND_OPCODE_HANDLER_ARGS)\n");
 			break;
 		case ZEND_VM_KIND_SWITCH:
 			if ($spec) {
@@ -497,12 +504,12 @@ function gen_handler($f, $spec, $kind, $name, $op1, $op2, $use, $code, $lineno)
 			}
 			break;
 		case ZEND_VM_KIND_GOTO:
-			out($f,$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_HANDLER:\n");
+			out($f,$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."_HANDLER: ZEND_VM_GUARD(".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2].");\n");
 			break;
 	}
 
 	// Generate opcode handler's code
-	gen_code($f, $spec, $kind, $code, $op1, $op2);
+	gen_code($f, $spec, $kind, 0, $code, $op1, $op2, $name);
 }
 
 // Generates helper
@@ -518,10 +525,10 @@ function gen_helper($f, $spec, $kind, $name, $op1, $op2, $param, $code, $lineno)
 		case ZEND_VM_KIND_CALL:
 			if ($param == null) {
 			  // Helper without parameters
-				out($f, "static int ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."(ZEND_OPCODE_HANDLER_ARGS)\n");
+				out($f, "static int ZEND_FASTCALL ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."(ZEND_OPCODE_HANDLER_ARGS)\n");
 			} else {
 			  // Helper with parameter
-				out($f, "static int ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."(".$param.", ZEND_OPCODE_HANDLER_ARGS)\n");
+				out($f, "static int ZEND_FASTCALL ".$name.($spec?"_SPEC":"").$prefix[$op1].$prefix[$op2]."(".$param.", ZEND_OPCODE_HANDLER_ARGS)\n");
 			}
 			break;
 		case ZEND_VM_KIND_SWITCH:
@@ -533,7 +540,7 @@ function gen_helper($f, $spec, $kind, $name, $op1, $op2, $param, $code, $lineno)
 	}
 
 	// Generate helper's code
-	gen_code($f, $spec, $kind, $code, $op1, $op2);
+	gen_code($f, $spec, $kind, 0, $code, $op1, $op2, $name);
 }
 
 // Generates array of opcode handlers (specialized or unspecialized)
@@ -685,10 +692,9 @@ function gen_null_handler($f) {
 	// for undefined opcodes, do we emit code for it only once
 	if (!$done) {
 		$done = 1;
-		out($f,"static int ZEND_NULL_HANDLER(ZEND_OPCODE_HANDLER_ARGS)\n");
+		out($f,"static int ZEND_FASTCALL ZEND_NULL_HANDLER(ZEND_OPCODE_HANDLER_ARGS)\n");
 		out($f,"{\n");
-		out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", EX(opline)->opcode, EX(opline)->op1.op_type, EX(opline)->op2.op_type);\n");
-		out($f,"\tZEND_VM_RETURN_FROM_EXECUTE_LOOP();\n");
+		out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", OPLINE->opcode, OPLINE->op1_type, OPLINE->op2_type);\n");
 		out($f,"}\n\n");
 	}
 }
@@ -762,13 +768,11 @@ function gen_executor_code($f, $spec, $kind, $prolog) {
 			break;
 		case ZEND_VM_KIND_SWITCH:
 			out($f,"default:\n");
-			out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", EX(opline)->opcode, EX(opline)->op1.op_type, EX(opline)->op2.op_type);\n");
-			out($f,"\tZEND_VM_RETURN_FROM_EXECUTE_LOOP();\n");
+			out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", OPLINE->opcode, OPLINE->op1_type, OPLINE->op2_type);\n");
 			break;
 		case ZEND_VM_KIND_GOTO:
 			out($f,"ZEND_NULL_HANDLER:\n");
-			out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", EX(opline)->opcode, EX(opline)->op1.op_type, EX(opline)->op2.op_type);\n");
-			out($f,"\tZEND_VM_RETURN_FROM_EXECUTE_LOOP();\n");
+			out($f,"\tzend_error_noreturn(E_ERROR, \"Invalid opcode %d/%d/%d.\", OPLINE->opcode, OPLINE->op1_type, OPLINE->op2_type);\n");
 			break;
 	}
 }
@@ -790,31 +794,118 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 		if (preg_match("/(.*)[{][%]([A-Z_]*)[%][}](.*)/", $line, $m)) {
 			switch ($m[2]) {
 				case "DEFINES":
-					if (ZEND_VM_OLD_EXECUTOR) {
+					if (ZEND_VM_OLD_EXECUTOR && $spec) {
 						out($f,"static int zend_vm_old_executor = 0;\n\n");
 					}
 					out($f,"static opcode_handler_t zend_vm_get_opcode_handler(zend_uchar opcode, zend_op* op);\n\n");
 					switch ($kind) {
 						case ZEND_VM_KIND_CALL:
-							out($f,"\n");
-							out($f,"#define ZEND_VM_CONTINUE()   return 0\n");
-							out($f,"#define ZEND_VM_RETURN()     return 1\n");
+							out($f,"\n");								
+							out($f,"#undef OPLINE\n");
+							out($f,"#undef DCL_OPLINE\n");
+							out($f,"#undef USE_OPLINE\n");
+							out($f,"#undef LOAD_OPLINE\n");
+							out($f,"#undef SAVE_OPLINE\n");
+							out($f,"#define OPLINE EX(opline)\n");
+							out($f,"#define DCL_OPLINE\n");
+							out($f,"#define USE_OPLINE zend_op *opline = EX(opline);\n");
+							out($f,"#define LOAD_OPLINE()\n");
+							out($f,"#define SAVE_OPLINE()\n");
+							out($f,"#undef CHECK_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION_LEAVE\n");
+							out($f,"#define CHECK_EXCEPTION() LOAD_OPLINE()\n");
+							out($f,"#define HANDLE_EXCEPTION() LOAD_OPLINE(); ZEND_VM_CONTINUE()\n");
+							out($f,"#define HANDLE_EXCEPTION_LEAVE() LOAD_OPLINE(); ZEND_VM_LEAVE()\n");
+							out($f,"#define LOAD_REGS()\n");
+							out($f,"#define ZEND_VM_CONTINUE()         return 0\n");
+							out($f,"#define ZEND_VM_RETURN()           return 1\n");
+							out($f,"#define ZEND_VM_ENTER()            return 2\n");
+							out($f,"#define ZEND_VM_LEAVE()            return 3\n");
 							out($f,"#define ZEND_VM_DISPATCH(opcode, opline) return zend_vm_get_opcode_handler(opcode, opline)(ZEND_OPCODE_HANDLER_ARGS_PASSTHRU);\n\n");
 							out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL execute_data TSRMLS_CC\n");
+							out($f,"#undef EX\n");
+							out($f,"#define EX(element) execute_data->element\n\n");
+							out($f,"#undef EX_CV\n");
+							out($f,"#define EX_CV(var) EX(CVs)[var]\n");
+							out($f,"#undef EX_CVs\n");
+							out($f,"#define EX_CVs() EX(CVs)\n");
+							out($f,"#undef EX_T\n");
+							out($f,"#define EX_T(offset) (*(temp_variable *)((char *) EX(Ts) + offset))\n");
+							out($f,"#undef EX_Ts\n");
+							out($f,"#define EX_Ts() EX(Ts)\n\n");
 							break;
 						case ZEND_VM_KIND_SWITCH:
 							out($f,"\n");
+							out($f,"#undef OPLINE\n");
+							out($f,"#undef DCL_OPLINE\n");
+							out($f,"#undef USE_OPLINE\n");
+							out($f,"#undef LOAD_OPLINE\n");
+							out($f,"#undef SAVE_OPLINE\n");
+							out($f,"#define OPLINE opline\n");
+							out($f,"#define DCL_OPLINE zend_op *opline;\n");
+							out($f,"#define USE_OPLINE\n");
+							out($f,"#define LOAD_OPLINE() opline = EX(opline)\n");
+							out($f,"#define SAVE_OPLINE() EX(opline) = opline\n");
+							out($f,"#undef CHECK_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION_LEAVE\n");
+							out($f,"#define CHECK_EXCEPTION() LOAD_OPLINE()\n");
+							out($f,"#define HANDLE_EXCEPTION() LOAD_OPLINE(); ZEND_VM_CONTINUE()\n");
+							out($f,"#define HANDLE_EXCEPTION_LEAVE() LOAD_OPLINE(); ZEND_VM_LEAVE()\n");
+							out($f,"#define LOAD_REGS()                do {Ts = EX(Ts); CVs = EX(CVs);} while (0)\n");
 							out($f,"#define ZEND_VM_CONTINUE() goto zend_vm_continue\n");
-							out($f,"#define ZEND_VM_RETURN()   return\n");
+							out($f,"#define ZEND_VM_RETURN()   EG(in_execution) = original_in_execution; return\n");
+							out($f,"#define ZEND_VM_ENTER()    op_array = EG(active_op_array); goto zend_vm_enter\n");
+							out($f,"#define ZEND_VM_LEAVE()    ZEND_VM_CONTINUE()\n");
 							out($f,"#define ZEND_VM_DISPATCH(opcode, opline) dispatch_handler = zend_vm_get_opcode_handler(opcode, opline); goto zend_vm_dispatch;\n\n");
-							out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL &execute_data TSRMLS_CC\n");
+							out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL execute_data TSRMLS_CC\n");
+							out($f,"#undef EX\n");
+							out($f,"#define EX(element) execute_data->element\n\n");
+							out($f,"#undef EX_CV\n");
+							out($f,"#define EX_CV(var) CVs[var]\n");
+							out($f,"#undef EX_CVs\n");
+							out($f,"#define EX_CVs() CVs\n");
+							out($f,"#undef EX_T\n");
+							out($f,"#define EX_T(offset) T(offset)\n");
+							out($f,"#undef EX_Ts\n");
+							out($f,"#define EX_Ts() Ts\n\n");
 							break;
 						case ZEND_VM_KIND_GOTO:
 							out($f,"\n");
-							out($f,"#define ZEND_VM_CONTINUE() goto *(void**)(EX(opline)->handler)\n");
-							out($f,"#define ZEND_VM_RETURN()   return\n");
+							out($f,"#undef OPLINE\n");
+							out($f,"#undef DCL_OPLINE\n");
+							out($f,"#undef USE_OPLINE\n");
+							out($f,"#undef LOAD_OPLINE\n");
+							out($f,"#undef SAVE_OPLINE\n");
+							out($f,"#define OPLINE opline\n");
+							out($f,"#define DCL_OPLINE zend_op *opline;\n");
+							out($f,"#define USE_OPLINE\n");
+							out($f,"#define LOAD_OPLINE() opline = EX(opline)\n");
+							out($f,"#define SAVE_OPLINE() EX(opline) = opline\n");
+							out($f,"#undef CHECK_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION\n");
+							out($f,"#undef HANDLE_EXCEPTION_LEAVE\n");
+							out($f,"#define CHECK_EXCEPTION() if (UNEXPECTED(EG(exception) != NULL)) goto ZEND_HANDLE_EXCEPTION_SPEC_HANDLER\n");
+							out($f,"#define HANDLE_EXCEPTION() goto ZEND_HANDLE_EXCEPTION_SPEC_HANDLER\n");
+							out($f,"#define HANDLE_EXCEPTION_LEAVE() goto ZEND_HANDLE_EXCEPTION_SPEC_HANDLER\n");
+							out($f,"#define LOAD_REGS()                do {Ts = EX(Ts); CVs = EX(CVs);} while (0)\n");
+							out($f,"#define ZEND_VM_CONTINUE() goto *(void**)(OPLINE->handler)\n");
+							out($f,"#define ZEND_VM_RETURN()   EG(in_execution) = original_in_execution; return\n");
+							out($f,"#define ZEND_VM_ENTER()    op_array = EG(active_op_array); goto zend_vm_enter\n");
+							out($f,"#define ZEND_VM_LEAVE()    ZEND_VM_CONTINUE()\n");
 							out($f,"#define ZEND_VM_DISPATCH(opcode, opline) goto *(void**)(zend_vm_get_opcode_handler(opcode, opline));\n\n");
-							out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL &execute_data TSRMLS_CC\n");
+							out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL execute_data TSRMLS_CC\n");
+							out($f,"#undef EX\n");
+							out($f,"#define EX(element) execute_data->element\n\n");
+							out($f,"#undef EX_CV\n");
+							out($f,"#define EX_CV(var) CVs[var]\n");
+							out($f,"#undef EX_CVs\n");
+							out($f,"#define EX_CVs() CVs\n");
+							out($f,"#undef EX_T\n");
+							out($f,"#define EX_T(offset) T(offset)\n\n");
+							out($f,"#undef EX_Ts\n");
+							out($f,"#define EX_Ts() Ts\n\n");
 							break;
 					}
 					break;
@@ -823,6 +914,8 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 					break;
 				case "HELPER_VARS":
 					if ($kind != ZEND_VM_KIND_CALL) {
+						out($f,$m[1]."temp_variable *Ts;\n");
+						out($f,$m[1]."zval ***CVs;\n");
 						if ($kind == ZEND_VM_KIND_SWITCH) {
 							out($f,$m[1]."opcode_handler_t dispatch_handler;\n");
 						}
@@ -834,6 +927,13 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 						skip_blanks($f, $m[1], $m[3]."\n");
 					}
 					break;
+				case "EXECUTION_STATUS":
+					if ($kind != ZEND_VM_KIND_GOTO) {
+						out($f, $m[1] . "zend_bool original_in_execution = EG(in_execution);\n");
+					} else {
+						out($f, $m[1] . "zend_bool original_in_execution = op_array? EG(in_execution) : 0;\n");
+					}
+					break;
 				case "INTERNAL_LABELS":
 					if ($kind == ZEND_VM_KIND_GOTO) {
 					  // Emit array of labels of opcode handlers and code for
@@ -851,7 +951,10 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 					}
 					break;
 				case "ZEND_VM_CONTINUE_LABEL":
-					if ($kind == ZEND_VM_KIND_SWITCH) {
+					if ($kind == ZEND_VM_KIND_CALL) {
+					  // Only SWITCH dispatch method use it
+						out($f,$m[1]."\tint ret;".$m[3]."\n");
+					} else if ($kind == ZEND_VM_KIND_SWITCH) {
 					  // Only SWITCH dispatch method use it
 						out($f,"zend_vm_continue:".$m[3]."\n");
 					} else {
@@ -862,20 +965,31 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 				  // Emit code that dispatches to opcode handler
 					switch ($kind) {
 						case ZEND_VM_KIND_CALL:
-							out($f, $m[1]."if (EX(opline)->handler(&execute_data TSRMLS_CC) > 0)".$m[3]."\n");
+							out($f, $m[1]."if ((ret = OPLINE->handler(execute_data TSRMLS_CC)) > 0)".$m[3]."\n");
 							break;
 						case ZEND_VM_KIND_SWITCH:
-							out($f, $m[1]."dispatch_handler = EX(opline)->handler;\nzend_vm_dispatch:\n".$m[1]."switch ((int)dispatch_handler)".$m[3]."\n");
+							out($f, $m[1]."dispatch_handler = OPLINE->handler;\nzend_vm_dispatch:\n".$m[1]."switch ((int)dispatch_handler)".$m[3]."\n");
 							break;
 						case ZEND_VM_KIND_GOTO:
-							out($f, $m[1]."goto *(void**)(EX(opline)->handler);".$m[3]."\n");
+							out($f, $m[1]."goto *(void**)(OPLINE->handler);".$m[3]."\n");
 							break;
 					}
 					break;
 				case "INTERNAL_EXECUTOR":
 					if ($kind == ZEND_VM_KIND_CALL) {
 					  // Executor is defined as a set of functions
-						out($f, $m[1]."return;".$m[3]."\n");
+						out($f, $m[1]."switch (ret) {\n" . 
+						        $m[1]."\tcase 1:\n" . 
+						        $m[1]."\t\tEG(in_execution) = original_in_execution;\n".
+						        $m[1]."\t\treturn;\n".
+						        $m[1]."\tcase 2:\n" . 
+						        $m[1]."\t\top_array = EG(active_op_array);\n".
+						        $m[1]."\t\tgoto zend_vm_enter;\n".
+						        $m[1]."\tcase 3:\n" . 
+						        $m[1]."\t\texecute_data = EG(current_execute_data);\n".
+						        $m[1]."\tdefault:\n".
+						        $m[1]."\t\tbreak;\n".
+						        $m[1]."}".$m[3]."\n");
 					} else {
 					  // Emit executor code
 						gen_executor_code($f, $spec, $kind, $m[1]);
@@ -886,9 +1000,7 @@ function gen_executor($f, $skl, $spec, $kind, $executor_name, $initializer_name,
 					  // Unspecialized executor with CALL threading is the same as the
 					  // old one, so we don't need to produce code twitch
 						if (!$old || ZEND_VM_SPEC || (ZEND_VM_KIND != ZEND_VM_KIND_CALL)) {
-							out($f,"#undef EX\n");
-							out($f,"#define EX(element) execute_data->element\n\n");
-						  // Emit executor code
+							// Emit executor code
 							gen_executor_code($f, $spec, $kind, $m[1]);
 						}
 					}
@@ -1097,18 +1209,22 @@ function gen_vm($def, $skel) {
 	// Insert header
 	out($f, $GLOBALS['header_text']);
 
+	// Suppress free_op1 warnings on Windows
+	out($f, "#ifdef ZEND_WIN32\n# pragma warning(once : 4101)\n#endif\n");
+	
 	// Support for ZEND_USER_OPCODE
-	out($f, "static opcode_handler_t zend_user_opcode_handlers[256] = {");
+	out($f, "static user_opcode_handler_t zend_user_opcode_handlers[256] = {\n");
 	for ($i = 0; $i < 255; ++$i) {
-		out($f, "(opcode_handler_t)NULL,");
+		out($f, "\t(user_opcode_handler_t)NULL,\n");
 	}
-	out($f, "(opcode_handler_t)NULL};\n\n");
+	out($f, "\t(user_opcode_handler_t)NULL\n};\n\n");
 
 	out($f, "static zend_uchar zend_user_opcodes[256] = {");
 	for ($i = 0; $i < 255; ++$i) {
+		if ($i % 16 == 1) out($f, "\n\t");
 		out($f, "$i,");
 	}
-	out($f, "255};\n\n");
+	out($f, "255\n};\n\n");
 
 	// Generate specialized executor
 	gen_executor($f, $skl, ZEND_VM_SPEC, ZEND_VM_KIND, "execute", "zend_init_opcodes_handlers", 0);
@@ -1118,8 +1234,16 @@ function gen_vm($def, $skel) {
 		out($f,"\n/* Old executor */\n\n");
 		out($f,"#undef EX\n");
 		out($f,"#define EX(element) execute_data.element\n\n");
+		out($f,"#undef EX_CV\n");
+		out($f,"#define EX_CV(var) EX(CVs)[var]\n");
+		out($f,"#undef EX_CVs\n");
+		out($f,"#define EX_CVs() EX(CVs)\n");
+		out($f,"#undef EX_T\n");
+		out($f,"#define EX_T(offset) (*(temp_variable *)((char *) EX(Ts) + offset))\n\n");
 		out($f,"#undef ZEND_VM_CONTINUE\n\n");
 		out($f,"#undef ZEND_VM_RETURN\n\n");
+		out($f,"#undef ZEND_VM_ENTER\n\n");
+		out($f,"#undef ZEND_VM_LEAVE\n\n");
 		out($f,"#undef ZEND_VM_DISPATCH\n\n");
 		out($f,"#undef ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL\n\n");
 		gen_executor($f, $skl, 0, ZEND_VM_KIND_CALL, "old_execute", "zend_vm_use_old_executor", 1);
@@ -1155,7 +1279,7 @@ function gen_vm($def, $skel) {
 		out($f, "\t\t\t_UNUSED_CODE, /* 15             */\n");
 		out($f, "\t\t\t_CV_CODE      /* 16 = IS_CV     */\n");
 		out($f, "\t\t};\n");
-		out($f, "\t\treturn zend_opcode_handlers[opcode * 25 + zend_vm_decode[op->op1.op_type] * 5 + zend_vm_decode[op->op2.op_type]];\n");
+		out($f, "\t\treturn zend_opcode_handlers[opcode * 25 + zend_vm_decode[op->op1_type] * 5 + zend_vm_decode[op->op2_type]];\n");
 		if (ZEND_VM_OLD_EXECUTOR) {
 			out($f, "\t}\n");
 		}
@@ -1172,14 +1296,40 @@ function gen_vm($def, $skel) {
 	if (count($export) > 0 &&
 	    !ZEND_VM_OLD_EXECUTOR &&
 	    ZEND_VM_KIND != ZEND_VM_KIND_CALL) {
+		out($f,"#undef OPLINE\n");
+		out($f,"#undef DCL_OPLINE\n");
+		out($f,"#undef USE_OPLINE\n");
+		out($f,"#undef LOAD_OPLINE\n");
+		out($f,"#undef SAVE_OPLINE\n");
+		out($f,"#define OPLINE EX(opline)\n");
+		out($f,"#define DCL_OPLINE\n");
+		out($f,"#define USE_OPLINE zend_op *opline = EX(opline);\n");
+		out($f,"#define LOAD_OPLINE()\n");
+		out($f,"#define SAVE_OPLINE()\n");
+		out($f,"#undef CHECK_EXCEPTION\n");
+		out($f,"#undef HANDLE_EXCEPTION\n");
+		out($f,"#undef HANDLE_EXCEPTION_LEAVE\n");
+		out($f,"#define CHECK_EXCEPTION() LOAD_OPLINE()\n");
+		out($f,"#define HANDLE_EXCEPTION() LOAD_OPLINE(); ZEND_VM_CONTINUE()\n");
+		out($f,"#define HANDLE_EXCEPTION_LEAVE() LOAD_OPLINE(); ZEND_VM_LEAVE()\n");
 		out($f,"#undef EX\n");
 		out($f,"#define EX(element) execute_data->element\n\n");
+		out($f,"#undef EX_CV\n");
+		out($f,"#define EX_CV(var) EX(CVs)[var]\n");
+		out($f,"#undef EX_CVs\n");
+		out($f,"#define EX_CVs() EX(CVs)\n");
+		out($f,"#undef EX_T\n");
+		out($f,"#define EX_T(offset) (*(temp_variable *)((char *) EX(Ts) + offset))\n\n");
 		out($f,"#undef ZEND_VM_CONTINUE\n");
 		out($f,"#undef ZEND_VM_RETURN\n");
+		out($f,"#undef ZEND_VM_ENTER\n");
+		out($f,"#undef ZEND_VM_LEAVE\n");
 		out($f,"#undef ZEND_VM_DISPATCH\n");
 		out($f,"#undef ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL\n\n");
 		out($f,"#define ZEND_VM_CONTINUE()   return 0\n");
 		out($f,"#define ZEND_VM_RETURN()     return 1\n");
+		out($f,"#define ZEND_VM_ENTER()      return 2\n");
+		out($f,"#define ZEND_VM_LEAVE()      return 3\n");
 		out($f,"#define ZEND_VM_DISPATCH(opcode, opline) return zend_vm_get_opcode_handler(opcode, opline)(ZEND_OPCODE_HANDLER_ARGS_PASSTHRU);\n\n");
 		out($f,"#define ZEND_OPCODE_HANDLER_ARGS_PASSTHRU_INTERNAL execute_data TSRMLS_CC\n\n");
 	}
@@ -1223,7 +1373,7 @@ function gen_vm($def, $skel) {
 			}
 		}
 		if (!$done) {
-			gen_code($f, 0, ZEND_VM_KIND_CALL, $code, 'ANY', 'ANY');
+			gen_code($f, 0, ZEND_VM_KIND_CALL, 1, $code, 'ANY', 'ANY', $name);
 		}
 	}
 
