@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2011 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: namednodemap.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -74,7 +74,7 @@ const zend_function_entry php_dom_namednodemap_class_functions[] = { /* {{{ */
 	PHP_FALIAS(getNamedItemNS, dom_namednodemap_get_named_item_ns, arginfo_dom_namednodemap_get_named_item_ns)
 	PHP_FALIAS(setNamedItemNS, dom_namednodemap_set_named_item_ns, arginfo_dom_namednodemap_set_named_item_ns)
 	PHP_FALIAS(removeNamedItemNS, dom_namednodemap_remove_named_item_ns, arginfo_dom_namednodemap_remove_named_item_ns)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 /* }}} */
 
@@ -126,7 +126,7 @@ Since:
 */
 PHP_FUNCTION(dom_namednodemap_get_named_item)
 {
-	zval *id, *rv = NULL;
+	zval *id;
 	int ret, namedlen=0;
 	dom_object *intern;
 	xmlNodePtr itemnode = NULL;
@@ -166,7 +166,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item)
 	}
 
 	if (itemnode) {
-		DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
+		DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
 		return;
 	} else {
 		RETVAL_NULL();
@@ -200,7 +200,7 @@ Since:
 */
 PHP_FUNCTION(dom_namednodemap_item)
 {
-	zval *id, *rv = NULL;
+	zval *id;
 	long index;
 	int ret;
 	dom_object *intern;
@@ -243,7 +243,7 @@ PHP_FUNCTION(dom_namednodemap_item)
 		}
 
 		if (itemnode) {
-			DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
+			DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
 			return;
 		}
 	}
@@ -258,7 +258,7 @@ Since: DOM Level 2
 */
 PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 {
-	zval *id, *rv = NULL;
+	zval *id;
 	int ret, namedlen=0, urilen=0;
 	dom_object *intern;
 	xmlNodePtr itemnode = NULL;
@@ -298,7 +298,7 @@ PHP_FUNCTION(dom_namednodemap_get_named_item_ns)
 	}
 
 	if (itemnode) {
-		DOM_RET_OBJ(rv, itemnode, &ret, objmap->baseobj);
+		DOM_RET_OBJ(itemnode, &ret, objmap->baseobj);
 		return;
 	} else {
 		RETVAL_NULL();
