@@ -63,6 +63,9 @@ FILE_RCSID("@(#)$File: apprentice.c,v 1.173 2011/12/08 12:38:24 rrt Exp $")
 #include <assert.h>
 #include <ctype.h>
 #include <fcntl.h>
+#ifndef PHP_WIN32
+#include <dirent.h>
+#endif
 
 #define	EATAB {while (isascii((unsigned char) *l) && \
 		      isspace((unsigned char) *l))  ++l;}
@@ -756,10 +759,8 @@ apprentice_load(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
 	size_t files = 0, maxfiles = 0;
 	char **filearr = NULL, mfn[MAXPATHLEN];
 	struct stat st;
-	php_stream *dir;
-	php_stream_dirent d;
-
-	TSRMLS_FETCH();
+	DIR *dir;
+	struct dirent *d;
 
 	ms->flags |= MAGIC_CHECK;	/* Enable checks for parsed files */
 
@@ -775,19 +776,17 @@ apprentice_load(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
         /* FIXME: Read file names and sort them to prevent
            non-determinism. See Debian bug #488562. */
 	if (php_sys_stat(fn, &st) == 0 && S_ISDIR(st.st_mode)) {
- 		int mflen;
- 		char mfn[MAXPATHLEN];
- 
- 		dir = php_stream_opendir(fn, REPORT_ERRORS, NULL);
+		dir = opendir(fn);
 		if (!dir) {
 			errs++;
 			goto out;
 		}
- 		while (php_stream_readdir(dir, &d)) {
- 			if ((mflen = snprintf(mfn, sizeof(mfn), "%s/%s", fn, d.d_name)) < 0) {
-				file_oomem(ms, strlen(fn) + strlen(d.d_name) + 2);
+		while ((d = readdir(dir)) != NULL) {
+			if ((mflen = snprintf(mfn, sizeof(mfn), "%s/%s", fn, d->d_name)) < 0) {
+				file_oomem(ms,
+				    strlen(fn) + strlen(d->d_name) + 2);
 				errs++;
- 				php_stream_closedir(dir);
+				closedir(dir);
 				goto out;
 			}
 			if (stat(mfn, &st) == -1 || !S_ISREG(st.st_mode)) {
@@ -800,14 +799,14 @@ apprentice_load(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
 				if ((filearr = CAST(char **,
 				    realloc(filearr, mlen))) == NULL) {
 					file_oomem(ms, mlen);
- 					php_stream_closedir(dir);
+					closedir(dir);
 					errs++;
 					goto out;
 				}
 			}
 			filearr[files++] = estrndup(mfn, mflen);
 		}
- 		php_stream_closedir(dir);
+		closedir(dir);
 		qsort(filearr, files, sizeof(*filearr), cmpstrp);
 		for (i = 0; i < files; i++) {
 			load_1(ms, action, filearr[i], &errs, &marray,
@@ -2205,16 +2204,6 @@ apprentice_map(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
 		goto internal_loaded;
 	}
 
-#ifdef PHP_WIN32
-	/* Don't bother on windows with php_stream_open_wrapper,
-	return to give apprentice_load() a chance. */
-	if (php_stream_stat_path_ex(fn, 0, &st, NULL) == SUCCESS) {
-               if (st.sb.st_mode & S_IFDIR) {
-                       goto error2;
-               }
-       }
-#endif
-
 	dbname = mkdbname(ms, fn, 0);
 	if (dbname == NULL)
 		goto error2;
@@ -2406,11 +2395,7 @@ mkdbname(struct magic_set *ms, const char *fn, int strip)
 	/* Compatibility with old code that looked in .mime */
 	if (ms->flags & MAGIC_MIME) {
 		spprintf(&buf, MAXPATHLEN, "%.*s.mime%s", (int)(q - fn), fn, ext);
-#ifdef PHP_WIN32
-		if (VCWD_ACCESS(buf, R_OK) == 0) {
-#else
 		if (VCWD_ACCESS(buf, R_OK) != -1) {
-#endif
 			ms->flags &= MAGIC_MIME_TYPE;
 			return buf;
 		}
