@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2011 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   |          Ulf Wendel <uw@php.net>                                     |
   +----------------------------------------------------------------------+
 
-  $Id: mysqli_nonapi.c 307223 2011-01-07 14:22:30Z andrey $
+  $Id$
 */
 
 #ifdef HAVE_CONFIG_H
@@ -141,10 +141,12 @@ void mysqli_common_connect(INTERNAL_FUNCTION_PARAMETERS, zend_bool is_real_conne
 		hostname = MyG(default_host);
 	}
 
-       if (mysql->mysql && mysqli_resource && (mysqli_resource->status > MYSQLI_STATUS_INITIALIZED || (strlen(SAFE_STR(hostname)) > 2 && !strncasecmp(hostname, "p:", 2)))) {
-                /* already connected, we should close the connection */
-                php_mysqli_close(mysql, MYSQLI_CLOSE_IMPLICIT, mysqli_resource->status TSRMLS_CC);
-        }
+	if (mysql->mysql && mysqli_resource &&
+		(mysqli_resource->status > MYSQLI_STATUS_INITIALIZED))
+	{
+		/* already connected, we should close the connection */
+		php_mysqli_close(mysql, MYSQLI_CLOSE_IMPLICIT, mysqli_resource->status TSRMLS_CC);
+	}
 
 	if (strlen(SAFE_STR(hostname)) > 2 && !strncasecmp(hostname, "p:", 2)) {
 		hostname += 2;
@@ -172,7 +174,7 @@ void mysqli_common_connect(INTERNAL_FUNCTION_PARAMETERS, zend_bool is_real_conne
 							/* reset variables */
 
 #ifndef MYSQLI_NO_CHANGE_USER_ON_PCONNECT
-							if (!mysqli_change_user_silent(mysql->mysql, username, passwd, dbname)) {
+							if (!mysqli_change_user_silent(mysql->mysql, username, passwd, dbname, passwd_len)) {
 #else
 							if (!mysql_ping(mysql->mysql)) {
 #endif
@@ -385,17 +387,6 @@ PHP_FUNCTION(mysqli_fetch_all)
 /* }}} */
 
 
-/* {{{ proto array mysqli_cache_stats(void) U
-   Returns statistics about the zval cache */
-PHP_FUNCTION(mysqli_get_cache_stats)
-{
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
-	array_init(return_value);
-}
-/* }}} */
-
 
 /* {{{ proto array mysqli_get_client_stats(void)
    Returns statistics about the zval cache */
@@ -427,6 +418,93 @@ PHP_FUNCTION(mysqli_get_connection_stats)
 #endif
 /* }}} */
 
+/* {{{ proto mixed mysqli_error_list (object connection)
+   Fetches all client errors */
+PHP_FUNCTION(mysqli_error_list)
+{
+	MY_MYSQL	*mysql;
+	zval		*mysql_link;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O", &mysql_link, mysqli_link_class_entry) == FAILURE) {
+		return;
+	}
+	MYSQLI_FETCH_RESOURCE_CONN(mysql, &mysql_link, MYSQLI_STATUS_VALID);
+	array_init(return_value);
+#if defined(MYSQLI_USE_MYSQLND)
+	if (mysql->mysql->data->error_info->error_list) {
+		MYSQLND_ERROR_LIST_ELEMENT * message;
+		zend_llist_position pos;
+		for (message = (MYSQLND_ERROR_LIST_ELEMENT *) zend_llist_get_first_ex(mysql->mysql->data->error_info->error_list, &pos);
+			 message;
+			 message = (MYSQLND_ERROR_LIST_ELEMENT *) zend_llist_get_next_ex(mysql->mysql->data->error_info->error_list, &pos)) 
+		{
+			zval * single_error;
+			MAKE_STD_ZVAL(single_error);
+			array_init(single_error);
+			add_assoc_long_ex(single_error, "errno", sizeof("errno"), message->error_no);
+			add_assoc_string_ex(single_error, "sqlstate", sizeof("sqlstate"), message->sqlstate, 1);
+			add_assoc_string_ex(single_error, "error", sizeof("error"), message->error, 1);
+			add_next_index_zval(return_value, single_error);
+		}
+	}
+#else
+	if (mysql_errno(mysql->mysql)) {
+		zval * single_error;
+		MAKE_STD_ZVAL(single_error);
+		array_init(single_error);
+		add_assoc_long_ex(single_error, "errno", sizeof("errno"), mysql_errno(mysql->mysql));
+		add_assoc_string_ex(single_error, "sqlstate", sizeof("sqlstate"), mysql_sqlstate(mysql->mysql), 1);
+		add_assoc_string_ex(single_error, "error", sizeof("error"), mysql_error(mysql->mysql), 1);
+		add_next_index_zval(return_value, single_error);
+	}
+#endif
+}
+/* }}} */
+
+
+/* {{{ proto string mysqli_stmt_error_list(object stmt)
+*/
+PHP_FUNCTION(mysqli_stmt_error_list)
+{
+	MY_STMT	*stmt;
+	zval 	*mysql_stmt;
+
+	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "O", &mysql_stmt, mysqli_stmt_class_entry) == FAILURE) {
+		return;
+	}
+	MYSQLI_FETCH_RESOURCE_STMT(stmt, &mysql_stmt, MYSQLI_STATUS_INITIALIZED);
+	array_init(return_value);
+#if defined(MYSQLI_USE_MYSQLND)
+	if (stmt->stmt && stmt->stmt->data && stmt->stmt->data->error_info->error_list) {
+		MYSQLND_ERROR_LIST_ELEMENT * message;
+		zend_llist_position pos;
+		for (message = (MYSQLND_ERROR_LIST_ELEMENT *) zend_llist_get_first_ex(stmt->stmt->data->error_info->error_list, &pos);
+			 message;
+			 message = (MYSQLND_ERROR_LIST_ELEMENT *) zend_llist_get_next_ex(stmt->stmt->data->error_info->error_list, &pos)) 
+		{
+			zval * single_error;
+			MAKE_STD_ZVAL(single_error);
+			array_init(single_error);
+			add_assoc_long_ex(single_error, "errno", sizeof("errno"), message->error_no);
+			add_assoc_string_ex(single_error, "sqlstate", sizeof("sqlstate"), message->sqlstate, 1);
+			add_assoc_string_ex(single_error, "error", sizeof("error"), message->error, 1);
+			add_next_index_zval(return_value, single_error);
+		}
+	}
+#else
+	if (mysql_stmt_errno(stmt->stmt)) {
+		zval * single_error;
+		MAKE_STD_ZVAL(single_error);
+		array_init(single_error);
+		add_assoc_long_ex(single_error, "errno", sizeof("errno"), mysql_stmt_errno(stmt->stmt));
+		add_assoc_string_ex(single_error, "sqlstate", sizeof("sqlstate"), mysql_stmt_sqlstate(stmt->stmt), 1);
+		add_assoc_string_ex(single_error, "error", sizeof("error"), mysql_stmt_error(stmt->stmt), 1);
+		add_next_index_zval(return_value, single_error);
+	}
+#endif
+}
+/* }}} */
+
 
 /* {{{ proto mixed mysqli_fetch_object (object result [, string class_name [, NULL|array ctor_params]])
    Fetch a result row as an object */
@@ -461,7 +539,7 @@ PHP_FUNCTION(mysqli_multi_query)
 		strcpy(s_sqlstate, mysql_sqlstate(mysql->mysql));
 		s_errno = mysql_errno(mysql->mysql);
 #else
-		MYSQLND_ERROR_INFO error_info = mysql->mysql->error_info;
+		MYSQLND_ERROR_INFO error_info = *mysql->mysql->data->error_info;
 #endif
 		MYSQLI_REPORT_MYSQL_ERROR(mysql->mysql);
 		MYSQLI_DISABLE_MQ;
@@ -472,7 +550,7 @@ PHP_FUNCTION(mysqli_multi_query)
 		strcpy(mysql->mysql->net.sqlstate, s_sqlstate);
 		mysql->mysql->net.last_errno = s_errno;
 #else
-		mysql->mysql->error_info = error_info;
+		*mysql->mysql->data->error_info = error_info;
 #endif
 		RETURN_FALSE;
 	}
@@ -703,6 +781,11 @@ PHP_FUNCTION(mysqli_poll)
 		RETURN_FALSE;
 	}
 
+	if (!r_array && !e_array) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "No stream arrays were passed");
+		RETURN_FALSE;
+	}
+
 	if (r_array != NULL) {
 		mysqlnd_zval_array_to_mysqlnd_array(r_array, &new_r_array TSRMLS_CC);
 	}
@@ -835,7 +918,11 @@ PHP_FUNCTION(mysqli_get_warnings)
 	MYSQLI_FETCH_RESOURCE_CONN(mysql, &mysql_link, MYSQLI_STATUS_VALID);
 
 	if (mysql_warning_count(mysql->mysql)) {
+#ifdef MYSQLI_USE_MYSQLND
+		w = php_get_warnings(mysql->mysql->data TSRMLS_CC);
+#else
 		w = php_get_warnings(mysql->mysql TSRMLS_CC);
+#endif
 	} else {
 		RETURN_FALSE;
 	}
@@ -846,6 +933,7 @@ PHP_FUNCTION(mysqli_get_warnings)
 }
 /* }}} */
 
+
 /* {{{ proto object mysqli_stmt_get_warnings(object link) */
 PHP_FUNCTION(mysqli_stmt_get_warnings)
 {
@@ -871,6 +959,7 @@ PHP_FUNCTION(mysqli_stmt_get_warnings)
 }
 /* }}} */
 
+
 #ifdef HAVE_MYSQLI_SET_CHARSET
 /* {{{ proto bool mysqli_set_charset(object link, string csname)
    sets client character set */
@@ -914,7 +1003,6 @@ PHP_FUNCTION(mysqli_get_charset)
 	}
 	MYSQLI_FETCH_RESOURCE_CONN(mysql, &mysql_link, MYSQLI_STATUS_VALID);
 
-	object_init(return_value);
 
 #if !defined(MYSQLI_USE_MYSQLND)
 	mysql_get_character_set_info(mysql->mysql, &cs);
@@ -927,7 +1015,11 @@ PHP_FUNCTION(mysqli_get_charset)
 	state = cs.state;
 	comment = cs.comment;
 #else
-	cs = mysql->mysql->charset;
+	cs = mysql->mysql->data->charset;
+	if (!cs) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "The connection has no charset associated");
+		RETURN_NULL();
+	}
 	name = cs->name;
 	collation = cs->collation;
 	minlength = cs->char_minlen;
@@ -936,6 +1028,7 @@ PHP_FUNCTION(mysqli_get_charset)
 	comment = cs->comment;
 	state = 1;	/* all charsets are compiled in */
 #endif
+	object_init(return_value);
 
 	add_property_string(return_value, "charset", (name) ? (char *)name : "", 1);
 	add_property_string(return_value, "collation",(collation) ? (char *)collation : "", 1);
