@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: pdo_dbh.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 /* The PDO Database Handle Class */
 
@@ -57,7 +57,7 @@ void pdo_raise_impl_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *sqlstate
 		pdo_err = &stmt->error_code;
 	}
 
-	strcpy(*pdo_err, sqlstate);
+	strncpy(*pdo_err, sqlstate, 6);
 
 	/* hash sqlstate to error messages */
 	msg = pdo_sqlstate_state_to_description(*pdo_err);
@@ -187,7 +187,7 @@ static char *dsn_from_uri(char *uri, char *buf, size_t buflen TSRMLS_DC) /* {{{
 	php_stream *stream;
 	char *dsn = NULL;
 
-	stream = php_stream_open_wrapper(uri, "rb", ENFORCE_SAFE_MODE|REPORT_ERRORS, NULL);
+	stream = php_stream_open_wrapper(uri, "rb", REPORT_ERRORS, NULL);
 	if (stream) {
 		dsn = php_stream_get_line(stream, buf, buflen, NULL);
 		php_stream_close(stream);
@@ -229,6 +229,7 @@ static PHP_METHOD(PDO, dbh_constructor)
 		snprintf(alt_dsn, sizeof(alt_dsn), "pdo.dsn.%s", data_source);
 		if (FAILURE == cfg_get_string(alt_dsn, &ini_dsn)) {
 			zend_throw_exception_ex(php_pdo_get_exception(), 0 TSRMLS_CC, "invalid data source name");
+			zval_dtor(object);
 			ZVAL_NULL(object);
 			return;
 		}
@@ -328,23 +329,28 @@ static PHP_METHOD(PDO, dbh_constructor)
 				memcpy((char *)pdbh->persistent_id, hashkey, plen+1);
 				pdbh->persistent_id_len = plen+1;
 				pdbh->refcount = 1;
+				pdbh->std.properties = NULL;
 			}
 		}
 
 		if (pdbh) {
 			/* let's copy the emalloc bits over from the other handle */
-			pdbh->ce = dbh->ce;
-			pdbh->def_stmt_ce = dbh->def_stmt_ce;
-			pdbh->def_stmt_ctor_args = dbh->def_stmt_ctor_args;
-			pdbh->properties = dbh->properties;
+			if (pdbh->std.properties) {
+				zend_hash_destroy(dbh->std.properties);	
+				efree(dbh->std.properties);
+			} else {
+				pdbh->std.ce = dbh->std.ce;
+				pdbh->def_stmt_ce = dbh->def_stmt_ce;
+				pdbh->def_stmt_ctor_args = dbh->def_stmt_ctor_args;
+				pdbh->std.properties = dbh->std.properties;
+				pdbh->std.properties_table = dbh->std.properties_table;
+			}
 			/* kill the non-persistent thingamy */
 			efree(dbh);
 			/* switch over to the persistent one */
 			dbh = pdbh;
 			zend_object_store_set_object(object, dbh TSRMLS_CC);
-			if (!call_factory) {
-				dbh->refcount++;
-			}
+			dbh->refcount++;
 		}
 
 		if (hashkey) {
@@ -352,11 +358,13 @@ static PHP_METHOD(PDO, dbh_constructor)
 		}
 	}
 	
-	dbh->data_source_len = strlen(colon + 1);
-	dbh->data_source = (const char*)pestrdup(colon + 1, is_persistent);
-	dbh->username = username ? pestrdup(username, is_persistent) : NULL;
-	dbh->password = password ? pestrdup(password, is_persistent) : NULL;
-	dbh->default_fetch_type = PDO_FETCH_BOTH;
+	if (call_factory) {
+		dbh->data_source_len = strlen(colon + 1);
+		dbh->data_source = (const char*)pestrdup(colon + 1, is_persistent);
+		dbh->username = username ? pestrdup(username, is_persistent) : NULL;
+		dbh->password = password ? pestrdup(password, is_persistent) : NULL;
+		dbh->default_fetch_type = PDO_FETCH_BOTH;
+	}	
 
 	dbh->auto_commit = pdo_attr_lval(options, PDO_ATTR_AUTOCOMMIT, 1 TSRMLS_CC);
 
@@ -394,7 +402,7 @@ options:
 		if (options) {
 			zval **attr_value;
 			char *str_key;
-			long long_key;
+			ulong long_key;
 			
 			zend_hash_internal_pointer_reset(Z_ARRVAL_P(options));
 			while (SUCCESS == zend_hash_get_current_data(Z_ARRVAL_P(options), (void**)&attr_value) 
@@ -429,8 +437,8 @@ static zval *pdo_stmt_instantiate(pdo_dbh_t *dbh, zval *object, zend_class_entry
 
 	Z_TYPE_P(object) = IS_OBJECT;
 	object_init_ex(object, dbstmt_ce);
-	object->refcount = 1;
-	object->is_ref = 1;
+	Z_SET_REFCOUNT_P(object, 1);
+	Z_SET_ISREF_P(object);
 	
 	return object;
 } /* }}} */
@@ -443,7 +451,7 @@ static void pdo_stmt_construct(pdo_stmt_t *stmt, zval *object, zend_class_entry
 	MAKE_STD_ZVAL(query_string);
 	ZVAL_STRINGL(query_string, stmt->query_string, stmt->query_stringlen, 1);
 	ZVAL_STRINGL(&z_key, "queryString", sizeof("queryString")-1, 0);
-	std_object_handlers.write_property(object, &z_key, query_string TSRMLS_CC);
+	std_object_handlers.write_property(object, &z_key, query_string, 0 TSRMLS_CC);
 	zval_ptr_dtor(&query_string);
 
 	if (dbstmt_ce->constructor) {
@@ -454,7 +462,7 @@ static void pdo_stmt_construct(pdo_stmt_t *stmt, zval *object, zend_class_entry
 		fci.size = sizeof(zend_fcall_info);
 		fci.function_table = &dbstmt_ce->function_table;
 		fci.function_name = NULL;
-		fci.object_pp = &object;
+		fci.object_ptr = object;
 		fci.symbol_table = NULL;
 		fci.retval_ptr_ptr = &retval;
 		if (ctor_args) {
@@ -477,7 +485,8 @@ static void pdo_stmt_construct(pdo_stmt_t *stmt, zval *object, zend_class_entry
 		fcc.initialized = 1;
 		fcc.function_handler = dbstmt_ce->constructor;
 		fcc.calling_scope = EG(scope);
-		fcc.object_pp = &object;
+		fcc.called_scope = Z_OBJCE_P(object);
+		fcc.object_ptr = object;
 
 		if (zend_call_function(&fci, &fcc TSRMLS_CC) == FAILURE) {
 			zval_dtor(object);
@@ -597,6 +606,9 @@ static PHP_METHOD(PDO, beginTransaction)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (dbh->in_txn) {
@@ -627,6 +639,9 @@ static PHP_METHOD(PDO, commit)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (!dbh->in_txn) {
@@ -650,6 +665,9 @@ static PHP_METHOD(PDO, rollBack)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 	PDO_CONSTRUCT_CHECK;
 
 	if (!dbh->in_txn) {
@@ -667,6 +685,25 @@ static PHP_METHOD(PDO, rollBack)
 }
 /* }}} */
 
+/* {{{ proto bool PDO::inTransaction()
+   determine if inside a transaction */
+static PHP_METHOD(PDO, inTransaction)
+{
+	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+	PDO_CONSTRUCT_CHECK;
+
+	if (!dbh->methods->in_transaction) {
+		RETURN_BOOL(dbh->in_txn);
+	}	
+
+	RETURN_BOOL(dbh->methods->in_transaction(dbh TSRMLS_CC));
+}
+/* }}} */
+
 static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_DC) /* {{{ */
 {
 
@@ -792,7 +829,7 @@ static int pdo_dbh_attribute_set(pdo_dbh_t *dbh, long attr, zval *value TSRMLS_D
 					PDO_HANDLE_DBH_ERR();
 					return FAILURE;
 				}
-				(*item)->refcount++;
+				Z_ADDREF_PP(item);
 				dbh->def_stmt_ctor_args = *item;
 			}
 			return SUCCESS;
@@ -835,6 +872,7 @@ static PHP_METHOD(PDO, setAttribute)
 		RETURN_FALSE;
 	}
 
+	PDO_DBH_CLEAR_ERR();
 	PDO_CONSTRUCT_CHECK;
 
 	if (pdo_dbh_attribute_set(dbh, attr, value TSRMLS_CC) != FAILURE) {
@@ -879,7 +917,7 @@ static PHP_METHOD(PDO, getAttribute)
 			array_init(return_value);
 			add_next_index_string(return_value, dbh->def_stmt_ce->name, 1);
 			if (dbh->def_stmt_ctor_args) {
-				dbh->def_stmt_ctor_args->refcount++;
+				Z_ADDREF_P(dbh->def_stmt_ctor_args);
 				add_next_index_zval(return_value, dbh->def_stmt_ctor_args);
 			}
 			return;
@@ -922,6 +960,7 @@ static PHP_METHOD(PDO, exec)
 	}
 
 	if (!statement_len) {
+		pdo_raise_impl_error(dbh, NULL, "HY000",  "trying to execute an empty query" TSRMLS_CC);
 		RETURN_FALSE;
 	}
 	PDO_DBH_CLEAR_ERR();
@@ -972,8 +1011,8 @@ static PHP_METHOD(PDO, errorCode)
 {
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (ZEND_NUM_ARGS()) {
-		RETURN_FALSE;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 	PDO_CONSTRUCT_CHECK;
 
@@ -981,6 +1020,14 @@ static PHP_METHOD(PDO, errorCode)
 		RETURN_STRING(dbh->query_stmt->error_code, 1);
 	}
 	
+	if (dbh->error_code[0] == '\0') {
+		RETURN_NULL();
+	}
+
+	/**
+	 * Making sure that we fallback to the default implementation
+	 * if the dbh->error_code is not null.
+	 */
 	RETURN_STRING(dbh->error_code, 1);
 }
 /* }}} */
@@ -989,11 +1036,16 @@ static PHP_METHOD(PDO, errorCode)
    Fetch extended error information associated with the last operation on the database handle */
 static PHP_METHOD(PDO, errorInfo)
 {
+	int error_count;
+	int error_count_diff 	 = 0;
+	int error_expected_count = 3;
+
 	pdo_dbh_t *dbh = zend_object_store_get_object(getThis() TSRMLS_CC);
 
-	if (ZEND_NUM_ARGS()) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
+
 	PDO_CONSTRUCT_CHECK;
 
 	array_init(return_value);
@@ -1003,9 +1055,26 @@ static PHP_METHOD(PDO, errorInfo)
 	} else {
 		add_next_index_string(return_value, dbh->error_code, 1);
 	}
+
 	if (dbh->methods->fetch_err) {
 		dbh->methods->fetch_err(dbh, dbh->query_stmt, return_value TSRMLS_CC);
 	}
+	
+	/**
+	 * In order to be consistent, we have to make sure we add the good amount
+	 * of nulls depending on the current number of elements. We make a simple
+	 * difference and add the needed elements
+	 */
+	error_count = zend_hash_num_elements(Z_ARRVAL_P(return_value));
+
+	if (error_expected_count > error_count) {
+		int current_index;
+
+		error_count_diff = error_expected_count - error_count;
+		for (current_index = 0; current_index < error_count_diff; current_index++) {
+			add_next_index_null(return_value);
+		}
+	}
 }
 /* }}} */
 
@@ -1018,6 +1087,12 @@ static PHP_METHOD(PDO, query)
 	char *statement;
 	int statement_len;
 
+	/* Return a meaningful error when no parameters were passed */
+	if (!ZEND_NUM_ARGS()) {
+		zend_parse_parameters(0 TSRMLS_CC, "z|z", NULL, NULL);
+		RETURN_FALSE;
+	}
+	
 	if (FAILURE == zend_parse_parameters(1 TSRMLS_CC, "s", &statement,
 			&statement_len)) {
 		RETURN_FALSE;
@@ -1048,8 +1123,8 @@ static PHP_METHOD(PDO, query)
 	ZVAL_NULL(&stmt->lazy_object_ref);
 
 	if (dbh->methods->preparer(dbh, statement, statement_len, stmt, NULL TSRMLS_CC)) {
+		PDO_STMT_CLEAR_ERR();
 		if (ZEND_NUM_ARGS() == 1 || SUCCESS == pdo_stmt_setup_fetch_mode(INTERNAL_FUNCTION_PARAM_PASSTHRU, stmt, 1)) {
-			PDO_STMT_CLEAR_ERR();
 
 			/* now execute the statement */
 			PDO_STMT_CLEAR_ERR();
@@ -1133,10 +1208,10 @@ static PHP_METHOD(PDO, getAvailableDrivers)
 	HashPosition pos;
 	pdo_driver_t **pdriver;
 
-	if (ZEND_NUM_ARGS()) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
-
+	
 	array_init(return_value);
 
 	zend_hash_internal_pointer_reset_ex(&pdo_driver_hash, &pos);
@@ -1154,53 +1229,63 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo___construct, 0, 0, 3)
 	ZEND_ARG_INFO(0, passwd)
 	ZEND_ARG_INFO(0, options) /* array */
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_prepare, 0, 0, 1)
-	ZEND_ARG_INFO(0, statment)
+	ZEND_ARG_INFO(0, statement)
 	ZEND_ARG_INFO(0, options) /* array */
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO(arginfo_pdo_setattribute, 0)
 	ZEND_ARG_INFO(0, attribute)
 	ZEND_ARG_INFO(0, value)
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO(arginfo_pdo_getattribute, 0)
 	ZEND_ARG_INFO(0, attribute)
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO(arginfo_pdo_exec, 0)
 	ZEND_ARG_INFO(0, query)
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_lastinsertid, 0, 0, 0)
 	ZEND_ARG_INFO(0, seqname)
 ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_pdo_quote, 0, 0, 1)
 	ZEND_ARG_INFO(0, string)
 	ZEND_ARG_INFO(0, paramtype)
 ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO(arginfo_pdo__void, 0)
+ZEND_END_ARG_INFO()
 /* }}} */
 
-zend_function_entry pdo_dbh_functions[] = {
+const zend_function_entry pdo_dbh_functions[] = {
 	ZEND_MALIAS(PDO, __construct, dbh_constructor,	arginfo_pdo___construct,	ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, prepare, 				arginfo_pdo_prepare,		ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, beginTransaction,		NULL,			ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, commit,			NULL,					ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, rollBack,		NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, beginTransaction,       arginfo_pdo__void,         ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, commit,                 arginfo_pdo__void,         ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, rollBack,               arginfo_pdo__void,         ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, inTransaction,          arginfo_pdo__void,         ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, setAttribute,	arginfo_pdo_setattribute,	ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, exec,			arginfo_pdo_exec,		ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, query,			NULL,					ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, lastInsertId,	arginfo_pdo_lastinsertid,	ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, errorCode,		NULL,					ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, errorInfo,		NULL,					ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, errorCode,              arginfo_pdo__void,         ZEND_ACC_PUBLIC)
+	PHP_ME(PDO, errorInfo,              arginfo_pdo__void,         ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, getAttribute,	arginfo_pdo_getattribute,	ZEND_ACC_PUBLIC)
 	PHP_ME(PDO, quote,			arginfo_pdo_quote,		ZEND_ACC_PUBLIC)
-	PHP_ME(PDO, __wakeup,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(PDO, __sleep,		NULL,					ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
-	PHP_ME(PDO, getAvailableDrivers, NULL,				ZEND_ACC_PUBLIC|ZEND_ACC_STATIC)
+	PHP_ME(PDO, __wakeup,               arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(PDO, __sleep,                arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(PDO, getAvailableDrivers,    arginfo_pdo__void,         ZEND_ACC_PUBLIC|ZEND_ACC_STATIC)
 	{NULL, NULL, NULL}
 };
 
 /* {{{ overloaded object handlers for PDO class */
 int pdo_hash_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
 {
-	zend_function_entry *funcs;
+	const zend_function_entry *funcs;
 	zend_function func;
 	zend_internal_function *ifunc = (zend_internal_function*)&func;
 	int namelen;
@@ -1222,30 +1307,38 @@ int pdo_hash_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
 	while (funcs->fname) {
 		ifunc->type = ZEND_INTERNAL_FUNCTION;
 		ifunc->handler = funcs->handler;
-		ifunc->function_name = funcs->fname;
-		ifunc->scope = dbh->ce;
+		ifunc->function_name = (char*)funcs->fname;
+		ifunc->scope = dbh->std.ce;
 		ifunc->prototype = NULL;
+		if (funcs->flags) {
+			ifunc->fn_flags = funcs->flags | ZEND_ACC_NEVER_CACHE;
+		} else {
+			ifunc->fn_flags = ZEND_ACC_PUBLIC | ZEND_ACC_NEVER_CACHE;
+		}
 		if (funcs->arg_info) {
-			ifunc->arg_info = funcs->arg_info + 1;
+			zend_internal_function_info *info = (zend_internal_function_info*)funcs->arg_info;
+
+			ifunc->arg_info = (zend_arg_info*)funcs->arg_info + 1;
 			ifunc->num_args = funcs->num_args;
-			if (funcs->arg_info[0].required_num_args == -1) {
+			if (info->required_num_args == -1) {
 				ifunc->required_num_args = funcs->num_args;
 			} else {
-				ifunc->required_num_args = funcs->arg_info[0].required_num_args;
+				ifunc->required_num_args = info->required_num_args;
+			}
+			if (info->pass_rest_by_reference) {
+				if (info->pass_rest_by_reference == ZEND_SEND_PREFER_REF) {
+					ifunc->fn_flags |= ZEND_ACC_PASS_REST_PREFER_REF;
+				} else {
+					ifunc->fn_flags |= ZEND_ACC_PASS_REST_BY_REFERENCE;
+				}
+			}
+			if (info->return_reference) {
+				ifunc->fn_flags |= ZEND_ACC_RETURN_REFERENCE;
 			}
-			ifunc->pass_rest_by_reference = funcs->arg_info[0].pass_by_reference;
-			ifunc->return_reference = funcs->arg_info[0].return_reference;
 		} else {
 			ifunc->arg_info = NULL;
 			ifunc->num_args = 0;
 			ifunc->required_num_args = 0;
-			ifunc->pass_rest_by_reference = 0;
-			ifunc->return_reference = 0;
-		}
-		if (funcs->flags) {
-			ifunc->fn_flags = funcs->flags;
-		} else {
-			ifunc->fn_flags = ZEND_ACC_PUBLIC;
 		}
 		namelen = strlen(funcs->fname);
 		lc_name = emalloc(namelen+1);
@@ -1264,7 +1357,7 @@ static union _zend_function *dbh_method_get(
 #else
 	zval *object,
 #endif
-	char *method_name, int method_len TSRMLS_DC)
+	char *method_name, int method_len, const zend_literal *key TSRMLS_DC)
 {
 	zend_function *fbc = NULL;
 	char *lc_method_name;
@@ -1276,7 +1369,7 @@ static union _zend_function *dbh_method_get(
 	lc_method_name = emalloc(method_len + 1);
 	zend_str_tolower_copy(lc_method_name, method_name, method_len);
 
-	if ((fbc = std_object_handlers.get_method(object_pp, lc_method_name, method_len TSRMLS_CC)) == NULL) {
+	if ((fbc = std_object_handlers.get_method(object_pp, method_name, method_len, key TSRMLS_CC)) == NULL) {
 		/* not a pre-defined method, nor a user-defined method; check
 		 * the driver specific methods */
 		if (!dbh->cls_methods[PDO_DBH_DRIVER_METHOD_KIND_DBH]) {
@@ -1293,7 +1386,6 @@ static union _zend_function *dbh_method_get(
 				fbc = NULL;
 			}
 		}
-		/* got it */
 	}
 
 out:
@@ -1306,14 +1398,7 @@ static int dbh_compare(zval *object1, zval *object2 TSRMLS_DC)
 	return -1;
 }
 
-static zend_object_value dbh_ze1_clone_obj(zval *object TSRMLS_DC)
-{
-	php_error(E_ERROR, "Cannot clone object of class %s due to 'zend.ze1_compatibility_mode'", Z_OBJCE_P(object)->name);
-	return object->value.obj;
-}
-
 static zend_object_handlers pdo_dbh_object_handlers;
-static zend_object_handlers pdo_dbh_object_handlers_ze1;
 
 void pdo_dbh_init(TSRMLS_D)
 {
@@ -1327,11 +1412,6 @@ void pdo_dbh_init(TSRMLS_D)
 	pdo_dbh_object_handlers.get_method = dbh_method_get;
 	pdo_dbh_object_handlers.compare_objects = dbh_compare;
 	
-	memcpy(&pdo_dbh_object_handlers_ze1, &std_object_handlers, sizeof(zend_object_handlers));
-	pdo_dbh_object_handlers_ze1.get_method = dbh_method_get;
-	pdo_dbh_object_handlers_ze1.compare_objects = dbh_compare;
-	pdo_dbh_object_handlers_ze1.clone_obj = dbh_ze1_clone_obj;
-
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_BOOL", (long)PDO_PARAM_BOOL);
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_NULL", (long)PDO_PARAM_NULL);
 	REGISTER_PDO_CLASS_CONST_LONG("PARAM_INT",  (long)PDO_PARAM_INT);
@@ -1489,37 +1569,30 @@ static void pdo_dbh_free_storage(pdo_dbh_t *dbh TSRMLS_DC)
 		dbh->methods->rollback(dbh TSRMLS_CC);
 		dbh->in_txn = 0;
 	}
-
-	if (dbh->properties) {
-		zend_hash_destroy(dbh->properties);
-		efree(dbh->properties);
-		dbh->properties = NULL;
-	}
 	
-	if (!dbh->is_persistent) {
-		dbh_free(dbh TSRMLS_CC);
-	} else if (dbh->methods && dbh->methods->persistent_shutdown) {
+	if (dbh->is_persistent && dbh->methods && dbh->methods->persistent_shutdown) {
 		dbh->methods->persistent_shutdown(dbh TSRMLS_CC);
 	}
+	zend_object_std_dtor(&dbh->std TSRMLS_CC);
+	dbh->std.properties = NULL;
+	dbh_free(dbh TSRMLS_CC);
 }
 
 zend_object_value pdo_dbh_new(zend_class_entry *ce TSRMLS_DC)
 {
 	zend_object_value retval;
 	pdo_dbh_t *dbh;
-	zval *tmp;
 
 	dbh = emalloc(sizeof(*dbh));
 	memset(dbh, 0, sizeof(*dbh));
-	dbh->ce = ce;
+	zend_object_std_init(&dbh->std, ce TSRMLS_CC);
+	object_properties_init(&dbh->std, ce);
+	rebuild_object_properties(&dbh->std);
 	dbh->refcount = 1;
-	ALLOC_HASHTABLE(dbh->properties);
-	zend_hash_init(dbh->properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-	zend_hash_copy(dbh->properties, &ce->default_properties, (copy_ctor_func_t) zval_add_ref, (void *) &tmp, sizeof(zval *));
 	dbh->def_stmt_ce = pdo_dbstmt_ce;
 	
 	retval.handle = zend_objects_store_put(dbh, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t)pdo_dbh_free_storage, NULL TSRMLS_CC);
-	retval.handlers = EG(ze1_compatibility_mode) ? &pdo_dbh_object_handlers_ze1 : &pdo_dbh_object_handlers;
+	retval.handlers = &pdo_dbh_object_handlers;
 	
 	return retval;
 }
