@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2011 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,14 +18,13 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_pdo_mysql_int.h 307529 2011-01-17 09:54:22Z kalle $ */
+/* $Id$ */
 
 #ifndef PHP_PDO_MYSQL_INT_H
 #define PHP_PDO_MYSQL_INT_H
 
 #if defined(PDO_USE_MYSQLND)
 #	include "ext/mysqlnd/mysqlnd.h"
-#	include "ext/mysql/mysql_mysqlnd.h"
 #	include "ext/mysqlnd/mysqlnd_libmysql_compat.h"
 #	define PDO_MYSQL_PARAM_BIND MYSQLND_PARAM_BIND
 #else
@@ -128,7 +127,6 @@ typedef struct {
 	long			*current_lengths;
 #endif
 	pdo_mysql_error_info einfo;
-#if HAVE_MYSQL_STMT_PREPARE || PDO_USE_MYSQLND
 #if PDO_USE_MYSQLND
 	MYSQLND_STMT 		*stmt;
 #else
@@ -145,7 +143,6 @@ typedef struct {
 	unsigned long		*out_length;
 	unsigned int		params_given;
 	unsigned		max_length:1;
-#endif
 } pdo_mysql_stmt;
 
 extern pdo_driver_t pdo_mysql_driver;
@@ -164,11 +161,16 @@ enum {
 	PDO_MYSQL_ATTR_READ_DEFAULT_FILE,
 	PDO_MYSQL_ATTR_READ_DEFAULT_GROUP,
 	PDO_MYSQL_ATTR_MAX_BUFFER_SIZE,
-	PDO_MYSQL_ATTR_COMPRESS,
 #endif
+	PDO_MYSQL_ATTR_COMPRESS,
 	PDO_MYSQL_ATTR_DIRECT_QUERY,
 	PDO_MYSQL_ATTR_FOUND_ROWS,
-	PDO_MYSQL_ATTR_IGNORE_SPACE
+	PDO_MYSQL_ATTR_IGNORE_SPACE,
+	PDO_MYSQL_ATTR_SSL_KEY,
+	PDO_MYSQL_ATTR_SSL_CERT,
+	PDO_MYSQL_ATTR_SSL_CA,
+	PDO_MYSQL_ATTR_SSL_CAPATH,
+	PDO_MYSQL_ATTR_SSL_CIPHER
 };
 
 #endif
