@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | phar php single-file executable PHP extension                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2013 The PHP Group                                |
+  | Copyright (c) 2006-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -485,9 +485,9 @@ typedef struct _phar_entry_data {
 	off_t                    position;
 	/* for copies of the phar fp, defines where 0 is */
 	off_t                    zero;
-	int                      for_write:1;
-	int                      is_zip:1;
-	int                      is_tar:1;
+	unsigned int             for_write:1;
+	unsigned int             is_zip:1;
+	unsigned int             is_tar:1;
 	phar_entry_info          *internal_file;
 } phar_entry_data;
 
