@@ -3,7 +3,7 @@
   | phar php single-file executable PHP extension                        |
   | utility functions                                                    |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2005-2011 The PHP Group                                |
+  | Copyright (c) 2005-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: util.c 306941 2011-01-01 02:48:19Z felipe $ */
+/* $Id$ */
 
 #include "phar_internal.h"
 #ifdef PHAR_HASH_OK
@@ -154,6 +154,8 @@ int phar_seek_efp(phar_entry_info *entry, off_t offset, int whence, off_t positi
 		case SEEK_SET:
 			temp = eoffset + offset;
 			break;
+		default:
+			temp = 0;
 	}
 
 	if (temp > eoffset + (off_t) entry->uncompressed_filesize) {
@@ -208,8 +210,6 @@ int phar_mount_entry(phar_archive_data *phar, char *filename, int filename_len,
 		return FAILURE;
 	}
 #endif
-
-	filename_len = strlen(entry.tmp);
 	filename = entry.tmp;
 
 	/* only check openbasedir for files, not for phar streams */
@@ -271,7 +271,7 @@ char *phar_find_in_include_path(char *filename, int filename_len, phar_archive_d
 		return phar_save_resolve_path(filename, filename_len TSRMLS_CC);
 	}
 
-	fname = zend_get_executed_filename(TSRMLS_C);
+	fname = (char*)zend_get_executed_filename(TSRMLS_C);
 	fname_len = strlen(fname);
 
 	if (PHAR_G(last_phar) && !memcmp(fname, "phar://", 7) && fname_len - 7 >= PHAR_G(last_phar_name_len) && !memcmp(fname + 7, PHAR_G(last_phar_name), PHAR_G(last_phar_name_len))) {
@@ -363,7 +363,7 @@ splitted:
 		goto doit;
 	}
 
-	fname = zend_get_executed_filename(TSRMLS_C);
+	fname = (char*)zend_get_executed_filename(TSRMLS_C);
 
 	if (SUCCESS != phar_split_fname(fname, strlen(fname), &arch, &arch_len, &entry, &entry_len, 1, 0 TSRMLS_CC)) {
 		goto doit;
@@ -521,7 +521,7 @@ not_stream:
 
 	/* check in calling scripts' current working directory as a fall back case */
 	if (zend_is_executing(TSRMLS_C)) {
-		char *exec_fname = zend_get_executed_filename(TSRMLS_C);
+		char *exec_fname = (char*)zend_get_executed_filename(TSRMLS_C);
 		int exec_fname_length = strlen(exec_fname);
 		const char *p;
 		int n = 0;
@@ -997,10 +997,12 @@ int phar_open_entry_fp(phar_entry_info *entry, char **error, int follow_links TS
 	php_stream_filter_append(&ufp->writefilters, filter);
 	php_stream_seek(phar_get_entrypfp(entry TSRMLS_CC), phar_get_fp_offset(entry TSRMLS_CC), SEEK_SET);
 
-	if (SUCCESS != phar_stream_copy_to_stream(phar_get_entrypfp(entry TSRMLS_CC), ufp, entry->compressed_filesize, NULL)) {
-		spprintf(error, 4096, "phar error: internal corruption of phar \"%s\" (actual filesize mismatch on file \"%s\")", phar->fname, entry->filename);
-		php_stream_filter_remove(filter, 1 TSRMLS_CC);
-		return FAILURE;
+	if (entry->uncompressed_filesize) {
+		if (SUCCESS != phar_stream_copy_to_stream(phar_get_entrypfp(entry TSRMLS_CC), ufp, entry->compressed_filesize, NULL)) {
+			spprintf(error, 4096, "phar error: internal corruption of phar \"%s\" (actual filesize mismatch on file \"%s\")", phar->fname, entry->filename);
+			php_stream_filter_remove(filter, 1 TSRMLS_CC);
+			return FAILURE;
+		}
 	}
 
 	php_stream_filter_flush(filter, 1);
@@ -1203,7 +1205,7 @@ int phar_get_archive(phar_archive_data **archive, char *fname, int fname_len, ch
 	phar_archive_data *fd, **fd_ptr;
 	char *my_realpath, *save;
 	int save_len;
-	ulong fhash, ahash;
+	ulong fhash, ahash = 0;
 
 	phar_request_initialize(TSRMLS_C);
 
@@ -2213,7 +2215,7 @@ int phar_create_signature(phar_archive_data *phar, php_stream *fp, char **signat
 
 void phar_add_virtual_dirs(phar_archive_data *phar, char *filename, int filename_len TSRMLS_DC) /* {{{ */
 {
-	char *s;
+	const char *s;
 
 	while ((s = zend_memrchr(filename, '/', filename_len))) {
 		filename_len = s - filename;
