@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_http.c 293491 2010-01-13 07:26:08Z fmk $ */
+/* $Id$ */
 
 #include "php_soap.h"
 #include "ext/standard/base64.h"
@@ -31,32 +31,8 @@ static int get_http_headers(php_stream *socketd,char **response, int *out_size T
 #define smart_str_append_const(str, const) \
 	smart_str_appendl(str,const,sizeof(const)-1)
 
-static int stream_alive(php_stream *stream  TSRMLS_DC)
-{
-	int socket;
-	char buf;
-
-	/* maybe better to use:
-	 * php_stream_set_option(stream, PHP_STREAM_OPTION_CHECK_LIVENESS, 0, NULL)
-	 * here instead */
-
-	if (stream == NULL || stream->eof || php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT, (void**)&socket, 0) != SUCCESS) {
-		return FALSE;
-	}
-	if (socket == -1) {
-		return FALSE;
-	} else {
-		if (php_pollfd_for_ms(socket, PHP_POLLREADABLE, 0) > 0) {
-			if (0 == recv(socket, &buf, sizeof(buf), MSG_PEEK) && php_socket_errno() != EAGAIN) {
-				return FALSE;
-			}
-		}
-	}
-	return TRUE;
-}
-
 /* Proxy HTTP Authentication */
-void proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
+int proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 {
 	zval **login, **password;
 
@@ -77,11 +53,13 @@ void proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 		smart_str_append_const(soap_headers, "\r\n");
 		efree(buf);
 		smart_str_free(&auth);
+		return 1;
 	}
+	return 0;
 }
 
 /* HTTP Authentication */
-void basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
+int basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 {
 	zval **login, **password;
 
@@ -103,15 +81,86 @@ void basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC)
 		smart_str_append_const(soap_headers, "\r\n");
 		efree(buf);
 		smart_str_free(&auth);
+		return 1;
+	}
+	return 0;
+}
+
+/* Additional HTTP headers */
+void http_context_headers(php_stream_context* context,
+                          zend_bool has_authorization,
+                          zend_bool has_proxy_authorization,
+                          zend_bool has_cookies,
+                          smart_str* soap_headers TSRMLS_DC)
+{
+	zval **tmp;
+
+	if (context &&
+		php_stream_context_get_option(context, "http", "header", &tmp) == SUCCESS &&
+		Z_TYPE_PP(tmp) == IS_STRING && Z_STRLEN_PP(tmp)) {
+		char *s = Z_STRVAL_PP(tmp);
+		char *p;
+		int name_len;
+
+		while (*s) {
+			/* skip leading newlines and spaces */
+			while (*s == ' ' || *s == '\t' || *s == '\r' || *s == '\n') {
+				s++;
+			}
+			/* extract header name */
+			p = s;
+			name_len = -1;
+			while (*p) {
+				if (*p == ':') {
+					if (name_len < 0) name_len = p - s;
+					break;
+				} else if (*p == ' ' || *p == '\t') {
+					if (name_len < 0) name_len = p - s;
+				} else if (*p == '\r' || *p == '\n') {
+					break;
+				}
+				p++;
+			}
+			if (*p == ':') {
+				/* extract header value */
+				while (*p && *p != '\r' && *p != '\n') {
+					p++;
+				}
+				/* skip some predefined headers */
+				if ((name_len != sizeof("host")-1 ||
+				     strncasecmp(s, "host", sizeof("host")-1) != 0) &&
+				    (name_len != sizeof("connection")-1 ||
+				     strncasecmp(s, "connection", sizeof("connection")-1) != 0) &&
+				    (name_len != sizeof("user-agent")-1 ||
+				     strncasecmp(s, "user-agent", sizeof("user-agent")-1) != 0) &&
+				    (name_len != sizeof("content-length")-1 ||
+				     strncasecmp(s, "content-length", sizeof("content-length")-1) != 0) &&
+				    (name_len != sizeof("content-type")-1 ||
+				     strncasecmp(s, "content-type", sizeof("content-type")-1) != 0) &&
+				    (!has_cookies ||
+				     name_len != sizeof("cookie")-1 ||
+				     strncasecmp(s, "cookie", sizeof("cookie")-1) != 0) &&
+				    (!has_authorization ||
+				     name_len != sizeof("authorization")-1 ||
+				     strncasecmp(s, "authorization", sizeof("authorization")-1) != 0) &&
+				    (!has_proxy_authorization ||
+				     name_len != sizeof("proxy-authorization")-1 ||
+				     strncasecmp(s, "proxy-authorization", sizeof("proxy-authorization")-1) != 0)) {
+				    /* add header */
+					smart_str_appendl(soap_headers, s, p-s);
+					smart_str_append_const(soap_headers, "\r\n");
+				}
+			}
+			s = (*p) ? (p + 1) : p;
+		}
 	}
 }
 
-static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, int *use_proxy TSRMLS_DC)
+static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, php_stream_context *context, int *use_proxy TSRMLS_DC)
 {
 	php_stream *stream;
 	zval **proxy_host, **proxy_port, **tmp;
 	char *host;
-	php_stream_context *context = NULL;
 	char *name;
 	long namelen;
 	int port;
@@ -140,15 +189,10 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, in
 	old_error_reporting = EG(error_reporting);
 	EG(error_reporting) &= ~(E_WARNING|E_NOTICE|E_USER_WARNING|E_USER_NOTICE);
 
-	if (SUCCESS == zend_hash_find(Z_OBJPROP_P(this_ptr),
-			"_stream_context", sizeof("_stream_context"), (void**)&tmp)) {
-		context = php_stream_context_from_zval(*tmp, 0);
-	}
-
 	namelen = spprintf(&name, 0, "%s://%s:%d", (use_ssl && !*use_proxy)? "ssl" : "tcp", host, port);
 
 	stream = php_stream_xport_create(name, namelen,
-		ENFORCE_SAFE_MODE | REPORT_ERRORS,
+		REPORT_ERRORS,
 		STREAM_XPORT_CLIENT | STREAM_XPORT_CONNECT,
 		NULL /*persistent_id*/,
 		timeout,
@@ -167,6 +211,13 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, in
 		smart_str_appendc(&soap_headers, ':');
 		smart_str_append_unsigned(&soap_headers, phpurl->port);
 		smart_str_append_const(&soap_headers, " HTTP/1.1\r\n");
+		smart_str_append_const(&soap_headers, "Host: ");
+		smart_str_appends(&soap_headers, phpurl->host);
+		if (phpurl->port != 80) {
+			smart_str_appendc(&soap_headers, ':');
+			smart_str_append_unsigned(&soap_headers, phpurl->port);
+		}
+		smart_str_append_const(&soap_headers, "\r\n");
 		proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
 		smart_str_append_const(&soap_headers, "\r\n");
 		if (php_stream_write(stream, soap_headers.c, soap_headers.len) != soap_headers.len) {
@@ -242,6 +293,9 @@ int make_http_soap_request(zval  *this_ptr,
 	char *http_msg = NULL;
 	zend_bool old_allow_url_fopen;
 	php_stream_context *context = NULL;
+	zend_bool has_authorization = 0;
+	zend_bool has_proxy_authorization = 0;
+	zend_bool has_cookies = 0;
 
 	if (this_ptr == NULL || Z_TYPE_P(this_ptr) != IS_OBJECT) {
 		return FALSE;
@@ -282,7 +336,7 @@ int make_http_soap_request(zval  *this_ptr,
 	      n = 3;
 				ZVAL_STRING(&func, "gzencode", 0);
 				smart_str_append_const(&soap_headers_z,"Content-Encoding: gzip\r\n");
-				ZVAL_LONG(params[2], 1);
+				ZVAL_LONG(params[2], 0x1f);
 	    }
 			if (call_user_function(CG(function_table), (zval**)NULL, &func, &retval, n, params TSRMLS_CC) == SUCCESS &&
 			    Z_TYPE(retval) == IS_STRING) {
@@ -379,7 +433,7 @@ try_again:
 	}
 
 	/* Check if keep-alive connection is still opened */
-	if (stream != NULL && !stream_alive(stream TSRMLS_CC)) {
+	if (stream != NULL && php_stream_eof(stream)) {
 		php_stream_close(stream);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpurl", sizeof("httpurl"));
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
@@ -389,7 +443,7 @@ try_again:
 	}
 
 	if (!stream) {
-		stream = http_connect(this_ptr, phpurl, use_ssl, &use_proxy TSRMLS_CC);
+		stream = http_connect(this_ptr, phpurl, use_ssl, context, &use_proxy TSRMLS_CC);
 		if (stream) {
 			php_stream_auto_cleanup(stream);
 			add_property_resource(this_ptr, "httpsocket", php_stream_get_resource_id(stream));
@@ -407,11 +461,20 @@ try_again:
 
 	if (stream) {
 		zval **cookies, **login, **password;
-	  int ret = zend_list_insert(phpurl, le_url);
+	  int ret = zend_list_insert(phpurl, le_url TSRMLS_CC);
 
 		add_property_resource(this_ptr, "httpurl", ret);
 		/*zend_list_addref(ret);*/
 
+		if (context && 
+		    php_stream_context_get_option(context, "http", "protocol_version", &tmp) == SUCCESS &&
+		    Z_TYPE_PP(tmp) == IS_DOUBLE &&
+		    Z_DVAL_PP(tmp) == 1.0) {
+			http_1_1 = 0;
+		} else {
+			http_1_1 = 1;
+		}
+
 		smart_str_append_const(&soap_headers, "POST ");
 		if (use_proxy && !use_ssl) {
 			smart_str_appends(&soap_headers, phpurl->scheme);
@@ -433,19 +496,26 @@ try_again:
 			smart_str_appendc(&soap_headers, '#');
 			smart_str_appends(&soap_headers, phpurl->fragment);
 		}
-		smart_str_append_const(&soap_headers, " HTTP/1.1\r\n"
-			"Host: ");
+		if (http_1_1) {
+			smart_str_append_const(&soap_headers, " HTTP/1.1\r\n");
+		} else {
+			smart_str_append_const(&soap_headers, " HTTP/1.0\r\n");
+		}
+		smart_str_append_const(&soap_headers, "Host: ");
 		smart_str_appends(&soap_headers, phpurl->host);
 		if (phpurl->port != (use_ssl?443:80)) {
 			smart_str_appendc(&soap_headers, ':');
 			smart_str_append_unsigned(&soap_headers, phpurl->port);
 		}
-		smart_str_append_const(&soap_headers, "\r\n"
-			"Connection: Keep-Alive\r\n");
-/*
-			"Connection: close\r\n"
-			"Accept: text/html; text/xml; text/plain\r\n"
-*/
+		if (!http_1_1 ||
+			(zend_hash_find(Z_OBJPROP_P(this_ptr), "_keep_alive", sizeof("_keep_alive"), (void **)&tmp) == SUCCESS &&
+			 Z_LVAL_PP(tmp) == 0)) {
+			smart_str_append_const(&soap_headers, "\r\n"
+				"Connection: close\r\n");
+		} else {
+			smart_str_append_const(&soap_headers, "\r\n"
+				"Connection: Keep-Alive\r\n");
+		}
 		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_user_agent", sizeof("_user_agent"), (void **)&tmp) == SUCCESS &&
 		    Z_TYPE_PP(tmp) == IS_STRING) {
 			if (Z_STRLEN_PP(tmp) > 0) {
@@ -453,6 +523,14 @@ try_again:
 				smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 				smart_str_append_const(&soap_headers, "\r\n");
 			}
+		} else if (context && 
+		           php_stream_context_get_option(context, "http", "user_agent", &tmp) == SUCCESS &&
+		           Z_TYPE_PP(tmp) == IS_STRING) {
+			if (Z_STRLEN_PP(tmp) > 0) {
+				smart_str_append_const(&soap_headers, "User-Agent: ");
+				smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+				smart_str_append_const(&soap_headers, "\r\n");
+			}
 		} else if (FG(user_agent)) {
 			smart_str_append_const(&soap_headers, "User-Agent: ");
 			smart_str_appends(&soap_headers, FG(user_agent));
@@ -488,6 +566,7 @@ try_again:
 		    Z_TYPE_PP(login) == IS_STRING) {
 			zval **digest;
 
+			has_authorization = 1;
 			if (zend_hash_find(Z_OBJPROP_P(this_ptr), "_digest", sizeof("_digest"), (void **)&digest) == SUCCESS) {
 				if (Z_TYPE_PP(digest) == IS_ARRAY) {
 					char          HA1[33], HA2[33], response[33], cnonce[33], nc[9];
@@ -659,7 +738,7 @@ try_again:
 
 		/* Proxy HTTP Authentication */
 		if (use_proxy && !use_ssl) {
-			proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
+			has_proxy_authorization = proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
 		}
 
 		/* Send cookies along with request */
@@ -668,6 +747,7 @@ try_again:
 			char *key;
 			int i, n;
 
+			has_cookies = 1;
 			n = zend_hash_num_elements(Z_ARRVAL_PP(cookies));
 			if (n > 0) {
 				zend_hash_internal_pointer_reset(Z_ARRVAL_PP(cookies));
@@ -699,6 +779,9 @@ try_again:
 				smart_str_append_const(&soap_headers, "\r\n");
 			}
 		}
+
+		http_context_headers(context, has_authorization, has_proxy_authorization, has_cookies, &soap_headers TSRMLS_CC);
+
 		smart_str_append_const(&soap_headers, "\r\n");
 		smart_str_0(&soap_headers);
 		if (zend_hash_find(Z_OBJPROP_P(this_ptr), "trace", sizeof("trace"), (void **) &trace) == SUCCESS &&
@@ -866,6 +949,7 @@ try_again:
 		efree(cookie);
 	}
 
+	/* See if the server requested a close */
 	if (http_1_1) {
 		http_close = FALSE;
 		if (use_proxy && !use_ssl) {
@@ -877,8 +961,35 @@ try_again:
 				efree(connection);
 			}
 		}
+		if (http_close == FALSE) {
+			connection = get_http_header_value(http_headers,"Connection: ");
+			if (connection) {
+				if (strncasecmp(connection, "close", sizeof("close")-1) == 0) {
+					http_close = TRUE;
+				}
+				efree(connection);
+			}
+		}
 	} else {
 		http_close = TRUE;
+		if (use_proxy && !use_ssl) {
+			connection = get_http_header_value(http_headers,"Proxy-Connection: ");
+			if (connection) {
+				if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
+					http_close = FALSE;
+				}
+				efree(connection);
+			}
+		}
+		if (http_close == TRUE) {
+			connection = get_http_header_value(http_headers,"Connection: ");
+			if (connection) {
+				if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
+					http_close = FALSE;
+				}
+				efree(connection);
+			}
+		}
 	}	
 
 	if (!get_http_body(stream, http_close, http_headers, &http_body, &http_body_size TSRMLS_CC)) {
@@ -897,31 +1008,6 @@ try_again:
 
 	if (request != buf) {efree(request);}
 
-	/* See if the server requested a close */
-	http_close = TRUE;
-	connection = get_http_header_value(http_headers,"Proxy-Connection: ");
-	if (connection) {
-		if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
-			http_close = FALSE;
-		}
-		efree(connection);
-/*
-	} else if (http_1_1) {
-		http_close = FALSE;
-*/
-	}
-	connection = get_http_header_value(http_headers,"Connection: ");
-	if (connection) {
-		if (strncasecmp(connection, "Keep-Alive", sizeof("Keep-Alive")-1) == 0) {
-			http_close = FALSE;
-		}
-		efree(connection);
-/*
-	} else if (http_1_1) {
-		http_close = FALSE;
-*/
-	}
-
 	if (http_close) {
 		php_stream_close(stream);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
@@ -1031,7 +1117,7 @@ try_again:
 			if (digest != NULL) {
 				php_url *new_url  = emalloc(sizeof(php_url));
 
-				digest->refcount--;
+				Z_DELREF_P(digest);
 				add_property_zval_ex(this_ptr, "_digest", sizeof("_digest"), digest TSRMLS_CC);
 
 				*new_url = *phpurl;
@@ -1242,15 +1328,15 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 	}
 
 	if (header_chunked) {
-		char ch, done, chunk_size[10], headerbuf[8192];
+		char ch, done, headerbuf[8192];
 
 		done = FALSE;
 
 		while (!done) {
 			int buf_size = 0;
 
-			php_stream_gets(stream, chunk_size, sizeof(chunk_size));
-			if (sscanf(chunk_size, "%x", &buf_size) > 0 ) {
+			php_stream_gets(stream, headerbuf, sizeof(headerbuf));
+			if (sscanf(headerbuf, "%x", &buf_size) > 0 ) {
 				if (buf_size > 0) {
 					int len_size = 0;
 
@@ -1314,10 +1400,10 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 		}
 
 	} else if (header_length) {
-		if (header_length < 0) {
+		if (header_length < 0 || header_length >= INT_MAX) {
 			return FALSE;
 		}
-		http_buf = emalloc(header_length + 1);
+		http_buf = safe_emalloc(1, header_length, 1);
 		while (http_buf_size < header_length) {
 			int len_read = php_stream_read(stream, http_buf + http_buf_size, header_length - http_buf_size);
 			if (len_read <= 0) {
@@ -1370,3 +1456,11 @@ static int get_http_headers(php_stream *stream, char **response, int *out_size T
 	(*out_size) = tmp_response.len;
 	return done;
 }
+/*
+ * Local variables:
+ * tab-width: 4
+ * c-basic-offset: 4
+ * End:
+ * vim600: sw=4 ts=4 fdm=marker
+ * vim<600: sw=4 ts=4
+ */
