@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_packet_soap.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #include "php_soap.h"
 
@@ -191,21 +191,21 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 
 			tmp = get_node(fault->children, "faultstring");
 			if (tmp != NULL && tmp->children != NULL) {
-				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp TSRMLS_CC);
 				faultstring = Z_STRVAL_P(zv);
 				FREE_ZVAL(zv);
 			}
 
 			tmp = get_node(fault->children, "faultactor");
 			if (tmp != NULL && tmp->children != NULL) {
-				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+				zval *zv = master_to_zval(get_conversion(IS_STRING), tmp TSRMLS_CC);
 				faultactor = Z_STRVAL_P(zv);
 				FREE_ZVAL(zv);
 			}
 
 			tmp = get_node(fault->children, "detail");
 			if (tmp != NULL) {
-				details = master_to_zval(NULL, tmp);
+				details = master_to_zval(NULL, tmp TSRMLS_CC);
 			}
 		} else {
 			tmp = get_node(fault->children, "Code");
@@ -221,7 +221,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				/* TODO: lang attribute */
 				tmp = get_node(tmp->children,"Text");
 				if (tmp != NULL && tmp->children != NULL) {
-					zval *zv = master_to_zval(get_conversion(IS_STRING), tmp);
+					zval *zv = master_to_zval(get_conversion(IS_STRING), tmp TSRMLS_CC);
 					faultstring = Z_STRVAL_P(zv);
 					FREE_ZVAL(zv);
 				}
@@ -229,7 +229,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 
 			tmp = get_node(fault->children,"Detail");
 			if (tmp != NULL) {
-				details = master_to_zval(NULL, tmp);
+				details = master_to_zval(NULL, tmp TSRMLS_CC);
 			}
 		}
 		add_soap_fault(this_ptr, faultcode, faultstring, faultactor, details TSRMLS_CC);
@@ -239,11 +239,9 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 		if (faultactor) {
 			efree(faultactor);
 		}
-#ifdef ZEND_ENGINE_2
 		if (details) {
-			details->refcount--;
+			Z_DELREF_P(details);
 		}
-#endif
 		xmlFreeDoc(response);
 		return FALSE;
 	}
@@ -327,9 +325,9 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 					} else {
 						/* Decoding value of parameter */
 						if (param != NULL) {
-							tmp = master_to_zval(param->encode, val);
+							tmp = master_to_zval(param->encode, val TSRMLS_CC);
 						} else {
-							tmp = master_to_zval(NULL, val);
+							tmp = master_to_zval(NULL, val TSRMLS_CC);
 						}
 					}
 					add_assoc_zval(return_value, param->paramName, tmp);
@@ -340,7 +338,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 				}
 			}
 		} else {
-		  /* Function hasn't WSDL description */
+		  /* Function has no WSDL description */
 			xmlNodePtr val;
 			val = resp->children;
 			while (val != NULL) {
@@ -352,7 +350,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 						zval *tmp;
 						zval **arr;
 
-						tmp = master_to_zval(NULL, val);
+						tmp = master_to_zval(NULL, val TSRMLS_CC);
 						if (val->name) {
 							if (zend_hash_find(Z_ARRVAL_P(return_value), (char*)val->name, strlen((char*)val->name)+1, (void**)&arr) == SUCCESS) {
 								add_next_index_zval(*arr, tmp);
@@ -387,7 +385,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 			zend_hash_internal_pointer_reset(Z_ARRVAL_P(return_value));
 			zend_hash_get_current_data(Z_ARRVAL_P(return_value), (void**)&tmp);
 			tmp = *(zval**)tmp;
-			tmp->refcount++;
+			Z_ADDREF_P(tmp);
 			zval_dtor(return_value);
 			*return_value = *tmp;
 			FREE_ZVAL(tmp);
@@ -416,7 +414,7 @@ int parse_packet_soap(zval *this_ptr, char *buffer, int buffer_size, sdlFunction
 					}
 					smart_str_free(&key);
 				}
-				val = master_to_zval(enc, trav);
+				val = master_to_zval(enc, trav TSRMLS_CC);
 				add_assoc_zval(soap_headers, (char*)trav->name, val);
 			}
 			trav = trav->next;
