@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,7 +22,7 @@
 #ifndef PHP_SOCKETS_H
 #define PHP_SOCKETS_H
 
-/* $Id: php_sockets.h 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #if HAVE_SOCKETS
 
@@ -30,10 +30,8 @@ extern zend_module_entry sockets_module_entry;
 #define phpext_sockets_ptr &sockets_module_entry
 
 #ifdef PHP_WIN32
-#define PHP_SOCKETS_API __declspec(dllexport)
 #include <winsock.h>
 #else
-#define PHP_SOCKETS_API
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -72,20 +70,35 @@ PHP_FUNCTION(socket_shutdown);
 #endif
 PHP_FUNCTION(socket_last_error);
 PHP_FUNCTION(socket_clear_error);
+PHP_FUNCTION(socket_import_stream);
 
 #ifndef PHP_WIN32
 typedef int PHP_SOCKET;
+# define PHP_SOCKETS_API PHPAPI
 #else
+# define PHP_SOCKETS_API __declspec(dllexport)
 typedef SOCKET PHP_SOCKET;
 #endif
 
 typedef struct {
-	PHP_SOCKET bsd_socket;
-	int		type;
-	int		error;
-	int		blocking;
+	PHP_SOCKET	bsd_socket;
+	int			type;
+	int			error;
+	int			blocking;
+	zval		*zstream;
 } php_socket;
 
+#ifdef PHP_WIN32
+struct	sockaddr_un {
+	short	sun_family;
+	char	sun_path[108];
+};
+#endif
+
+PHP_SOCKETS_API int php_sockets_le_socket(void);
+
+#define php_sockets_le_socket_name "Socket"
+
 /* Prototypes */
 #ifdef ilia_0 /* not needed, only causes a compiler warning */
 static int php_open_listen_sock(php_socket **php_sock, int port, int backlog TSRMLS_DC);
