@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -282,7 +282,7 @@ PHP_FUNCTION(convert_cyr_string)
 	str = (unsigned char*) estrndup(input, input_len);
 
 	php_convert_cyr_string(str, input_len, fr_cs[0], to_cs[0] TSRMLS_CC);
-	RETVAL_STRING((char *)str, 0)
+	RETVAL_STRING((char *)str, 0);
 }
 /* }}} */
 
