@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -38,30 +38,19 @@
 #include <sys/utsname.h>
 #endif
 
+
 #ifdef PHP_WIN32
 typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
 typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);
-
 # include "winver.h"
 
-#if _MSC_VER < 1300
-# define OSVERSIONINFOEX php_win_OSVERSIONINFOEX
-#endif
-
-#endif
-
-#if HAVE_MBSTRING
-#include "ext/mbstring/mbstring.h"
-ZEND_EXTERN_MODULE_GLOBALS(mbstring)
-#endif
-
-#if HAVE_ICONV
-#include "ext/iconv/php_iconv.h"
-ZEND_EXTERN_MODULE_GLOBALS(iconv)
+# if _MSC_VER < 1300
+#  define OSVERSIONINFOEX php_win_OSVERSIONINFOEX
+# endif
 #endif
 
 #define SECTION(name)	if (!sapi_module.phpinfo_as_text) { \
-							PUTS("<h2>" name "</h2>\n"); \
+							php_info_print("<h2>" name "</h2>\n"); \
 						} else { \
 							php_info_print_table_start(); \
 							php_info_print_table_header(1, name); \
@@ -71,29 +60,99 @@ ZEND_EXTERN_MODULE_GLOBALS(iconv)
 PHPAPI extern char *php_ini_opened_path;
 PHPAPI extern char *php_ini_scanned_path;
 PHPAPI extern char *php_ini_scanned_files;
-	
-static int php_info_write_wrapper(const char *str, uint str_length)
+
+static int php_info_print_html_esc(const char *str, int len) /* {{{ */
 {
-	int new_len, written;
-	char *elem_esc;
-
+	size_t new_len;
+	int written;
+	char *new_str;
 	TSRMLS_FETCH();
-
-	elem_esc = php_escape_html_entities((unsigned char *)str, str_length, &new_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
-
-	written = php_body_write(elem_esc, new_len TSRMLS_CC);
-
-	efree(elem_esc);
-
+	
+	new_str = php_escape_html_entities((unsigned char *) str, len, &new_len, 0, ENT_QUOTES, "utf-8" TSRMLS_CC);
+	written = php_output_write(new_str, new_len TSRMLS_CC);
+	efree(new_str);
 	return written;
 }
+/* }}} */
 
+static int php_info_printf(const char *fmt, ...) /* {{{ */
+{
+	char *buf;
+	int len, written;
+	va_list argv;
+	TSRMLS_FETCH();
+	
+	va_start(argv, fmt);
+	len = vspprintf(&buf, 0, fmt, argv);
+	va_end(argv);
+	
+	written = php_output_write(buf, len TSRMLS_CC);
+	efree(buf);
+	return written;
+}
+/* }}} */
+
+static void php_info_print_request_uri(TSRMLS_D) /* {{{ */
+{
+	if (SG(request_info).request_uri) {
+		php_info_print_html_esc(SG(request_info).request_uri, strlen(SG(request_info).request_uri));
+	}
+}
+/* }}} */
+
+static int php_info_print(const char *str) /* {{{ */
+{
+	TSRMLS_FETCH();
+	return php_output_write(str, strlen(str) TSRMLS_CC);
+}
+/* }}} */
+
+static void php_info_print_stream_hash(const char *name, HashTable *ht TSRMLS_DC) /* {{{ */
+{
+	char *key;
+	uint len;
+	
+	if (ht) {
+		if (zend_hash_num_elements(ht)) {
+			HashPosition pos;
+
+			if (!sapi_module.phpinfo_as_text) {
+				php_info_printf("<tr><td class=\"e\">Registered %s</td><td class=\"v\">", name);
+			} else {
+				php_info_printf("\nRegistered %s => ", name);
+			}
+			
+			zend_hash_internal_pointer_reset_ex(ht, &pos);
+			while (zend_hash_get_current_key_ex(ht, &key, &len, NULL, 0, &pos) == HASH_KEY_IS_STRING)
+			{
+				php_info_print(key);
+				zend_hash_move_forward_ex(ht, &pos);
+				if (zend_hash_get_current_key_ex(ht, &key, &len, NULL, 0, &pos) == HASH_KEY_IS_STRING) {
+					php_info_print(", ");
+				} else {
+					break;
+				}
+			}
+			
+			if (!sapi_module.phpinfo_as_text) {
+				php_info_print("</td></tr>\n");
+			}
+		} else {
+			char reg_name[128];
+			snprintf(reg_name, sizeof(reg_name), "Registered %s", name);
+			php_info_print_table_row(2, reg_name, "none registered");
+		}
+	} else {
+		php_info_print_table_row(2, name, "disabled");
+	}
+}
+/* }}} */
 
 PHPAPI void php_info_print_module(zend_module_entry *zend_module TSRMLS_DC) /* {{{ */
 {
 	if (zend_module->info_func || zend_module->version) {
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<h2><a name=\"module_%s\">%s</a></h2>\n", zend_module->name, zend_module->name);
+			php_info_printf("<h2><a name=\"module_%s\">%s</a></h2>\n", zend_module->name, zend_module->name);
 		} else {
 			php_info_print_table_start();
 			php_info_print_table_header(1, zend_module->name);
@@ -109,9 +168,9 @@ PHPAPI void php_info_print_module(zend_module_entry *zend_module TSRMLS_DC) /* {
 		}
 	} else {
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<tr><td>%s</td></tr>\n", zend_module->name);
+			php_info_printf("<tr><td>%s</td></tr>\n", zend_module->name);
 		} else {
-			php_printf("%s\n", zend_module->name);
+			php_info_printf("%s\n", zend_module->name);
 		}	
 	}
 }
@@ -151,70 +210,66 @@ static void php_print_gpcse_array(char *name, uint name_length TSRMLS_DC)
 		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(data));
 		while (zend_hash_get_current_data(Z_ARRVAL_PP(data), (void **) &tmp) == SUCCESS) {
 			if (!sapi_module.phpinfo_as_text) {
-				PUTS("<tr>");
-				PUTS("<td class=\"e\">");
-
+				php_info_print("<tr>");
+				php_info_print("<td class=\"e\">");
 			}
 
-			PUTS(name);
-			PUTS("[\"");
+			php_info_print(name);
+			php_info_print("[\"");
 			
 			switch (zend_hash_get_current_key_ex(Z_ARRVAL_PP(data), &string_key, &string_len, &num_key, 0, NULL)) {
 				case HASH_KEY_IS_STRING:
 					if (!sapi_module.phpinfo_as_text) {
-						php_info_html_esc_write(string_key, string_len - 1 TSRMLS_CC);
+						php_info_print_html_esc(string_key, string_len-1);
 					} else {
-						PHPWRITE(string_key, string_len - 1);
-					}	
+						php_info_print(string_key);
+					}
 					break;
 				case HASH_KEY_IS_LONG:
-					php_printf("%ld", num_key);
+					php_info_printf("%ld", num_key);
 					break;
 			}
-			PUTS("\"]");
+			php_info_print("\"]");
 			if (!sapi_module.phpinfo_as_text) {
-				PUTS("</td><td class=\"v\">");
+				php_info_print("</td><td class=\"v\">");
 			} else {
-				PUTS(" => ");
+				php_info_print(" => ");
 			}
 			if (Z_TYPE_PP(tmp) == IS_ARRAY) {
 				if (!sapi_module.phpinfo_as_text) {
-					PUTS("<pre>");
-					zend_print_zval_r_ex((zend_write_func_t) php_info_write_wrapper, *tmp, 0 TSRMLS_CC);
-					PUTS("</pre>");
+					php_info_print("<pre>");
+					zend_print_zval_r_ex((zend_write_func_t) php_info_print_html_esc, *tmp, 0 TSRMLS_CC);
+					php_info_print("</pre>");
 				} else {
 					zend_print_zval_r(*tmp, 0 TSRMLS_CC);
 				}
-			} else if (Z_TYPE_PP(tmp) != IS_STRING) {
+			} else {
 				tmp2 = **tmp;
-				zval_copy_ctor(&tmp2);
-				convert_to_string(&tmp2);
+				if (Z_TYPE_PP(tmp) != IS_STRING) {
+					tmp = NULL;
+					zval_copy_ctor(&tmp2);
+					convert_to_string(&tmp2);
+				}
+
 				if (!sapi_module.phpinfo_as_text) {
 					if (Z_STRLEN(tmp2) == 0) {
-						PUTS("<i>no value</i>");
+						php_info_print("<i>no value</i>");
 					} else {
-						php_info_html_esc_write(Z_STRVAL(tmp2), Z_STRLEN(tmp2) TSRMLS_CC);
-					} 
-				} else {
-					PHPWRITE(Z_STRVAL(tmp2), Z_STRLEN(tmp2));
-				}	
-				zval_dtor(&tmp2);
-			} else {
-				if (!sapi_module.phpinfo_as_text) {
-					if (Z_STRLEN_PP(tmp) == 0) {
-						PUTS("<i>no value</i>");
-					} else {
-						php_info_html_esc_write(Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp) TSRMLS_CC);
+						php_info_print_html_esc(Z_STRVAL(tmp2), Z_STRLEN(tmp2));
 					}
 				} else {
-					PHPWRITE(Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
-				}	
+					php_info_print(Z_STRVAL(tmp2));
+				}
+
+				if (!tmp) {
+					zval_dtor(&tmp2);
+				}
 			}
 			if (!sapi_module.phpinfo_as_text) {
-				PUTS("</td></tr>\n");
+				php_info_print("</td></tr>\n");
 			} else {
-				PUTS("\n");
-			}	
+				php_info_print("\n");
+			}
 			zend_hash_move_forward(Z_ARRVAL_PP(data));
 		}
 	}
@@ -225,21 +280,9 @@ static void php_print_gpcse_array(char *name, uint name_length TSRMLS_DC)
  */
 void php_info_print_style(TSRMLS_D)
 {
-	php_printf("<style type=\"text/css\">\n");
+	php_info_printf("<style type=\"text/css\">\n");
 	php_info_print_css(TSRMLS_C);
-	php_printf("</style>\n");
-}
-/* }}} */
-
-/* {{{ php_info_html_esc_write
- */
-PHPAPI void php_info_html_esc_write(char *string, int str_len TSRMLS_DC)
-{
-	int new_len;
-	char *ret = php_escape_html_entities((unsigned char *)string, str_len, &new_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
-
-	PHPWRITE(ret, new_len);
-	efree(ret);
+	php_info_printf("</style>\n");
 }
 /* }}} */
 
@@ -247,14 +290,14 @@ PHPAPI void php_info_html_esc_write(char *string, int str_len TSRMLS_DC)
  */
 PHPAPI char *php_info_html_esc(char *string TSRMLS_DC)
 {
-	int new_len;
-	return php_escape_html_entities((unsigned char *)string, strlen(string), &new_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
+	size_t new_len;
+	return php_escape_html_entities(string, strlen(string), &new_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
 }
 /* }}} */
 
-
 #ifdef PHP_WIN32
 /* {{{  */
+
 char* php_get_windows_name()
 {
 	OSVERSIONINFOEX osvi;
@@ -281,7 +324,7 @@ char* php_get_windows_name()
 	}
 
 	if (VER_PLATFORM_WIN32_NT==osvi.dwPlatformId && osvi.dwMajorVersion > 4 ) {
-		if (osvi.dwMajorVersion == 6) {
+		if (osvi.dwMajorVersion == 6)	{
 			if( osvi.dwMinorVersion == 0 ) {
 				if( osvi.wProductType == VER_NT_WORKSTATION ) {
 					major = "Windows Vista";
@@ -296,7 +339,7 @@ char* php_get_windows_name()
 					major = "Windows Server 2008 R2";
 				}
 			} else {
-				major = "Unknow Windows version";
+				major = "Unknown Windows version";
 			}
 
 			pGPI = (PGPI) GetProcAddress(GetModuleHandle("kernel32.dll"), "GetProductInfo");
@@ -357,12 +400,12 @@ char* php_get_windows_name()
 			}
 		}
 
-		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )	{
+		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 ) {
 			if (GetSystemMetrics(SM_SERVERR2))
 				major = "Windows Server 2003 R2";
-			else if (osvi.wSuiteMask == VER_SUITE_STORAGE_SERVER)
+			else if (osvi.wSuiteMask==VER_SUITE_STORAGE_SERVER)
 				major = "Windows Storage Server 2003";
-			else if (osvi.wSuiteMask == VER_SUITE_WH_SERVER)
+			else if (osvi.wSuiteMask==VER_SUITE_WH_SERVER)
 				major = "Windows Home Server";
 			else if (osvi.wProductType == VER_NT_WORKSTATION &&
 				si.wProcessorArchitecture==PROCESSOR_ARCHITECTURE_AMD64) {
@@ -402,9 +445,17 @@ char* php_get_windows_name()
 
 		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )	{
 			major = "Windows XP";
-			if( osvi.wSuiteMask & VER_SUITE_PERSONAL )
+			if( osvi.wSuiteMask & VER_SUITE_PERSONAL ) {
 				sub = "Home Edition";
-			else sub = "Professional";
+			} else if (GetSystemMetrics(SM_MEDIACENTER)) {
+				sub = "Media Center Edition";
+			} else if (GetSystemMetrics(SM_STARTER)) {
+				sub = "Starter Edition";
+			} else if (GetSystemMetrics(SM_TABLETPC)) {
+				sub = "Tablet PC Edition";
+			} else {
+				sub = "Professional";
+			}
 		}
 
 		if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 ) {
@@ -486,11 +537,7 @@ PHPAPI char *php_get_uname(char mode)
 	GetComputerName(ComputerName, &dwSize);
 
 	if (mode == 's') {
-		if (dwVersion < 0x80000000) {
-			php_uname = "Windows NT";
-		} else {
-			php_uname = "Windows 9x";
-		}
+		php_uname = "Windows NT";
 	} else if (mode == 'r') {
 		snprintf(tmp_uname, sizeof(tmp_uname), "%d.%d", dwWindowsMajorVersion, dwWindowsMinorVersion);
 		php_uname = tmp_uname;
@@ -512,23 +559,16 @@ PHPAPI char *php_get_uname(char mode)
 		php_get_windows_cpu(tmp_uname, sizeof(tmp_uname));
 		php_uname = tmp_uname;
 	} else { /* assume mode == 'a' */
-		/* Get build numbers for Windows NT or Win95 */
-		if (dwVersion < 0x80000000){
-			char *winver = php_get_windows_name();
-			char wincpu[20];
+		char *winver = php_get_windows_name();
+		char wincpu[20];
 
-			php_get_windows_cpu(wincpu, sizeof(wincpu));
-			dwBuild = (DWORD)(HIWORD(dwVersion));
-			snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d build %d (%s) %s",
-					 "Windows NT", ComputerName,
-					 dwWindowsMajorVersion, dwWindowsMinorVersion, dwBuild, winver?winver:"unknown", wincpu);
-			if(winver) {
-				efree(winver);
-			}
-		} else {
-			snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d",
-					 "Windows 9x", ComputerName,
-					 dwWindowsMajorVersion, dwWindowsMinorVersion);
+		php_get_windows_cpu(wincpu, sizeof(wincpu));
+		dwBuild = (DWORD)(HIWORD(dwVersion));
+		snprintf(tmp_uname, sizeof(tmp_uname), "%s %s %d.%d build %d (%s) %s",
+				 "Windows NT", ComputerName,
+				 dwWindowsMajorVersion, dwWindowsMinorVersion, dwBuild, winver?winver:"unknown", wincpu);
+		if(winver) {
+			efree(winver);
 		}
 		php_uname = tmp_uname;
 	}
@@ -588,54 +628,18 @@ PHPAPI char *php_get_uname(char mode)
 }
 /* }}} */
 
-
 /* {{{ php_print_info_htmlhead
  */
 PHPAPI void php_print_info_htmlhead(TSRMLS_D)
 {
-
-/*** none of this is needed now ***
-
-	const char *charset = NULL;
-
-	if (SG(default_charset)) {
-		charset = SG(default_charset);
-	}
-
-#if HAVE_MBSTRING
-	if (php_ob_handler_used("mb_output_handler" TSRMLS_CC)) {
-		if (MBSTRG(current_http_output_encoding) == mbfl_no_encoding_pass) {
-			charset = "US-ASCII";
-		} else {
-			charset = mbfl_no2preferred_mime_name(MBSTRG(current_http_output_encoding));
-		}
-	}
-#endif   
-
-#if HAVE_ICONV
-	if (php_ob_handler_used("ob_iconv_handler" TSRMLS_CC)) {
-		charset = ICONVG(output_encoding);
-	}
-#endif
-
-	if (!charset || !charset[0]) {
-		charset = "US-ASCII";
-	}
-
-*** none of that is needed now ***/
-
-
-	PUTS("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"DTD/xhtml1-transitional.dtd\">\n");
-	PUTS("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
-	PUTS("<head>\n");
+	php_info_print("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"DTD/xhtml1-transitional.dtd\">\n");
+	php_info_print("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
+	php_info_print("<head>\n");
 	php_info_print_style(TSRMLS_C);
-	PUTS("<title>phpinfo()</title>");
-	PUTS("<meta name=\"ROBOTS\" content=\"NOINDEX,NOFOLLOW,NOARCHIVE\" />");
-/*
-	php_printf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\" />\n", charset);
-*/
-	PUTS("</head>\n");
-	PUTS("<body><div class=\"center\">\n");
+	php_info_print("<title>phpinfo()</title>");
+	php_info_print("<meta name=\"ROBOTS\" content=\"NOINDEX,NOFOLLOW,NOARCHIVE\" />");
+	php_info_print("</head>\n");
+	php_info_print("<body><div class=\"center\">\n");
 }
 /* }}} */
 
@@ -661,8 +665,8 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 	if (!sapi_module.phpinfo_as_text) {
 		php_print_info_htmlhead(TSRMLS_C);
 	} else {
-		PUTS("phpinfo()\n");
-	}	
+		php_info_print("phpinfo()\n");
+	}
 
 	if (flag & PHP_INFO_GENERAL) {
 		char *zend_version = get_zend_version();
@@ -676,21 +680,17 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		}
 
 		if (expose_php && !sapi_module.phpinfo_as_text) {
-			PUTS("<a href=\"http://www.php.net/\"><img border=\"0\" src=\"");
-			if (SG(request_info).request_uri) {
-				char *elem_esc = php_info_html_esc(SG(request_info).request_uri TSRMLS_CC);
-				PUTS(elem_esc);
-				efree(elem_esc);
-			}
-			PUTS("?=");
+			php_info_print("<a href=\"http://www.php.net/\"><img border=\"0\" src=\"");
+			php_info_print_request_uri(TSRMLS_C);
+			php_info_print("?=");
 			logo_guid = php_logo_guid();
-			PUTS(logo_guid);
+			php_info_print(logo_guid);
 			efree(logo_guid);
-			PUTS("\" alt=\"PHP Logo\" /></a>");
+			php_info_print("\" alt=\"PHP Logo\" /></a>");
 		}
 
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<h1 class=\"p\">PHP Version %s</h1>\n", PHP_VERSION);
+			php_info_printf("<h1 class=\"p\">PHP Version %s</h1>\n", PHP_VERSION);
 		} else {
 			php_info_print_table_row(2, "PHP Version", PHP_VERSION);
 		}	
@@ -747,152 +747,55 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		php_info_print_table_row(2, "Thread Safety", "disabled" );
 #endif
 
+#ifdef ZEND_SIGNALS
+		php_info_print_table_row(2, "Zend Signal Handling", "enabled" );
+#else
+		php_info_print_table_row(2, "Zend Signal Handling", "disabled" );
+#endif
+
 		php_info_print_table_row(2, "Zend Memory Manager", is_zend_mm(TSRMLS_C) ? "enabled" : "disabled" );
 
-#ifdef ZEND_MULTIBYTE
-		php_info_print_table_row(2, "Zend Multibyte Support", "enabled");
-#else
-		php_info_print_table_row(2, "Zend Multibyte Support", "disabled");
-#endif
+		{
+			const zend_multibyte_functions *functions = zend_multibyte_get_functions(TSRMLS_C);
+			char *descr;
+			if (functions) {
+				spprintf(&descr, 0, "provided by %s", functions->provider_name);
+			} else {
+				descr = estrdup("disabled");
+			}
+            php_info_print_table_row(2, "Zend Multibyte Support", descr);
+			efree(descr);
+		}
 
 #if HAVE_IPV6
 		php_info_print_table_row(2, "IPv6 Support", "enabled" );
 #else
 		php_info_print_table_row(2, "IPv6 Support", "disabled" );
 #endif
-		{
-			HashTable *url_stream_wrappers_hash;
-			char *stream_protocol, *stream_protocols_buf = NULL;
-			int stream_protocol_len, stream_protocols_buf_len = 0;
-			ulong num_key;
 
-			if ((url_stream_wrappers_hash = php_stream_get_url_stream_wrappers_hash())) {
-				HashPosition pos;
-				for (zend_hash_internal_pointer_reset_ex(url_stream_wrappers_hash, &pos);
-						zend_hash_get_current_key_ex(url_stream_wrappers_hash, &stream_protocol, (uint *)&stream_protocol_len, &num_key, 0, &pos) == HASH_KEY_IS_STRING;
-						zend_hash_move_forward_ex(url_stream_wrappers_hash, &pos)) {
-					stream_protocols_buf = erealloc(stream_protocols_buf, stream_protocols_buf_len + stream_protocol_len + 2 + 1);
-					memcpy(stream_protocols_buf + stream_protocols_buf_len, stream_protocol, stream_protocol_len - 1);
-					stream_protocols_buf[stream_protocols_buf_len + stream_protocol_len - 1] = ',';
-					stream_protocols_buf[stream_protocols_buf_len + stream_protocol_len] = ' ';
-					stream_protocols_buf_len += stream_protocol_len + 1;
-				}
-				if (stream_protocols_buf) {
-					stream_protocols_buf[stream_protocols_buf_len - 2] = ' ';
-					stream_protocols_buf[stream_protocols_buf_len] = 0;
-					php_info_print_table_row(2, "Registered PHP Streams", stream_protocols_buf);
-					efree(stream_protocols_buf);
-				} else {
-					/* Any chances we will ever hit this? */
-					php_info_print_table_row(2, "Registered PHP Streams", "no streams registered");
-				}
-			} else {
-				/* Any chances we will ever hit this? */
-				php_info_print_table_row(2, "PHP Streams", "disabled"); /* ?? */
-			}
-		}
-
-		{
-			HashTable *stream_xport_hash;
-			char *xport_name, *xport_buf = NULL;
-			int xport_name_len, xport_buf_len = 0, xport_buf_size = 0;
-			ulong num_key;
-
-			if ((stream_xport_hash = php_stream_xport_get_hash())) {
-				HashPosition pos;
-				for(zend_hash_internal_pointer_reset_ex(stream_xport_hash, &pos);
-					zend_hash_get_current_key_ex(stream_xport_hash, &xport_name, (uint *)&xport_name_len, &num_key, 0, &pos) == HASH_KEY_IS_STRING;
-					zend_hash_move_forward_ex(stream_xport_hash, &pos)) {
-					if (xport_buf_len + xport_name_len + 2 > xport_buf_size) {
-						while (xport_buf_len + xport_name_len + 2 > xport_buf_size) {
-							xport_buf_size += 256;
-						}
-						if (xport_buf) {
-							xport_buf = erealloc(xport_buf, xport_buf_size);
-						} else {
-							xport_buf = emalloc(xport_buf_size);
-						}
-					}
-					if (xport_buf_len > 0) {
-						xport_buf[xport_buf_len++] = ',';
-						xport_buf[xport_buf_len++] = ' ';
-					}
-					memcpy(xport_buf + xport_buf_len, xport_name, xport_name_len - 1);
-					xport_buf_len += xport_name_len - 1;
-					xport_buf[xport_buf_len] = '\0';
-				}
-				if (xport_buf) {
-					php_info_print_table_row(2, "Registered Stream Socket Transports", xport_buf);
-					efree(xport_buf);
-				} else {
-					/* Any chances we will ever hit this? */
-					php_info_print_table_row(2, "Registered Stream Socket Transports", "no transports registered");
-				}
-			} else {
-				/* Any chances we will ever hit this? */
-				php_info_print_table_row(2, "Stream Socket Transports", "disabled"); /* ?? */
-			}
-		}
-
-		{
-			HashTable *stream_filter_hash;
-			char *filter_name, *filter_buf = NULL;
-			int filter_name_len, filter_buf_len = 0, filter_buf_size = 0;
-			ulong num_key;
-
-			if ((stream_filter_hash = php_get_stream_filters_hash())) {
-				HashPosition pos;
-				for(zend_hash_internal_pointer_reset_ex(stream_filter_hash, &pos);
-					zend_hash_get_current_key_ex(stream_filter_hash, &filter_name, (uint *)&filter_name_len, &num_key, 0, &pos) == HASH_KEY_IS_STRING;
-					zend_hash_move_forward_ex(stream_filter_hash, &pos)) {
-					if (filter_buf_len + filter_name_len + 2 > filter_buf_size) {
-						while (filter_buf_len + filter_name_len + 2 > filter_buf_size) {
-							filter_buf_size += 256;
-						}
-						if (filter_buf) {
-							filter_buf = erealloc(filter_buf, filter_buf_size);
-						} else {
-							filter_buf = emalloc(filter_buf_size);
-						}
-					}
-					if (filter_buf_len > 0) {
-						filter_buf[filter_buf_len++] = ',';
-						filter_buf[filter_buf_len++] = ' ';
-					}
-					memcpy(filter_buf + filter_buf_len, filter_name, filter_name_len - 1);
-					filter_buf_len += filter_name_len - 1;
-					filter_buf[filter_buf_len] = '\0';
-				}
-				if (filter_buf) {
-					php_info_print_table_row(2, "Registered Stream Filters", filter_buf);
-					efree(filter_buf);
-				} else {
-					/* Any chances we will ever hit this? */
-					php_info_print_table_row(2, "Registered Stream Filters", "no filters registered");
-				}
-			} else {
-				/* Any chances we will ever hit this? */
-				php_info_print_table_row(2, "Stream Filters", "disabled"); /* ?? */
-			}
-		}
+#if HAVE_DTRACE
+		php_info_print_table_row(2, "DTrace Support", "enabled" );
+#else
+		php_info_print_table_row(2, "DTrace Support", "disabled" );
+#endif
 		
+		php_info_print_stream_hash("PHP Streams",  php_stream_get_url_stream_wrappers_hash() TSRMLS_CC);
+		php_info_print_stream_hash("Stream Socket Transports", php_stream_xport_get_hash() TSRMLS_CC);
+		php_info_print_stream_hash("Stream Filters", php_get_stream_filters_hash() TSRMLS_CC);
+
 		php_info_print_table_end();
 
 		/* Zend Engine */
 		php_info_print_box_start(0);
 		if (expose_php && !sapi_module.phpinfo_as_text) {
-			PUTS("<a href=\"http://www.zend.com/\"><img border=\"0\" src=\"");
-			if (SG(request_info).request_uri) {
-				char *elem_esc = php_info_html_esc(SG(request_info).request_uri TSRMLS_CC);
-				PUTS(elem_esc);
-				efree(elem_esc);
-			}
-			PUTS("?="ZEND_LOGO_GUID"\" alt=\"Zend logo\" /></a>\n");
+			php_info_print("<a href=\"http://www.zend.com/\"><img border=\"0\" src=\"");
+			php_info_print_request_uri(TSRMLS_C);
+			php_info_print("?="ZEND_LOGO_GUID"\" alt=\"Zend logo\" /></a>\n");
 		}
-		PUTS("This program makes use of the Zend Scripting Language Engine:");
-		PUTS(!sapi_module.phpinfo_as_text?"<br />":"\n");
+		php_info_print("This program makes use of the Zend Scripting Language Engine:");
+		php_info_print(!sapi_module.phpinfo_as_text?"<br />":"\n");
 		if (sapi_module.phpinfo_as_text) {
-			PUTS(zend_version);
+			php_info_print(zend_version);
 		} else {
 			zend_html_puts(zend_version, strlen(zend_version) TSRMLS_CC);
 		}
@@ -902,15 +805,11 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 
 	if ((flag & PHP_INFO_CREDITS) && expose_php && !sapi_module.phpinfo_as_text) {	
 		php_info_print_hr();
-		PUTS("<h1><a href=\"");
-		if (SG(request_info).request_uri) {
-			char *elem_esc = php_info_html_esc(SG(request_info).request_uri TSRMLS_CC);
-			PUTS(elem_esc);
-			efree(elem_esc);
-		}
-		PUTS("?=PHPB8B5F2A0-3C92-11d3-A3A9-4C7B08C10000\">");
-		PUTS("PHP Credits");
-		PUTS("</a></h1>\n");
+		php_info_print("<h1><a href=\"");
+		php_info_print_request_uri(TSRMLS_C);
+		php_info_print("?=PHPB8B5F2A0-3C92-11d3-A3A9-4C7B08C10000\">");
+		php_info_print("PHP Credits");
+		php_info_print("</a></h1>\n");
 	}
 
 	zend_ini_sort_entries(TSRMLS_C);
@@ -918,7 +817,7 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 	if (flag & PHP_INFO_CONFIGURATION) {
 		php_info_print_hr();
 		if (!sapi_module.phpinfo_as_text) {
-			PUTS("<h1>Configuration</h1>\n");
+			php_info_print("<h1>Configuration</h1>\n");
 		} else {
 			SECTION("Configuration");
 		}	
@@ -972,25 +871,25 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 
 		php_info_print_table_start();
 		php_info_print_table_header(2, "Variable", "Value");
-		if (zend_hash_find(&EG(symbol_table), "PHP_SELF", sizeof("PHP_SELF"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
+		if (zend_hash_find(&EG(symbol_table), "PHP_SELF", sizeof("PHP_SELF"), (void **) &data) != FAILURE) {
 			php_info_print_table_row(2, "PHP_SELF", Z_STRVAL_PP(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_TYPE", sizeof("PHP_AUTH_TYPE"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
+		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_TYPE", sizeof("PHP_AUTH_TYPE"), (void **) &data) != FAILURE) {
 			php_info_print_table_row(2, "PHP_AUTH_TYPE", Z_STRVAL_PP(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_USER", sizeof("PHP_AUTH_USER"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
+		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_USER", sizeof("PHP_AUTH_USER"), (void **) &data) != FAILURE) {
 			php_info_print_table_row(2, "PHP_AUTH_USER", Z_STRVAL_PP(data));
 		}
-		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_PW", sizeof("PHP_AUTH_PW"), (void **) &data) != FAILURE && Z_TYPE_PP(data) == IS_STRING) {
+		if (zend_hash_find(&EG(symbol_table), "PHP_AUTH_PW", sizeof("PHP_AUTH_PW"), (void **) &data) != FAILURE) {
 			php_info_print_table_row(2, "PHP_AUTH_PW", Z_STRVAL_PP(data));
 		}
-		php_print_gpcse_array("_REQUEST", sizeof("_REQUEST")-1 TSRMLS_CC);
-		php_print_gpcse_array("_GET", sizeof("_GET")-1 TSRMLS_CC);
-		php_print_gpcse_array("_POST", sizeof("_POST")-1 TSRMLS_CC);
-		php_print_gpcse_array("_FILES", sizeof("_FILES")-1 TSRMLS_CC);
-		php_print_gpcse_array("_COOKIE", sizeof("_COOKIE")-1 TSRMLS_CC);
-		php_print_gpcse_array("_SERVER", sizeof("_SERVER")-1 TSRMLS_CC);
-		php_print_gpcse_array("_ENV", sizeof("_ENV")-1 TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_REQUEST") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_GET") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_POST") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_FILES") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_COOKIE") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_SERVER") TSRMLS_CC);
+		php_print_gpcse_array(ZEND_STRL("_ENV") TSRMLS_CC);
 		php_info_print_table_end();
 	}
 
@@ -998,103 +897,108 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 		if (!sapi_module.phpinfo_as_text) {
 			SECTION("PHP License");
 			php_info_print_box_start(0);
-			PUTS("<p>\n");
-			PUTS("This program is free software; you can redistribute it and/or modify ");
-			PUTS("it under the terms of the PHP License as published by the PHP Group ");
-			PUTS("and included in the distribution in the file:  LICENSE\n");
-			PUTS("</p>\n");
-			PUTS("<p>");
-			PUTS("This program is distributed in the hope that it will be useful, ");
-			PUTS("but WITHOUT ANY WARRANTY; without even the implied warranty of ");
-			PUTS("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
-			PUTS("</p>\n");
-			PUTS("<p>");
-			PUTS("If you did not receive a copy of the PHP license, or have any questions about ");
-			PUTS("PHP licensing, please contact license@php.net.\n");
-			PUTS("</p>\n");
+			php_info_print("<p>\n");
+			php_info_print("This program is free software; you can redistribute it and/or modify ");
+			php_info_print("it under the terms of the PHP License as published by the PHP Group ");
+			php_info_print("and included in the distribution in the file:  LICENSE\n");
+			php_info_print("</p>\n");
+			php_info_print("<p>");
+			php_info_print("This program is distributed in the hope that it will be useful, ");
+			php_info_print("but WITHOUT ANY WARRANTY; without even the implied warranty of ");
+			php_info_print("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
+			php_info_print("</p>\n");
+			php_info_print("<p>");
+			php_info_print("If you did not receive a copy of the PHP license, or have any questions about ");
+			php_info_print("PHP licensing, please contact license@php.net.\n");
+			php_info_print("</p>\n");
 			php_info_print_box_end();
 		} else {
-			PUTS("\nPHP License\n");
-			PUTS("This program is free software; you can redistribute it and/or modify\n");
-			PUTS("it under the terms of the PHP License as published by the PHP Group\n");
-			PUTS("and included in the distribution in the file:  LICENSE\n");
-			PUTS("\n");
-			PUTS("This program is distributed in the hope that it will be useful,\n");
-			PUTS("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
-			PUTS("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
-			PUTS("\n");
-			PUTS("If you did not receive a copy of the PHP license, or have any\n");
-			PUTS("questions about PHP licensing, please contact license@php.net.\n");
+			php_info_print("\nPHP License\n");
+			php_info_print("This program is free software; you can redistribute it and/or modify\n");
+			php_info_print("it under the terms of the PHP License as published by the PHP Group\n");
+			php_info_print("and included in the distribution in the file:  LICENSE\n");
+			php_info_print("\n");
+			php_info_print("This program is distributed in the hope that it will be useful,\n");
+			php_info_print("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
+			php_info_print("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
+			php_info_print("\n");
+			php_info_print("If you did not receive a copy of the PHP license, or have any\n");
+			php_info_print("questions about PHP licensing, please contact license@php.net.\n");
 		}
 	}
 	if (!sapi_module.phpinfo_as_text) {
-		PUTS("</div></body></html>");
+		php_info_print("</div></body></html>");
 	}	
 }
 /* }}} */
 
-
-PHPAPI void php_info_print_table_start(void)
+PHPAPI void php_info_print_table_start(void) /* {{{ */
 {
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("<table border=\"0\" cellpadding=\"3\" width=\"600\">\n");
+		php_info_print("<table border=\"0\" cellpadding=\"3\" width=\"600\">\n");
 	} else {
-		php_printf("\n");
+		php_info_print("\n");
 	}	
 }
+/* }}} */
 
-PHPAPI void php_info_print_table_end(void)
+PHPAPI void php_info_print_table_end(void) /* {{{ */
 {
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("</table><br />\n");
+		php_info_print("</table><br />\n");
 	}
 
 }
+/* }}} */
 
-PHPAPI void php_info_print_box_start(int flag)
+PHPAPI void php_info_print_box_start(int flag) /* {{{ */
 {
 	php_info_print_table_start();
 	if (flag) {
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<tr class=\"h\"><td>\n");
+			php_info_print("<tr class=\"h\"><td>\n");
 		}
 	} else {
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<tr class=\"v\"><td>\n");
+			php_info_print("<tr class=\"v\"><td>\n");
 		} else {
-			php_printf("\n");
+			php_info_print("\n");
 		}	
 	}
 }
+/* }}} */
 
-PHPAPI void php_info_print_box_end(void)
+PHPAPI void php_info_print_box_end(void) /* {{{ */
 {
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("</td></tr>\n");
+		php_info_print("</td></tr>\n");
 	}
 	php_info_print_table_end();
 }
+/* }}} */
 
-PHPAPI void php_info_print_hr(void)
+PHPAPI void php_info_print_hr(void) /* {{{ */
 {
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("<hr />\n");
+		php_info_print("<hr />\n");
 	} else {
-		php_printf("\n\n _______________________________________________________________________\n\n");
+		php_info_print("\n\n _______________________________________________________________________\n\n");
 	}
 }
+/* }}} */
 
-PHPAPI void php_info_print_table_colspan_header(int num_cols, char *header)
+PHPAPI void php_info_print_table_colspan_header(int num_cols, char *header) /* {{{ */
 {
 	int spaces;
 
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("<tr class=\"h\"><th colspan=\"%d\">%s</th></tr>\n", num_cols, header );
+		php_info_printf("<tr class=\"h\"><th colspan=\"%d\">%s</th></tr>\n", num_cols, header );
 	} else {
 		spaces = (74 - strlen(header));
-		php_printf("%*s%s%*s\n", (int)(spaces/2), " ", header, (int)(spaces/2), " ");
+		php_info_printf("%*s%s%*s\n", (int)(spaces/2), " ", header, (int)(spaces/2), " ");
 	}	
 }
+/* }}} */
 
 /* {{{ php_info_print_table_header
  */
@@ -1104,11 +1008,9 @@ PHPAPI void php_info_print_table_header(int num_cols, ...)
 	va_list row_elements;
 	char *row_element;
 
-	TSRMLS_FETCH();
-
 	va_start(row_elements, num_cols);
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("<tr class=\"h\">");
+		php_info_print("<tr class=\"h\">");
 	}	
 	for (i=0; i<num_cols; i++) {
 		row_element = va_arg(row_elements, char *);
@@ -1116,21 +1018,21 @@ PHPAPI void php_info_print_table_header(int num_cols, ...)
 			row_element = " ";
 		}
 		if (!sapi_module.phpinfo_as_text) {
-			PUTS("<th>");
-			PUTS(row_element);
-			PUTS("</th>");
+			php_info_print("<th>");
+			php_info_print(row_element);
+			php_info_print("</th>");
 		} else {
-			PUTS(row_element);
+			php_info_print(row_element);
 			if (i < num_cols-1) {
-				PUTS(" => ");
+				php_info_print(" => ");
 			} else {
-				PUTS("\n");
-			}	
-		}	
+				php_info_print("\n");
+			}
+		}
 	}
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("</tr>\n");
-	}	
+		php_info_print("</tr>\n");
+	}
 
 	va_end(row_elements);
 }
@@ -1143,49 +1045,41 @@ static void php_info_print_table_row_internal(int num_cols,
 {
 	int i;
 	char *row_element;
-	char *elem_esc = NULL;
-/*
-	int elem_esc_len;
-*/
-
-	TSRMLS_FETCH();
 
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("<tr>");
+		php_info_print("<tr>");
 	}	
 	for (i=0; i<num_cols; i++) {
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf("<td class=\"%s\">",
+			php_info_printf("<td class=\"%s\">",
 			   (i==0 ? "e" : value_class )
 			);
 		}	
 		row_element = va_arg(row_elements, char *);
 		if (!row_element || !*row_element) {
 			if (!sapi_module.phpinfo_as_text) {
-				PUTS( "<i>no value</i>" );
+				php_info_print( "<i>no value</i>" );
 			} else {
-				PUTS( " " );
+				php_info_print( " " );
 			}
 		} else {
 			if (!sapi_module.phpinfo_as_text) {
-				elem_esc = php_info_html_esc(row_element TSRMLS_CC);
-				PUTS(elem_esc);
-				efree(elem_esc);
+				php_info_print_html_esc(row_element, strlen(row_element));
 			} else {
-				PUTS(row_element);
+				php_info_print(row_element);
 				if (i < num_cols-1) {
-					PUTS(" => ");
+					php_info_print(" => ");
 				}	
 			}
 		}
 		if (!sapi_module.phpinfo_as_text) {
-			php_printf(" </td>");
+			php_info_print(" </td>");
 		} else if (i == (num_cols - 1)) {
-			PUTS("\n");
+			php_info_print("\n");
 		}
 	}
 	if (!sapi_module.phpinfo_as_text) {
-		php_printf("</tr>\n");
+		php_info_print("</tr>\n");
 	}
 }
 /* }}} */
@@ -1249,9 +1143,9 @@ PHP_FUNCTION(phpinfo)
 	}
 
 	/* Andale!  Andale!  Yee-Hah! */
-	php_start_ob_buffer(NULL, 4096, 0 TSRMLS_CC);
+	php_output_start_default(TSRMLS_C);
 	php_print_info(flag TSRMLS_CC);
-	php_end_ob_buffer(1, 0 TSRMLS_CC);
+	php_output_end(TSRMLS_C);
 
 	RETURN_TRUE;
 }
@@ -1262,20 +1156,18 @@ PHP_FUNCTION(phpinfo)
    Return the current PHP version */
 PHP_FUNCTION(phpversion)
 {
-	zval **arg;
-	const char *version;
-	int argc = ZEND_NUM_ARGS();
+	char *ext_name = NULL;
+	int ext_name_len = 0;
 
-	if (argc == 0) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &ext_name, &ext_name_len) == FAILURE) {
+		return;
+	}
+
+	if (!ext_name) {
 		RETURN_STRING(PHP_VERSION, 1);
 	} else {
-		if (zend_parse_parameters(argc TSRMLS_CC, "Z", &arg) == FAILURE) {
-			return;
-		}
-			
-		convert_to_string_ex(arg);
-		version = zend_get_module_version(Z_STRVAL_PP(arg));
-		
+		const char *version;
+		version = zend_get_module_version(ext_name);
 		if (version == NULL) {
 			RETURN_FALSE;
 		}
@@ -1299,7 +1191,6 @@ PHP_FUNCTION(phpcredits)
 }
 /* }}} */
 
-
 /* {{{ php_logo_guid
  */
 PHPAPI char *php_logo_guid(void)
@@ -1327,7 +1218,6 @@ PHPAPI char *php_logo_guid(void)
    Return the special ID used to request the PHP logo in phpinfo screens*/
 PHP_FUNCTION(php_logo_guid)
 {
-
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
@@ -1340,7 +1230,6 @@ PHP_FUNCTION(php_logo_guid)
    Return the special ID used to request the PHP logo in phpinfo screens*/
 PHP_FUNCTION(php_real_logo_guid)
 {
-
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
 	}
@@ -1396,6 +1285,7 @@ PHP_FUNCTION(php_uname)
 {
 	char *mode = "a";
 	int modelen = sizeof("a")-1;
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &mode, &modelen) == FAILURE) {
 		return;
 	}
