@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: streamsfuncs.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -36,6 +36,7 @@
 typedef unsigned long long php_timeout_ull;
 #else
 #include "win32/select.h"
+#include "win32/sockets.h"
 typedef unsigned __int64 php_timeout_ull;
 #endif
 
@@ -69,7 +70,7 @@ PHP_FUNCTION(stream_socket_pair)
 	s1 = php_stream_sock_open_from_socket(pair[0], 0);
 	s2 = php_stream_sock_open_from_socket(pair[1], 0);
 
-	/* set the __exposed flag. 
+	/* set the __exposed flag.
 	 * php_stream_to_zval() does, add_next_index_resource() does not */
 	php_stream_auto_cleanup(s1);
 	php_stream_auto_cleanup(s2);
@@ -98,23 +99,27 @@ PHP_FUNCTION(stream_socket_client)
 	php_stream_context *context = NULL;
 
 	RETVAL_FALSE;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zzd!lr", &host, &host_len, &zerrno, &zerrstr, &timeout, &flags, &zcontext) == FAILURE) {
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zzdlr", &host, &host_len, &zerrno, &zerrstr, &timeout, &flags, &zcontext) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	context = php_stream_context_from_zval(zcontext, flags & PHP_FILE_NO_DEFAULT_CONTEXT);
 
 	if (flags & PHP_STREAM_CLIENT_PERSISTENT) {
 		spprintf(&hashkey, 0, "stream_socket_client__%s", host);
 	}
-	
+
 	/* prepare the timeout value for use */
 	conv = (php_timeout_ull) (timeout * 1000000.0);
+#ifdef PHP_WIN32
+	tv.tv_sec = (long)(conv / 1000000);
+	tv.tv_usec =(long)(conv % 1000000);
+#else
 	tv.tv_sec = conv / 1000000;
 	tv.tv_usec = conv % 1000000;
-
-	if (zerrno) {
+#endif
+	if (zerrno)	{
 		zval_dtor(zerrno);
 		ZVAL_LONG(zerrno, 0);
 	}
@@ -123,16 +128,16 @@ PHP_FUNCTION(stream_socket_client)
 		ZVAL_STRING(zerrstr, "", 1);
 	}
 
-	stream = php_stream_xport_create(host, host_len, ENFORCE_SAFE_MODE | REPORT_ERRORS,
+	stream = php_stream_xport_create(host, host_len, REPORT_ERRORS,
 			STREAM_XPORT_CLIENT | (flags & PHP_STREAM_CLIENT_CONNECT ? STREAM_XPORT_CONNECT : 0) |
 			(flags & PHP_STREAM_CLIENT_ASYNC_CONNECT ? STREAM_XPORT_CONNECT_ASYNC : 0),
 			hashkey, &tv, context, &errstr, &err);
-		
+
 
 	if (stream == NULL) {
 		/* host might contain binary characters */
 		char *quoted_host = php_addslashes(host, host_len, NULL, 0 TSRMLS_CC);
-		
+
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to connect to %s (%s)", quoted_host, errstr == NULL ? "Unknown error" : errstr);
 		efree(quoted_host);
 	}
@@ -140,7 +145,7 @@ PHP_FUNCTION(stream_socket_client)
 	if (hashkey) {
 		efree(hashkey);
 	}
-	
+
 	if (stream == NULL)	{
 		if (zerrno) {
 			zval_dtor(zerrno);
@@ -162,9 +167,6 @@ PHP_FUNCTION(stream_socket_client)
 
 	php_stream_to_zval(stream, return_value);
 
-	if (zcontext) {
-		zend_list_addref(Z_RESVAL_P(zcontext));
-	}
 }
 /* }}} */
 
@@ -182,13 +184,17 @@ PHP_FUNCTION(stream_socket_server)
 	php_stream_context *context = NULL;
 
 	RETVAL_FALSE;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|zzlr", &host, &host_len, &zerrno, &zerrstr, &flags, &zcontext) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	context = php_stream_context_from_zval(zcontext, flags & PHP_FILE_NO_DEFAULT_CONTEXT);
 
+	if (context) {
+		zend_list_addref(context->rsrc_id);
+	}
+
 	if (zerrno)	{
 		zval_dtor(zerrno);
 		ZVAL_LONG(zerrno, 0);
@@ -198,14 +204,14 @@ PHP_FUNCTION(stream_socket_server)
 		ZVAL_STRING(zerrstr, "", 1);
 	}
 
-	stream = php_stream_xport_create(host, host_len, ENFORCE_SAFE_MODE | REPORT_ERRORS,
+	stream = php_stream_xport_create(host, host_len, REPORT_ERRORS,
 			STREAM_XPORT_SERVER | flags,
 			NULL, NULL, context, &errstr, &err);
-			
+
 	if (stream == NULL) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to connect to %s (%s)", host, errstr == NULL ? "Unknown error" : errstr);
 	}
-	
+
 	if (stream == NULL)	{
 		if (zerrno) {
 			zval_dtor(zerrno);
@@ -220,57 +226,59 @@ PHP_FUNCTION(stream_socket_server)
 		}
 		RETURN_FALSE;
 	}
-	
+
 	if (errstr) {
 		efree(errstr);
 	}
-	
-	php_stream_to_zval(stream, return_value);
 
-	if (zcontext) {
-		zend_list_addref(Z_RESVAL_P(zcontext));
-	}
+	php_stream_to_zval(stream, return_value);
 }
 /* }}} */
 
-/* {{{ proto resource stream_socket_accept(resource serverstream [, double timeout [, string &peername ]])
+/* {{{ proto resource stream_socket_accept(resource serverstream, [ double timeout [, string &peername ]])
    Accept a client connection from a server socket */
 PHP_FUNCTION(stream_socket_accept)
 {
 	double timeout = FG(default_socket_timeout);
-	zval *peername = NULL;
+	zval *zpeername = NULL;
+	char *peername = NULL;
+	int peername_len;
 	php_timeout_ull conv;
 	struct timeval tv;
 	php_stream *stream = NULL, *clistream = NULL;
 	zval *zstream;
 
 	char *errstr = NULL;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|dz", &zstream, &timeout, &peername) == FAILURE) {
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|dz", &zstream, &timeout, &zpeername) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	php_stream_from_zval(stream, &zstream);
-	
+
 	/* prepare the timeout value for use */
 	conv = (php_timeout_ull) (timeout * 1000000.0);
+#ifdef PHP_WIN32
+	tv.tv_sec = (long)(conv / 1000000);
+	tv.tv_usec = (long)(conv % 1000000);
+#else
 	tv.tv_sec = conv / 1000000;
 	tv.tv_usec = conv % 1000000;
-
-	if (peername) {
-		zval_dtor(peername);
-		ZVAL_NULL(peername);
+#endif
+	if (zpeername) {
+		zval_dtor(zpeername);
+		ZVAL_NULL(zpeername);
 	}
 
 	if (0 == php_stream_xport_accept(stream, &clistream,
-				peername ? &Z_STRVAL_P(peername) : NULL,
-				peername ? &Z_STRLEN_P(peername) : NULL,
+				zpeername ? &peername : NULL,
+				zpeername ? &peername_len : NULL,
 				NULL, NULL,
 				&tv, &errstr
 				TSRMLS_CC) && clistream) {
-		
+
 		if (peername) {
-			Z_TYPE_P(peername) = IS_STRING;
+			ZVAL_STRINGL(zpeername, peername, peername_len, 0);
 		}
 		php_stream_to_zval(clistream, return_value);
 	} else {
@@ -291,22 +299,24 @@ PHP_FUNCTION(stream_socket_get_name)
 	php_stream *stream;
 	zval *zstream;
 	zend_bool want_peer;
-	
+	char *name = NULL;
+	int name_len;
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rb", &zstream, &want_peer) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	php_stream_from_zval(stream, &zstream);
 
-	Z_TYPE_P(return_value) = IS_STRING;
-	
 	if (0 != php_stream_xport_get_name(stream, want_peer,
-				&Z_STRVAL_P(return_value),
-				&Z_STRLEN_P(return_value),
+				&name,
+				&name_len,
 				NULL, NULL
 				TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
+
+	RETURN_STRINGL(name, name_len, 0);
 }
 /* }}} */
 
@@ -321,7 +331,7 @@ PHP_FUNCTION(stream_socket_sendto)
 	int datalen, target_addr_len = 0;
 	php_sockaddr_storage sa;
 	socklen_t sl = 0;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|ls", &zstream, &data, &datalen, &flags, &target_addr, &target_addr_len) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -345,49 +355,43 @@ PHP_FUNCTION(stream_socket_recvfrom)
 {
 	php_stream *stream;
 	zval *zstream, *zremote = NULL;
+	char *remote_addr = NULL;
+	int remote_addr_len;
 	long to_read = 0;
 	char *read_buf;
 	long flags = 0;
 	int recvd;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl|lz", &zstream, &to_read, &flags, &zremote) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	php_stream_from_zval(stream, &zstream);
 
 	if (zremote) {
 		zval_dtor(zremote);
 		ZVAL_NULL(zremote);
-		Z_STRLEN_P(zremote) = 0;
 	}
 
 	if (to_read <= 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0");
 		RETURN_FALSE;
 	}
-	
+
 	read_buf = safe_emalloc(1, to_read, 1);
-	
+
 	recvd = php_stream_xport_recvfrom(stream, read_buf, to_read, flags, NULL, NULL,
-			zremote ? &Z_STRVAL_P(zremote) : NULL,
-			zremote ? &Z_STRLEN_P(zremote) : NULL
+			zremote ? &remote_addr : NULL,
+			zremote ? &remote_addr_len : NULL
 			TSRMLS_CC);
 
 	if (recvd >= 0) {
-		if (zremote && Z_STRLEN_P(zremote)) {
-			Z_TYPE_P(zremote) = IS_STRING;
+		if (zremote) {
+			ZVAL_STRINGL(zremote, remote_addr, remote_addr_len, 0);
 		}
 		read_buf[recvd] = '\0';
 
-		if (PG(magic_quotes_runtime)) {
-			Z_TYPE_P(return_value) = IS_STRING;
-			Z_STRVAL_P(return_value) = php_addslashes(Z_STRVAL_P(return_value),
-					Z_STRLEN_P(return_value), &Z_STRLEN_P(return_value), 1 TSRMLS_CC);
-			return;
-		} else {
-			RETURN_STRINGL(read_buf, recvd, 0);
-		}
+		RETURN_STRINGL(read_buf, recvd, 0);
 	}
 
 	efree(read_buf);
@@ -395,34 +399,46 @@ PHP_FUNCTION(stream_socket_recvfrom)
 }
 /* }}} */
 
-/* {{{ proto long stream_get_contents(resource source [, long maxlen [, long offset]])
+/* {{{ proto string stream_get_contents(resource source [, long maxlen [, long offset]])
    Reads all remaining bytes (or up to maxlen bytes) from a stream and returns them as a string. */
 PHP_FUNCTION(stream_get_contents)
 {
-	php_stream *stream;
-	zval *zsrc;
-	long maxlen = PHP_STREAM_COPY_ALL, pos = 0;
-	int len, newlen;
-	char *contents = NULL;
+	php_stream	*stream;
+	zval		*zsrc;
+	long		maxlen		= PHP_STREAM_COPY_ALL,
+				desiredpos	= -1L;
+	int			len;
+	char		*contents	= NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|ll", &zsrc, &maxlen, &pos) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|ll", &zsrc, &maxlen, &desiredpos) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	php_stream_from_zval(stream, &zsrc);
 
-	if ((pos > 0 || (pos == 0 && ZEND_NUM_ARGS() > 2)) && php_stream_seek(stream, pos, SEEK_SET) < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to position %ld in the stream", pos);
-		RETURN_FALSE;
+	if (desiredpos >= 0) {
+		int		seek_res = 0;
+		off_t	position;
+
+		position = php_stream_tell(stream);
+		if (position >= 0 && desiredpos > position) {
+			/* use SEEK_CUR to allow emulation in streams that don't support seeking */
+			seek_res = php_stream_seek(stream, desiredpos - position, SEEK_CUR);
+		} else if (desiredpos < position)  {
+			/* desired position before position or error on tell */
+			seek_res = php_stream_seek(stream, desiredpos, SEEK_SET);
+		}
+
+		if (seek_res != 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,
+				"Failed to seek to position %ld in the stream", desiredpos);
+			RETURN_FALSE;
+		}
 	}
 
 	len = php_stream_copy_to_mem(stream, &contents, maxlen, 0);
-	
+
 	if (contents) {
-		if (len && PG(magic_quotes_runtime)) {
-			contents = php_addslashes(contents, len, &newlen, 1 TSRMLS_CC); /* 1 = free source string */
-			len = newlen;
-		}
 		RETVAL_STRINGL(contents, len, 0);
 	} else {
 		RETVAL_EMPTY_STRING();
@@ -465,22 +481,20 @@ PHP_FUNCTION(stream_copy_to_stream)
     Retrieves header/meta data from streams/file pointers */
 PHP_FUNCTION(stream_get_meta_data)
 {
-	zval **arg1;
+	zval *arg1;
 	php_stream *stream;
 	zval *newval;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg1) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &arg1) == FAILURE) {
+		return;
 	}
-	php_stream_from_zval(stream, arg1);
+	php_stream_from_zval(stream, &arg1);
 
 	array_init(return_value);
-	
+
 	if (stream->wrapperdata) {
 		MAKE_STD_ZVAL(newval);
-		*newval = *(stream->wrapperdata);
-		zval_copy_ctor(newval);
-		INIT_PZVAL(newval);
+		MAKE_COPY_ZVAL(&stream->wrapperdata, newval);
 
 		add_assoc_zval(return_value, "wrapper_data", newval);
 	}
@@ -490,14 +504,14 @@ PHP_FUNCTION(stream_get_meta_data)
 	add_assoc_string(return_value, "stream_type", (char *)stream->ops->label, 1);
 
 	add_assoc_string(return_value, "mode", stream->mode, 1);
-	
+
 #if 0	/* TODO: needs updating for new filter API */
 	if (stream->filterhead) {
 		php_stream_filter *filter;
-		
+
 		MAKE_STD_ZVAL(newval);
 		array_init(newval);
-		
+
 		for (filter = stream->filterhead; filter != NULL; filter = filter->next) {
 			add_next_index_string(newval, (char *)filter->fops->label, 1);
 		}
@@ -505,7 +519,7 @@ PHP_FUNCTION(stream_get_meta_data)
 		add_assoc_zval(return_value, "filters", newval);
 	}
 #endif
-	
+
 	add_assoc_long(return_value, "unread_bytes", stream->writepos - stream->readpos);
 
 	add_assoc_bool(return_value, "seekable", (stream->ops->seek) && (stream->flags & PHP_STREAM_FLAG_NO_SEEK) == 0);
@@ -528,21 +542,22 @@ PHP_FUNCTION(stream_get_transports)
 {
 	HashTable *stream_xport_hash;
 	char *stream_xport;
-	int stream_xport_len;
+	uint stream_xport_len;
 	ulong num_key;
 
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if ((stream_xport_hash = php_stream_xport_get_hash())) {
+		HashPosition pos;
 		array_init(return_value);
-		zend_hash_internal_pointer_reset(stream_xport_hash);
+		zend_hash_internal_pointer_reset_ex(stream_xport_hash, &pos);
 		while (zend_hash_get_current_key_ex(stream_xport_hash,
 					&stream_xport, &stream_xport_len,
-					&num_key, 0, NULL) == HASH_KEY_IS_STRING) {
+					&num_key, 0, &pos) == HASH_KEY_IS_STRING) {
 			add_next_index_stringl(return_value, stream_xport, stream_xport_len - 1, 1);
-			zend_hash_move_forward(stream_xport_hash);
+			zend_hash_move_forward_ex(stream_xport_hash, &pos);
 		}
 	} else {
 		RETURN_FALSE;
@@ -556,18 +571,20 @@ PHP_FUNCTION(stream_get_wrappers)
 {
 	HashTable *url_stream_wrappers_hash;
 	char *stream_protocol;
-	int key_flags, stream_protocol_len = 0;
+	int key_flags;
+	uint stream_protocol_len = 0;
 	ulong num_key;
 
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	if ((url_stream_wrappers_hash = php_stream_get_url_stream_wrappers_hash())) {
+		HashPosition pos;
 		array_init(return_value);
-		for (zend_hash_internal_pointer_reset(url_stream_wrappers_hash);
-			(key_flags = zend_hash_get_current_key_ex(url_stream_wrappers_hash, &stream_protocol, &stream_protocol_len, &num_key, 0, NULL)) != HASH_KEY_NON_EXISTANT;
-			zend_hash_move_forward(url_stream_wrappers_hash)) {
+		for (zend_hash_internal_pointer_reset_ex(url_stream_wrappers_hash, &pos);
+			(key_flags = zend_hash_get_current_key_ex(url_stream_wrappers_hash, &stream_protocol, &stream_protocol_len, &num_key, 0, &pos)) != HASH_KEY_NON_EXISTANT;
+			zend_hash_move_forward_ex(url_stream_wrappers_hash, &pos)) {
 				if (key_flags == HASH_KEY_IS_STRING) {
 					add_next_index_stringl(return_value, stream_protocol, stream_protocol_len - 1, 1);
 				}
@@ -585,6 +602,7 @@ static int stream_array_to_fd_set(zval *stream_array, fd_set *fds, php_socket_t
 	zval **elem;
 	php_stream *stream;
 	php_socket_t this_fd;
+	int cnt = 0;
 
 	if (Z_TYPE_P(stream_array) != IS_ARRAY) {
 		return 0;
@@ -602,16 +620,17 @@ static int stream_array_to_fd_set(zval *stream_array, fd_set *fds, php_socket_t
 		 * when casting.  It is only used here so that the buffered data warning
 		 * is not displayed.
 		 * */
-		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd >= 0) {
-			
+		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd != -1) {
+
 			PHP_SAFE_FD_SET(this_fd, fds);
 
 			if (this_fd > *max_fd) {
 				*max_fd = this_fd;
 			}
+			cnt++;
 		}
 	}
-	return 1;
+	return cnt ? 1 : 0;
 }
 
 static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
@@ -619,30 +638,48 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 	zval **elem, **dest_elem;
 	php_stream *stream;
 	HashTable *new_hash;
-	int this_fd, ret = 0;
+	php_socket_t this_fd;
+	int ret = 0;
 
 	if (Z_TYPE_P(stream_array) != IS_ARRAY) {
 		return 0;
 	}
 	ALLOC_HASHTABLE(new_hash);
 	zend_hash_init(new_hash, zend_hash_num_elements(Z_ARRVAL_P(stream_array)), NULL, ZVAL_PTR_DTOR, 0);
-	
+
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
-		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
+		 zend_hash_has_more_elements(Z_ARRVAL_P(stream_array)) == SUCCESS;
 		 zend_hash_move_forward(Z_ARRVAL_P(stream_array))) {
 
+		int type;
+		char *key;
+		uint key_len;
+		ulong num_ind;
+
+		type = zend_hash_get_current_key_ex(Z_ARRVAL_P(stream_array),
+				&key, &key_len, &num_ind, 0, NULL);
+		if (type == HASH_KEY_NON_EXISTANT ||
+			zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == FAILURE) {
+			continue; /* should not happen */
+		}
+
 		php_stream_from_zval_no_verify(stream, elem);
 		if (stream == NULL) {
 			continue;
 		}
-		/* get the fd 
+		/* get the fd
 		 * NB: Most other code will NOT use the PHP_STREAM_CAST_INTERNAL flag
 		 * when casting.  It is only used here so that the buffered data warning
 		 * is not displayed.
 		 */
-		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd >= 0) {
+		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd != -1) {
 			if (PHP_SAFE_FD_ISSET(this_fd, fds)) {
-				zend_hash_next_index_insert(new_hash, (void *)elem, sizeof(zval *), (void **)&dest_elem);
+				if (type == HASH_KEY_IS_LONG) {
+					zend_hash_index_update(new_hash, num_ind, (void *)elem, sizeof(zval *), (void **)&dest_elem);
+				} else { /* HASH_KEY_IS_STRING */
+					zend_hash_update(new_hash, key, key_len, (void *)elem, sizeof(zval *), (void **)&dest_elem);
+				}
+				
 				if (dest_elem) {
 					zval_add_ref(dest_elem);
 				}
@@ -658,7 +695,7 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 
 	zend_hash_internal_pointer_reset(new_hash);
 	Z_ARRVAL_P(stream_array) = new_hash;
-	
+
 	return ret;
 }
 
@@ -674,7 +711,7 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
 	}
 	ALLOC_HASHTABLE(new_hash);
 	zend_hash_init(new_hash, zend_hash_num_elements(Z_ARRVAL_P(stream_array)), NULL, ZVAL_PTR_DTOR, 0);
-	
+
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
 		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
 		 zend_hash_move_forward(Z_ARRVAL_P(stream_array))) {
@@ -710,7 +747,7 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
 		zend_hash_destroy(new_hash);
 		FREE_HASHTABLE(new_hash);
 	}
-	
+
 	return ret;
 }
 /* }}} */
@@ -741,7 +778,7 @@ PHP_FUNCTION(stream_select)
 			max_set_count = set_count;
 		sets += set_count;
 	}
-	
+
 	if (w_array != NULL) {
 		set_count = stream_array_to_fd_set(w_array, &wfds, &max_fd TSRMLS_CC);
 		if (set_count > max_set_count)
@@ -778,7 +815,7 @@ PHP_FUNCTION(stream_select)
 		/* Solaris + BSD do not like microsecond values which are >= 1 sec */
 		if (usec > 999999) {
 			tv.tv_sec = Z_LVAL_PP(sec) + (usec / 1000000);
-			tv.tv_usec = usec % 1000000;			
+			tv.tv_usec = usec % 1000000;
 		} else {
 			tv.tv_sec = Z_LVAL_PP(sec);
 			tv.tv_usec = usec;
@@ -803,7 +840,7 @@ PHP_FUNCTION(stream_select)
 			RETURN_LONG(retval);
 		}
 	}
-	
+
 	retval = php_select(max_fd+1, &rfds, &wfds, &efds, tv_p);
 
 	if (retval == -1) {
@@ -830,14 +867,14 @@ static void user_space_stream_notifier(php_stream_context *context, int notifyco
 	zval *ps[6];
 	zval **ptps[6];
 	int i;
-	
+
 	for (i = 0; i < 6; i++) {
 		INIT_ZVAL(zvs[i]);
 		ps[i] = &zvs[i];
 		ptps[i] = &ps[i];
 		MAKE_STD_ZVAL(ps[i]);
 	}
-		
+
 	ZVAL_LONG(ps[0], notifycode);
 	ZVAL_LONG(ps[1], severity);
 	if (xmsg) {
@@ -868,15 +905,15 @@ static void user_space_stream_notifier_dtor(php_stream_notifier *notifier)
 	}
 }
 
-static int parse_context_options(php_stream_context *context, zval *options)
+static int parse_context_options(php_stream_context *context, zval *options TSRMLS_DC)
 {
 	HashPosition pos, opos;
 	zval **wval, **oval;
 	char *wkey, *okey;
-	int wkey_len, okey_len;
+	uint wkey_len, okey_len;
 	int ret = SUCCESS;
 	ulong num_key;
-	
+
 	zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(options), &pos);
 	while (SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(options), (void**)&wval, &pos)) {
 		if (HASH_KEY_IS_STRING == zend_hash_get_current_key_ex(Z_ARRVAL_P(options), &wkey, &wkey_len, &num_key, 0, &pos)
@@ -892,7 +929,7 @@ static int parse_context_options(php_stream_context *context, zval *options)
 			}
 
 		} else {
-			zend_error(E_WARNING, "options should have the form [\"wrappername\"][\"optionname\"] = $value");
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "options should have the form [\"wrappername\"][\"optionname\"] = $value");
 		}
 		zend_hash_move_forward_ex(Z_ARRVAL_P(options), &pos);
 	}
@@ -906,7 +943,7 @@ static int parse_context_params(php_stream_context *context, zval *params TSRMLS
 	zval **tmp;
 
 	if (SUCCESS == zend_hash_find(Z_ARRVAL_P(params), "notification", sizeof("notification"), (void**)&tmp)) {
-		
+
 		if (context->notifier) {
 			php_stream_notification_free(context->notifier);
 			context->notifier = NULL;
@@ -915,17 +952,17 @@ static int parse_context_params(php_stream_context *context, zval *params TSRMLS
 		context->notifier = php_stream_notification_alloc();
 		context->notifier->func = user_space_stream_notifier;
 		context->notifier->ptr = *tmp;
-		ZVAL_ADDREF(*tmp);
+		Z_ADDREF_P(*tmp);
 		context->notifier->dtor = user_space_stream_notifier_dtor;
 	}
 	if (SUCCESS == zend_hash_find(Z_ARRVAL_P(params), "options", sizeof("options"), (void**)&tmp)) {
 		if (Z_TYPE_PP(tmp) == IS_ARRAY) {
-			parse_context_options(context, *tmp);
+			parse_context_options(context, *tmp TSRMLS_CC);
 		} else {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
 		}
 	}
-	
+
 	return ret;
 }
 
@@ -936,7 +973,7 @@ static php_stream_context *decode_context_param(zval *contextresource TSRMLS_DC)
 {
 	php_stream_context *context = NULL;
 
-	context = zend_fetch_resource(&contextresource TSRMLS_CC, -1, NULL, NULL, 1, php_le_stream_context());
+	context = zend_fetch_resource(&contextresource TSRMLS_CC, -1, NULL, NULL, 1, php_le_stream_context(TSRMLS_C));
 	if (context == NULL) {
 		php_stream *stream;
 
@@ -949,7 +986,7 @@ static php_stream_context *decode_context_param(zval *contextresource TSRMLS_DC)
 				   param, but then something is called which requires a context.
 				   Don't give them the default one though since they already said they
 	 			   didn't want it. */
-				context = stream->context = php_stream_context_alloc();
+				context = stream->context = php_stream_context_alloc(TSRMLS_C);
 			}
 		}
 	}
@@ -1006,7 +1043,7 @@ PHP_FUNCTION(stream_context_set_option)
 
 	if (options) {
 		/* handle the array syntax */
-		RETVAL_BOOL(parse_context_options(context, options) == SUCCESS);
+		RETVAL_BOOL(parse_context_options(context, options TSRMLS_CC) == SUCCESS);
 	} else {
 		php_stream_context_set_option(context, wrappername, optionname, zvalue);
 		RETVAL_TRUE;
@@ -1035,6 +1072,34 @@ PHP_FUNCTION(stream_context_set_params)
 }
 /* }}} */
 
+/* {{{ proto array stream_context_get_params(resource context|resource stream)
+   Get parameters of a file context */
+PHP_FUNCTION(stream_context_get_params)
+{
+	zval *zcontext, *options;
+	php_stream_context *context;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zcontext) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	context = decode_context_param(zcontext TSRMLS_CC);
+	if (!context) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid stream/context parameter");
+		RETURN_FALSE;
+	}
+
+	array_init(return_value);
+	if (context->notifier && context->notifier->ptr && context->notifier->func == user_space_stream_notifier) {
+		add_assoc_zval_ex(return_value, ZEND_STRS("notification"), context->notifier->ptr);
+		Z_ADDREF_P(context->notifier->ptr);
+	}
+	ALLOC_INIT_ZVAL(options);
+	ZVAL_ZVAL(options, context->options, 1, 0);
+	add_assoc_zval_ex(return_value, ZEND_STRS("options"), options);
+}
+/* }}} */
+
 /* {{{ proto resource stream_context_get_default([array options])
    Get a handle on the default file/stream context and optionally set parameters */
 PHP_FUNCTION(stream_context_get_default)
@@ -1045,38 +1110,64 @@ PHP_FUNCTION(stream_context_get_default)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a", &params) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	if (FG(default_context) == NULL) {
-		FG(default_context) = php_stream_context_alloc();
+		FG(default_context) = php_stream_context_alloc(TSRMLS_C);
 	}
 	context = FG(default_context);
-	
+
 	if (params) {
-		parse_context_options(context, params);
+		parse_context_options(context, params TSRMLS_CC);
 	}
-	
+
 	php_stream_context_to_zval(context, return_value);
 }
 /* }}} */
 
-/* {{{ proto resource stream_context_create([array options])
+/* {{{ proto resource stream_context_set_default(array options)
+   Set default file/stream context, returns the context as a resource */
+PHP_FUNCTION(stream_context_set_default)
+{
+	zval *options = NULL;
+	php_stream_context *context;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &options) == FAILURE) {
+		return;
+	}
+
+	if (FG(default_context) == NULL) {
+		FG(default_context) = php_stream_context_alloc(TSRMLS_C);
+	}
+	context = FG(default_context);
+
+	parse_context_options(context, options TSRMLS_CC);
+
+	php_stream_context_to_zval(context, return_value);
+}
+/* }}} */
+
+/* {{{ proto resource stream_context_create([array options[, array params]])
    Create a file context and optionally set parameters */
 PHP_FUNCTION(stream_context_create)
 {
-	zval *params = NULL;
+	zval *options = NULL, *params = NULL;
 	php_stream_context *context;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a", &params) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a!a!", &options, &params) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
-	context = php_stream_context_alloc();
-	
-	if (params) {
-		parse_context_options(context, params);
+
+	context = php_stream_context_alloc(TSRMLS_C);
+
+	if (options) {
+		parse_context_options(context, options TSRMLS_CC);
 	}
-	
-	php_stream_context_to_zval(context, return_value);
+
+	if (params) {
+		parse_context_params(context, params TSRMLS_CC);
+	}
+
+	RETURN_RESOURCE(context->rsrc_id);
 }
 /* }}} */
 
@@ -1103,7 +1194,7 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 		/* Chain not specified.
 		 * Examine stream->mode to determine which filters are needed
 		 * There's no harm in attaching a filter to an unused chain,
-		 * but why waste the memory and clock cycles? 
+		 * but why waste the memory and clock cycles?
 		 */
 		if (strchr(stream->mode, 'r') || strchr(stream->mode, '+')) {
 			read_write |= PHP_STREAM_FILTER_READ;
@@ -1119,7 +1210,7 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 			RETURN_FALSE;
 		}
 
-		if (append) { 
+		if (append) {
 			ret = php_stream_filter_append_ex(&stream->readfilters, filter TSRMLS_CC);
 		} else {
 			ret = php_stream_filter_prepend_ex(&stream->readfilters, filter TSRMLS_CC);
@@ -1136,7 +1227,7 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 			RETURN_FALSE;
 		}
 
-		if (append) { 
+		if (append) {
 			ret = php_stream_filter_append_ex(&stream->writefilters, filter TSRMLS_CC);
 		} else {
 			ret = php_stream_filter_prepend_ex(&stream->writefilters, filter TSRMLS_CC);
@@ -1214,7 +1305,7 @@ PHP_FUNCTION(stream_get_line)
 	char *buf;
 	size_t buf_size;
 	php_stream *stream;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl|s", &zstream, &max_length, &str, &str_len) == FAILURE) {
 		RETURN_FALSE;
 	}
@@ -1242,52 +1333,53 @@ PHP_FUNCTION(stream_get_line)
    Set blocking/non-blocking mode on a socket or stream */
 PHP_FUNCTION(stream_set_blocking)
 {
-	zval **arg1, **arg2;
+	zval *arg1;
 	int block;
+	long arg2;
 	php_stream *stream;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &arg1, &arg2) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &arg1, &arg2) == FAILURE) {
+		return;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	php_stream_from_zval(stream, &arg1);
 
-	convert_to_long_ex(arg2);
-	block = Z_LVAL_PP(arg2);
+	block = arg2;
 
-	if (php_stream_set_option(stream, PHP_STREAM_OPTION_BLOCKING, block == 0 ? 0 : 1, NULL) == -1)
+	if (php_stream_set_option(stream, PHP_STREAM_OPTION_BLOCKING, block == 0 ? 0 : 1, NULL) == -1) {
 		RETURN_FALSE;
+	}
+
 	RETURN_TRUE;
 }
 
 /* }}} */
 
-/* {{{ proto bool stream_set_timeout(resource stream, int seconds, int microseconds)
+/* {{{ proto bool stream_set_timeout(resource stream, int seconds [, int microseconds])
    Set timeout on stream read to seconds + microseonds */
 #if HAVE_SYS_TIME_H || defined(PHP_WIN32)
 PHP_FUNCTION(stream_set_timeout)
 {
-	zval **socket, **seconds, **microseconds;
+	zval *socket;
+	long seconds, microseconds = 0;
 	struct timeval t;
 	php_stream *stream;
+	int argc = ZEND_NUM_ARGS();
 
-	if (ZEND_NUM_ARGS() < 2 || ZEND_NUM_ARGS() > 3 ||
-		zend_get_parameters_ex(ZEND_NUM_ARGS(), &socket, &seconds, &microseconds)==FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(argc TSRMLS_CC, "rl|l", &socket, &seconds, &microseconds) == FAILURE) {
+		return;
 	}
 
-	php_stream_from_zval(stream, socket);
+	php_stream_from_zval(stream, &socket);
 
-	convert_to_long_ex(seconds);
-	t.tv_sec = Z_LVAL_PP(seconds);
+	t.tv_sec = seconds;
 
-	if (ZEND_NUM_ARGS() == 3) {
-		convert_to_long_ex(microseconds);
-		t.tv_usec = Z_LVAL_PP(microseconds) % 1000000;
-		t.tv_sec += Z_LVAL_PP(microseconds) / 1000000;
-	}
-	else
+	if (argc == 3) {
+		t.tv_usec = microseconds % 1000000;
+		t.tv_sec += microseconds / 1000000;
+	} else {
 		t.tv_usec = 0;
+	}
 
 	if (PHP_STREAM_OPTION_RETURN_OK == php_stream_set_option(stream, PHP_STREAM_OPTION_READ_TIMEOUT, 0, &t)) {
 		RETURN_TRUE;
@@ -1302,27 +1394,19 @@ PHP_FUNCTION(stream_set_timeout)
    Set file write buffer */
 PHP_FUNCTION(stream_set_write_buffer)
 {
-	zval **arg1, **arg2;
+	zval *arg1;
 	int ret;
+	long arg2;
 	size_t buff;
 	php_stream *stream;
 
-	switch (ZEND_NUM_ARGS()) {
-	case 2:
-		if (zend_get_parameters_ex(2, &arg1, &arg2)==FAILURE) {
-			RETURN_FALSE;
-		}
-		break;
-	default:
-		WRONG_PARAM_COUNT;
-		/* NOTREACHED */
-		break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &arg1, &arg2) == FAILURE) {
+		RETURN_FALSE;
 	}
-	
-	php_stream_from_zval(stream, arg1);
-	
-	convert_to_long_ex(arg2);
-	buff = Z_LVAL_PP(arg2);
+
+	php_stream_from_zval(stream, &arg1);
+
+	buff = arg2;
 
 	/* if buff is 0 then set to non-buffered */
 	if (buff == 0) {
@@ -1335,27 +1419,90 @@ PHP_FUNCTION(stream_set_write_buffer)
 }
 /* }}} */
 
+/* {{{ proto int stream_set_chunk_size(resource fp, int chunk_size)
+   Set the stream chunk size */
+PHP_FUNCTION(stream_set_chunk_size)
+{
+	int			ret;
+	long		csize;
+	zval		*zstream;
+	php_stream	*stream;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &zstream, &csize) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	if (csize <= 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "The chunk size must be a positive integer, given %ld", csize);
+		RETURN_FALSE;
+	}
+	/* stream.chunk_size is actually a size_t, but php_stream_set_option 
+	 * can only use an int to accept the new value and return the old one.
+	 * In any case, values larger than INT_MAX for a chunk size make no sense.
+	 */
+	if (csize > INT_MAX) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "The chunk size cannot be larger than %d", INT_MAX);
+		RETURN_FALSE;
+	}
+	
+	php_stream_from_zval(stream, &zstream);
+
+	ret = php_stream_set_option(stream, PHP_STREAM_OPTION_SET_CHUNK_SIZE, (int)csize, NULL);
+	
+	RETURN_LONG(ret > 0 ? (long)ret : (long)EOF);
+}
+/* }}} */
+
+/* {{{ proto int stream_set_read_buffer(resource fp, int buffer)
+   Set file read buffer */
+PHP_FUNCTION(stream_set_read_buffer)
+{
+	zval *arg1;
+	int ret;
+	long arg2;
+	size_t buff;
+	php_stream *stream;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &arg1, &arg2) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	php_stream_from_zval(stream, &arg1);
+
+	buff = arg2;
+
+	/* if buff is 0 then set to non-buffered */
+	if (buff == 0) {
+		ret = php_stream_set_option(stream, PHP_STREAM_OPTION_READ_BUFFER, PHP_STREAM_BUFFER_NONE, NULL);
+	} else {
+		ret = php_stream_set_option(stream, PHP_STREAM_OPTION_READ_BUFFER, PHP_STREAM_BUFFER_FULL, &buff);
+	}
+
+	RETURN_LONG(ret == 0 ? 0 : EOF);
+}
+/* }}} */
+
 /* {{{ proto int stream_socket_enable_crypto(resource stream, bool enable [, int cryptokind [, resource sessionstream]])
    Enable or disable a specific kind of crypto on the stream */
 PHP_FUNCTION(stream_socket_enable_crypto)
 {
-	long cryptokind;
+	long cryptokind = 0;
 	zval *zstream, *zsessstream = NULL;
 	php_stream *stream, *sessstream = NULL;
 	zend_bool enable;
 	int ret;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rb|lr", &zstream, &enable, &cryptokind, &zsessstream) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	php_stream_from_zval(stream, &zstream);
 
 	if (ZEND_NUM_ARGS() >= 3) {
 		if (zsessstream) {
 			php_stream_from_zval(sessstream, &zsessstream);
 		}
-		
+
 		if (php_stream_xport_crypto_setup(stream, cryptokind, sessstream TSRMLS_CC) < 0) {
 			RETURN_FALSE;
 		}
@@ -1371,13 +1518,33 @@ PHP_FUNCTION(stream_socket_enable_crypto)
 
 		case 0:
 			RETURN_LONG(0);
-		
+
 		default:
 			RETURN_TRUE;
 	}
 }
 /* }}} */
 
+/* {{{ proto string stream_resolve_include_path(string filename)
+Determine what file will be opened by calls to fopen() with a relative path */
+PHP_FUNCTION(stream_resolve_include_path)
+{
+	char *filename, *resolved_path;
+	int filename_len;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
+		return;
+	}
+
+	resolved_path = zend_resolve_path(filename, filename_len TSRMLS_CC);
+
+	if (resolved_path) {
+		RETURN_STRING(resolved_path, 0);
+	}
+	RETURN_FALSE;
+}
+/* }}} */
+
 /* {{{ proto bool stream_is_local(resource stream|string url) U
 */
 PHP_FUNCTION(stream_is_local)
@@ -1398,7 +1565,7 @@ PHP_FUNCTION(stream_is_local)
 		wrapper = stream->wrapper;
 	} else {
 		convert_to_string_ex(zstream);
-		
+
 		wrapper = php_stream_locate_url_wrapper(Z_STRVAL_PP(zstream), NULL, 0 TSRMLS_CC);
 	}
 
@@ -1410,6 +1577,26 @@ PHP_FUNCTION(stream_is_local)
 }
 /* }}} */
 
+/* {{{ proto bool stream_supports_lock(resource stream)
+   Tells wether the stream supports locking through flock(). */
+PHP_FUNCTION(stream_supports_lock)
+{
+	php_stream *stream;
+	zval *zsrc;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zsrc) == FAILURE) {
+		RETURN_FALSE;
+	}
+
+	php_stream_from_zval(stream, &zsrc);
+
+	if (!php_stream_supports_lock(stream)) {
+		RETURN_FALSE;
+	}
+
+	RETURN_TRUE;
+}
+
 #ifdef HAVE_SHUTDOWN
 /* {{{ proto int stream_socket_shutdown(resource stream, int how)
 	causes all or part of a full-duplex connection on the socket associated
@@ -1422,11 +1609,11 @@ PHP_FUNCTION(stream_socket_shutdown)
 	long how;
 	zval *zstream;
 	php_stream *stream;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &zstream, &how) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	if (how != STREAM_SHUT_RD &&
 	    how != STREAM_SHUT_WR &&
 	    how != STREAM_SHUT_RDWR) {
@@ -1437,8 +1624,8 @@ PHP_FUNCTION(stream_socket_shutdown)
 
 	RETURN_BOOL(php_stream_xport_shutdown(stream, (stream_shutdown_t)how TSRMLS_CC) == 0);
 }
-#endif
 /* }}} */
+#endif
 
 /*
  * Local variables:
