@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: user_filters.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -54,7 +54,6 @@ static int le_bucket;
 PHP_FUNCTION(user_filter_nop)
 {
 }
-static
 ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_filter, 0)
 	ZEND_ARG_INFO(0, in)
 	ZEND_ARG_INFO(0, out)
@@ -62,19 +61,17 @@ ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_filter, 0)
 	ZEND_ARG_INFO(0, closing)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_onCreate, 0)
 ZEND_END_ARG_INFO()
 
-static
 ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_onClose, 0)
 ZEND_END_ARG_INFO()
 
-static zend_function_entry user_filter_class_funcs[] = {
+static const zend_function_entry user_filter_class_funcs[] = {
 	PHP_NAMED_FE(filter,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_filter)
 	PHP_NAMED_FE(onCreate,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_onCreate)
 	PHP_NAMED_FE(onClose,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_onClose)
-	{ NULL, NULL, NULL }
+	PHP_FE_END
 };
 
 static zend_class_entry user_filter_class_entry;
@@ -262,7 +259,7 @@ php_stream_filter_status_t userfilter_filter(
 	 * from being destroyed properly */
 	INIT_ZVAL(zpropname);
 	ZVAL_STRINGL(&zpropname, "stream", sizeof("stream")-1, 0);
-	Z_OBJ_HANDLER_P(obj, unset_property)(obj, &zpropname TSRMLS_CC);
+	Z_OBJ_HANDLER_P(obj, unset_property)(obj, &zpropname, 0 TSRMLS_CC);
 
 	zval_ptr_dtor(&zclosing);
 	zval_ptr_dtor(&zconsumed);
@@ -314,7 +311,7 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 			period = wildcard + (period - filtername);
 			while (period) {
 				*period = '\0';
-				strcat(wildcard, ".*");
+				strncat(wildcard, ".*", 2);
 				if (SUCCESS == zend_hash_find(BG(user_filter_map), wildcard, strlen(wildcard) + 1, (void**)&fdat)) {
 					period = NULL;
 				} else {
@@ -352,8 +349,8 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 	/* create the object */
 	ALLOC_ZVAL(obj);
 	object_init_ex(obj, fdat->ce);
-	ZVAL_REFCOUNT(obj) = 1;
-	PZVAL_IS_REF(obj) = 1;
+	Z_SET_REFCOUNT_P(obj, 1);
+	Z_SET_ISREF_P(obj);
 
 	/* filtername */
 	add_property_string(obj, "filtername", (char*)filtername, 1);
@@ -547,12 +544,13 @@ PHP_FUNCTION(stream_bucket_new)
 PHP_FUNCTION(stream_get_filters)
 {
 	char *filter_name;
-	int key_flags, filter_name_len = 0;
+	int key_flags;
+	uint filter_name_len = 0;
 	HashTable *filters_hash;
 	ulong num_key;
 
-	if (ZEND_NUM_ARGS() != 0) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
 	}
 
 	array_init(return_value);
@@ -601,7 +599,7 @@ PHP_FUNCTION(stream_filter_register)
 		zend_hash_init(BG(user_filter_map), 5, NULL, (dtor_func_t) filter_item_dtor, 0);
 	}
 
-	fdat = ecalloc(1, sizeof(*fdat) + classname_len);
+	fdat = ecalloc(1, sizeof(struct php_user_filter_data) + classname_len);
 	memcpy(fdat->classname, classname, classname_len);
 
 	if (zend_hash_add(BG(user_filter_map), filtername, filtername_len + 1, (void*)fdat,
