@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -1191,11 +1191,10 @@ static int php_sybase_finish_results(sybase_result *result TSRMLS_DC)
 		ZVAL_STRINGL(&result, buf, length- 1, 1);       \
 	}
 
-static int php_sybase_fetch_result_row (sybase_result *result, int numrows)
+static int php_sybase_fetch_result_row(sybase_result *result, int numrows TSRMLS_DC)
 {
 	int i, j;
 	CS_INT retcode;
-	TSRMLS_FETCH();
 	
 	/* We've already fetched everything */
 	if (result->last_retcode == CS_END_DATA || result->last_retcode == CS_END_RESULTS) {
@@ -1294,7 +1293,7 @@ static int php_sybase_fetch_result_row (sybase_result *result, int numrows)
 	return retcode;
 }
 
-static sybase_result * php_sybase_fetch_result_set (sybase_link *sybase_ptr, int buffered, int store)
+static sybase_result * php_sybase_fetch_result_set(sybase_link *sybase_ptr, int buffered, int store TSRMLS_DC)
 {
 	int num_fields;
 	sybase_result *result;
@@ -1413,7 +1412,7 @@ static sybase_result * php_sybase_fetch_result_set (sybase_link *sybase_ptr, int
 	if (buffered) {
 		retcode = CS_SUCCEED;
 	} else {
-		if ((retcode = php_sybase_fetch_result_row(result, -1)) == CS_FAIL) {
+		if ((retcode = php_sybase_fetch_result_row(result, -1 TSRMLS_CC)) == CS_FAIL) {
 			return NULL;
 		}
 	}
@@ -1550,7 +1549,7 @@ static void php_sybase_query (INTERNAL_FUNCTION_PARAMETERS, int buffered)
 			case CS_PARAM_RESULT:
 			case CS_ROW_RESULT:
 			case CS_STATUS_RESULT:
-				result = php_sybase_fetch_result_set(sybase_ptr, buffered, store);
+				result = php_sybase_fetch_result_set(sybase_ptr, buffered, store TSRMLS_CC);
 				if (result == NULL) {
 					ct_cancel(NULL, sybase_ptr->cmd, CS_CANCEL_ALL);
 					RETURN_FALSE;
@@ -1576,7 +1575,7 @@ static void php_sybase_query (INTERNAL_FUNCTION_PARAMETERS, int buffered)
 					case CS_PARAM_RESULT:
 					case CS_ROW_RESULT:
 						if (status != Q_RESULT) {
-							result = php_sybase_fetch_result_set(sybase_ptr, buffered, store);
+							result = php_sybase_fetch_result_set(sybase_ptr, buffered, store TSRMLS_CC);
 							if (result == NULL) {
 								ct_cancel(NULL, sybase_ptr->cmd, CS_CANCEL_ALL);
 								sybase_ptr->dead = 1;
@@ -1768,7 +1767,7 @@ PHP_FUNCTION(sybase_fetch_row)
 
 	/* Unbuffered? */
 	if (result->last_retcode != CS_END_DATA && result->last_retcode != CS_END_RESULTS) {
-		php_sybase_fetch_result_row(result, 1);
+		php_sybase_fetch_result_row(result, 1 TSRMLS_CC);
 	}
 	
 	/* At the end? */
@@ -1804,7 +1803,7 @@ static void php_sybase_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int numerics)
 
 	/* Unbuffered ? Fetch next row */
 	if (result->last_retcode != CS_END_DATA && result->last_retcode != CS_END_RESULTS) {
-		php_sybase_fetch_result_row(result, 1);
+		php_sybase_fetch_result_row(result, 1 TSRMLS_CC);
 	}
 
 	/* At the end? */
@@ -1819,11 +1818,7 @@ static void php_sybase_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int numerics)
 		ALLOC_ZVAL(tmp);
 		*tmp = result->data[result->store ? result->cur_row : 0][i];
 		INIT_PZVAL(tmp);
-		if (PG(magic_quotes_runtime) && Z_TYPE_P(tmp) == IS_STRING) {
-			Z_STRVAL_P(tmp) = php_addslashes(Z_STRVAL_P(tmp), Z_STRLEN_P(tmp), &Z_STRLEN_P(tmp), 0 TSRMLS_CC);
-		} else {
-			zval_copy_ctor(tmp);
-		}
+		zval_copy_ctor(tmp);
 		if (numerics) {
 			zend_hash_index_update(Z_ARRVAL_P(return_value), i, (void *) &tmp, sizeof(zval *), NULL);
 			Z_ADDREF_P(tmp);
@@ -1922,7 +1917,7 @@ PHP_FUNCTION(sybase_data_seek)
 
 	/* Unbuffered ? */
 	if (result->last_retcode != CS_END_DATA && result->last_retcode != CS_END_RESULTS && offset >= result->num_rows) {
-		php_sybase_fetch_result_row(result, offset+ 1);
+		php_sybase_fetch_result_row(result, offset+ 1 TSRMLS_CC);
 	}
 	
 	if (offset < 0 || offset >= result->num_rows) {
@@ -2056,7 +2051,7 @@ PHP_FUNCTION(sybase_result)
 	
 	/* Unbuffered ? */
 	if (result->last_retcode != CS_END_DATA && result->last_retcode != CS_END_RESULTS && row >= result->num_rows) {
-		php_sybase_fetch_result_row(result, row);
+		php_sybase_fetch_result_row(result, row TSRMLS_CC);
 	}
 
 	if (row < 0 || row >= result->num_rows) {
