@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2011 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: sysvmsg.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id: 6360183f0075ec66a3dfa01633b2ed808dd482af $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -95,7 +95,7 @@ const zend_function_entry sysvmsg_functions[] = {
 	PHP_FE(msg_stat_queue,				arginfo_msg_stat_queue)
 	PHP_FE(msg_set_queue,				arginfo_msg_set_queue)
 	PHP_FE(msg_queue_exists,			arginfo_msg_queue_exists)
-	{NULL, NULL, NULL}	/* Must be the last line in sysvmsg_functions[] */
+	PHP_FE_END
 };
 /* }}} */
 
@@ -145,7 +145,7 @@ PHP_MINFO_FUNCTION(sysvmsg)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "sysvmsg support", "enabled");
-	php_info_print_table_row(2, "Revision", "$Revision: 306939 $");
+	php_info_print_table_row(2, "Revision", "$Id: 6360183f0075ec66a3dfa01633b2ed808dd482af $");
 	php_info_print_table_end();
 }
 /* }}} */
@@ -271,7 +271,7 @@ PHP_FUNCTION(msg_get_queue)
 			RETURN_FALSE;
 		}
 	}
-	RETVAL_RESOURCE(zend_list_insert(mq, le_sysvmsg));
+	RETVAL_RESOURCE(zend_list_insert(mq, le_sysvmsg TSRMLS_CC));
 }
 /* }}} */
 
