@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2011 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
  
-/* $Id: sysvsem.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id$ */
 
 /* Latest update build anc tested on Linux 2.2.14
  *
@@ -84,7 +84,7 @@ const zend_function_entry sysvsem_functions[] = {
 	PHP_FE(sem_acquire,		arginfo_sem_acquire)
 	PHP_FE(sem_release,		arginfo_sem_release)
 	PHP_FE(sem_remove,		arginfo_sem_remove)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 /* }}} */
 
