@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: userspace.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id$ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -26,6 +26,15 @@
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
+#include <stddef.h>
+
+#if HAVE_UTIME
+# ifdef PHP_WIN32
+#  include <sys/utime.h>
+# else
+#  include <utime.h>
+# endif
+#endif
 
 static int le_protocols;
 
@@ -42,6 +51,7 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char *url_to, int options, php_stream_context *context TSRMLS_DC);
 static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode, int options, php_stream_context *context TSRMLS_DC);
 static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int options, php_stream_context *context TSRMLS_DC);
+static int user_wrapper_metadata(php_stream_wrapper *wrapper, char *url, int option, void *value, php_stream_context *context TSRMLS_DC);
 static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filename, char *mode,
 		int options, char **opened_path, php_stream_context *context STREAMS_DC TSRMLS_DC);
 
@@ -55,7 +65,8 @@ static php_stream_wrapper_ops user_stream_wops = {
 	user_wrapper_unlink,
 	user_wrapper_rename,
 	user_wrapper_mkdir,
-	user_wrapper_rmdir
+	user_wrapper_rmdir,
+	user_wrapper_metadata
 };
 
 
@@ -77,7 +88,6 @@ PHP_MINIT_FUNCTION(user_streams)
 
 	REGISTER_LONG_CONSTANT("STREAM_USE_PATH", 			USE_PATH, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_IGNORE_URL", 		IGNORE_URL, CONST_CS|CONST_PERSISTENT);
-	REGISTER_LONG_CONSTANT("STREAM_ENFORCE_SAFE_MODE",	ENFORCE_SAFE_MODE, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_REPORT_ERRORS", 		REPORT_ERRORS, CONST_CS|CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_MUST_SEEK", 			STREAM_MUST_SEEK, CONST_CS|CONST_PERSISTENT);
 
@@ -86,6 +96,25 @@ PHP_MINIT_FUNCTION(user_streams)
 	REGISTER_LONG_CONSTANT("STREAM_MKDIR_RECURSIVE",	PHP_STREAM_MKDIR_RECURSIVE,		CONST_CS|CONST_PERSISTENT);
 
 	REGISTER_LONG_CONSTANT("STREAM_IS_URL",	PHP_STREAM_IS_URL,		CONST_CS|CONST_PERSISTENT);
+
+	REGISTER_LONG_CONSTANT("STREAM_OPTION_BLOCKING",	PHP_STREAM_OPTION_BLOCKING,		CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_OPTION_READ_TIMEOUT",	PHP_STREAM_OPTION_READ_TIMEOUT,		CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_OPTION_READ_BUFFER",	PHP_STREAM_OPTION_READ_BUFFER,		CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_OPTION_WRITE_BUFFER",	PHP_STREAM_OPTION_WRITE_BUFFER,		CONST_CS|CONST_PERSISTENT);
+
+	REGISTER_LONG_CONSTANT("STREAM_BUFFER_NONE",		PHP_STREAM_BUFFER_NONE,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_BUFFER_LINE",		PHP_STREAM_BUFFER_LINE,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_BUFFER_FULL",		PHP_STREAM_BUFFER_FULL,			CONST_CS|CONST_PERSISTENT);
+
+	REGISTER_LONG_CONSTANT("STREAM_CAST_AS_STREAM",		PHP_STREAM_AS_STDIO,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_CAST_FOR_SELECT",	PHP_STREAM_AS_FD_FOR_SELECT,		CONST_CS|CONST_PERSISTENT);
+
+	REGISTER_LONG_CONSTANT("STREAM_META_TOUCH",			PHP_STREAM_META_TOUCH,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_META_OWNER",			PHP_STREAM_META_OWNER,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_META_OWNER_NAME",	PHP_STREAM_META_OWNER_NAME,		CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_META_GROUP",			PHP_STREAM_META_GROUP,			CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_META_GROUP_NAME",	PHP_STREAM_META_GROUP_NAME,		CONST_CS|CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("STREAM_META_ACCESS",		PHP_STREAM_META_ACCESS,			CONST_CS|CONST_PERSISTENT);
 	return SUCCESS;
 }
 
@@ -115,35 +144,39 @@ typedef struct _php_userstream_data php_userstream_data_t;
 #define USERSTREAM_DIR_REWIND	"dir_rewinddir"
 #define USERSTREAM_DIR_CLOSE	"dir_closedir"
 #define USERSTREAM_LOCK     "stream_lock"
+#define USERSTREAM_CAST		"stream_cast"
+#define USERSTREAM_SET_OPTION	"stream_set_option"
+#define USERSTREAM_TRUNCATE	"stream_truncate"
+#define USERSTREAM_METADATA	"stream_metadata"
 
 /* {{{ class should have methods like these:
- 
+
 	function stream_open($path, $mode, $options, &$opened_path)
 	{
 	  	return true/false;
 	}
-	
+
 	function stream_read($count)
 	{
 	   	return false on error;
 		else return string;
 	}
-	
+
 	function stream_write($data)
 	{
 	   	return false on error;
 		else return count written;
 	}
-	
+
 	function stream_close()
 	{
 	}
-	
+
 	function stream_flush()
 	{
 		return true/false;
 	}
-	
+
 	function stream_seek($offset, $whence)
 	{
 		return true/false;
@@ -164,6 +197,33 @@ typedef struct _php_userstream_data php_userstream_data_t;
 		return array( just like that returned by fstat() );
 	}
 
+	function stream_cast($castas)
+	{
+		if ($castas == STREAM_CAST_FOR_SELECT) {
+			return $this->underlying_stream;
+		}
+		return false;
+	}
+
+	function stream_set_option($option, $arg1, $arg2)
+	{
+		switch($option) {
+		case STREAM_OPTION_BLOCKING:
+			$blocking = $arg1;
+			...
+		case STREAM_OPTION_READ_TIMEOUT:
+			$sec = $arg1;
+			$usec = $arg2;
+			...
+		case STREAM_OPTION_WRITE_BUFFER:
+			$mode = $arg1;
+			$size = $arg2;
+			...
+		default:
+			return false;
+		}
+	}
+
 	function url_stat(string $url, int $flags)
 	{
 		return array( just like that returned by stat() );
@@ -208,18 +268,78 @@ typedef struct _php_userstream_data php_userstream_data_t;
 	{
 		reset to start of dir list;
 	}
-  
+
+	function stream_lock($operation)
+	{
+		return true / false;
+	}
+
+ 	function stream_truncate($new_size)
+	{
+		return true / false;
+	}
+
 	}}} **/
 
+static zval *user_stream_create_object(struct php_user_stream_wrapper *uwrap, php_stream_context *context TSRMLS_DC)
+{
+	zval *object;
+	/* create an instance of our class */
+	ALLOC_ZVAL(object);
+	object_init_ex(object, uwrap->ce);
+	Z_SET_REFCOUNT_P(object, 1);
+	Z_SET_ISREF_P(object);
+
+	if (context) {
+		add_property_resource(object, "context", context->rsrc_id);
+		zend_list_addref(context->rsrc_id);
+	} else {
+		add_property_null(object, "context");
+	}
+
+	if (uwrap->ce->constructor) {
+		zend_fcall_info fci;
+		zend_fcall_info_cache fcc;
+		zval *retval_ptr;
+
+		fci.size = sizeof(fci);
+		fci.function_table = &uwrap->ce->function_table;
+		fci.function_name = NULL;
+		fci.symbol_table = NULL;
+		fci.object_ptr = object;
+		fci.retval_ptr_ptr = &retval_ptr;
+		fci.param_count = 0;
+		fci.params = NULL;
+		fci.no_separation = 1;
+
+		fcc.initialized = 1;
+		fcc.function_handler = uwrap->ce->constructor;
+		fcc.calling_scope = EG(scope);
+		fcc.called_scope = Z_OBJCE_P(object);
+		fcc.object_ptr = object;
+
+		if (zend_call_function(&fci, &fcc TSRMLS_CC) == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not execute %s::%s()", uwrap->ce->name, uwrap->ce->constructor->common.function_name);
+			zval_dtor(object);
+			FREE_ZVAL(object);
+			return NULL;
+		} else {
+			if (retval_ptr) {
+				zval_ptr_dtor(&retval_ptr);
+			}
+		}
+	}
+	return object;
+}
+
 static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filename, char *mode, int options, char **opened_path, php_stream_context *context STREAMS_DC TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
 	php_userstream_data_t *us;
 	zval *zfilename, *zmode, *zopened, *zoptions, *zretval = NULL, *zfuncname;
-	zval **args[4];	
+	zval **args[4];
 	int call_result;
 	php_stream *stream = NULL;
-	zval *zcontext = NULL;
 	zend_bool old_in_user_include;
 
 	/* Try to catch bad usage without preventing flexibility */
@@ -228,73 +348,29 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 		return NULL;
 	}
 	FG(user_stream_current_filename) = filename;
-	
+
 	/* if the user stream was registered as local and we are in include context,
 		we add allow_url_include restrictions to allow_url_fopen ones */
 	/* we need only is_url == 0 here since if is_url == 1 and remote wrappers
 		were restricted we wouldn't get here */
 	old_in_user_include = PG(in_user_include);
-	if(uwrap->wrapper.is_url == 0 && 
+	if(uwrap->wrapper.is_url == 0 &&
 		(options & STREAM_OPEN_FOR_INCLUDE) &&
 		!PG(allow_url_include)) {
 		PG(in_user_include) = 1;
 	}
 
 	us = emalloc(sizeof(*us));
-	us->wrapper = uwrap;	
+	us->wrapper = uwrap;
 
-	/* create an instance of our class */
-	ALLOC_ZVAL(us->object);
-	object_init_ex(us->object, uwrap->ce);
-	ZVAL_REFCOUNT(us->object) = 1;
-	PZVAL_IS_REF(us->object) = 1;
-	
-	if (uwrap->ce->constructor) {
-		zend_fcall_info fci;
-		zend_fcall_info_cache fcc;
-		zval *retval_ptr;
-		
-		fci.size = sizeof(fci);
-		fci.function_table = &uwrap->ce->function_table;
-		fci.function_name = NULL;
-		fci.symbol_table = NULL;
-		fci.object_pp = &us->object;
-		fci.retval_ptr_ptr = &retval_ptr;
-		fci.param_count = 0;
-		fci.params = NULL;
-		fci.no_separation = 1;
-		
-		fcc.initialized = 1;
-		fcc.function_handler = uwrap->ce->constructor;
-		fcc.calling_scope = EG(scope);
-		fcc.object_pp = &us->object;
-
-		if (zend_call_function(&fci, &fcc TSRMLS_CC) == FAILURE) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Could not execute %s::%s()", uwrap->ce->name, uwrap->ce->constructor->common.function_name);
-			zval_dtor(us->object);
-			FREE_ZVAL(us->object);
-			efree(us);
-			FG(user_stream_current_filename) = NULL;
-			PG(in_user_include) = old_in_user_include;
-			return NULL;
-		} else {
-			if (retval_ptr) {
-				zval_ptr_dtor(&retval_ptr);
-			}
-		}
+	us->object = user_stream_create_object(uwrap, context TSRMLS_CC);
+	if(us->object == NULL) {
+		FG(user_stream_current_filename) = NULL;
+		PG(in_user_include) = old_in_user_include;
+		efree(us);
+		return NULL;
 	}
 
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(us->object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(us->object, "context");
-	}
-	
 	/* call it's stream_open method - set up params first */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, filename, 1);
@@ -309,21 +385,21 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	args[2] = &zoptions;
 
 	MAKE_STD_ZVAL(zopened);
-	ZVAL_REFCOUNT(zopened) = 1;
-	PZVAL_IS_REF(zopened) = 1;
+	Z_SET_REFCOUNT_P(zopened, 1);
+	Z_SET_ISREF_P(zopened);
 	ZVAL_NULL(zopened);
 	args[3] = &zopened;
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_OPEN, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&us->object,
 			zfuncname,
 			&zretval,
 			4, args,
 			0, NULL	TSRMLS_CC);
-	
+
 	if (call_result == SUCCESS && zretval != NULL && zval_is_true(zretval)) {
 		/* the stream is now open! */
 		stream = php_stream_alloc_rel(&php_stream_userspace_ops, us, 0, mode);
@@ -340,7 +416,7 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 		php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "\"%s::" USERSTREAM_OPEN "\" call failed",
 			us->wrapper->classname);
 	}
-	
+
 	/* destroy everything else */
 	if (stream == NULL) {
 		zval_ptr_dtor(&us->object);
@@ -348,7 +424,7 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	}
 	if (zretval)
 		zval_ptr_dtor(&zretval);
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zopened);
 	zval_ptr_dtor(&zoptions);
@@ -356,7 +432,7 @@ static php_stream *user_wrapper_opener(php_stream_wrapper *wrapper, char *filena
 	zval_ptr_dtor(&zfilename);
 
 	FG(user_stream_current_filename) = NULL;
-		
+
 	PG(in_user_include) = old_in_user_include;
 	return stream;
 }
@@ -366,8 +442,8 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
 	php_userstream_data_t *us;
-	zval *zfilename, *zoptions, *zretval = NULL, *zfuncname, *zcontext;
-	zval **args[2];	
+	zval *zfilename, *zoptions, *zretval = NULL, *zfuncname;
+	zval **args[2];
 	int call_result;
 	php_stream *stream = NULL;
 
@@ -377,27 +453,17 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 		return NULL;
 	}
 	FG(user_stream_current_filename) = filename;
-	
+
 	us = emalloc(sizeof(*us));
-	us->wrapper = uwrap;	
+	us->wrapper = uwrap;
 
-	/* create an instance of our class */
-	ALLOC_ZVAL(us->object);
-	object_init_ex(us->object, uwrap->ce);
-	ZVAL_REFCOUNT(us->object) = 1;
-	PZVAL_IS_REF(us->object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(us->object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(us->object, "context");
+	us->object = user_stream_create_object(uwrap, context TSRMLS_CC);
+	if(us == NULL) {
+		FG(user_stream_current_filename) = NULL;
+		efree(us);
+		return NULL;
 	}
-	
+
 	/* call it's dir_open method - set up params first */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, filename, 1);
@@ -409,14 +475,14 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_DIR_OPEN, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&us->object,
 			zfuncname,
 			&zretval,
 			2, args,
 			0, NULL	TSRMLS_CC);
-	
+
 	if (call_result == SUCCESS && zretval != NULL && zval_is_true(zretval)) {
 		/* the stream is now open! */
 		stream = php_stream_alloc_rel(&php_stream_userspace_dir_ops, us, 0, mode);
@@ -428,7 +494,7 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 		php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "\"%s::" USERSTREAM_DIR_OPEN "\" call failed",
 			us->wrapper->classname);
 	}
-	
+
 	/* destroy everything else */
 	if (stream == NULL) {
 		zval_ptr_dtor(&us->object);
@@ -436,13 +502,13 @@ static php_stream *user_wrapper_opendir(php_stream_wrapper *wrapper, char *filen
 	}
 	if (zretval)
 		zval_ptr_dtor(&zretval);
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zoptions);
 	zval_ptr_dtor(&zfilename);
 
 	FG(user_stream_current_filename) = NULL;
-		
+
 	return stream;
 }
 
@@ -456,11 +522,11 @@ PHP_FUNCTION(stream_wrapper_register)
 	struct php_user_stream_wrapper * uwrap;
 	int rsrc_id;
 	long flags = 0;
-	
+
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|l", &protocol, &protocol_len, &classname, &classname_len, &flags) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	uwrap = (struct php_user_stream_wrapper *)ecalloc(1, sizeof(*uwrap));
 	uwrap->protoname = estrndup(protocol, protocol_len);
 	uwrap->classname = estrndup(classname, classname_len);
@@ -546,7 +612,7 @@ PHP_FUNCTION(stream_wrapper_restore)
 	if (php_register_url_stream_wrapper_volatile(protocol, wrapper TSRMLS_CC) == FAILURE) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to restore original %s:// wrapper", protocol);
 		RETURN_FALSE;
-	}	
+	}
 
 	RETURN_TRUE;
 }
@@ -594,10 +660,10 @@ static size_t php_userstreamop_write(php_stream *stream, const char *buf, size_t
 				(long)(didwrite - count), (long)didwrite, (long)count);
 		didwrite = count;
 	}
-	
+
 	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	return didwrite;
 }
 
@@ -682,9 +748,9 @@ static int php_userstreamop_close(php_stream *stream, int close_handle TSRMLS_DC
 	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
 
 	assert(us != NULL);
-	
+
 	ZVAL_STRINGL(&func_name, USERSTREAM_CLOSE, sizeof(USERSTREAM_CLOSE)-1, 0);
-	
+
 	call_user_function_ex(NULL,
 			&us->object,
 			&func_name,
@@ -693,11 +759,11 @@ static int php_userstreamop_close(php_stream *stream, int close_handle TSRMLS_DC
 
 	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	zval_ptr_dtor(&us->object);
 
 	efree(us);
-	
+
 	return 0;
 }
 
@@ -711,7 +777,7 @@ static int php_userstreamop_flush(php_stream *stream TSRMLS_DC)
 	assert(us != NULL);
 
 	ZVAL_STRINGL(&func_name, USERSTREAM_FLUSH, sizeof(USERSTREAM_FLUSH)-1, 0);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&us->object,
 			&func_name,
@@ -722,10 +788,10 @@ static int php_userstreamop_flush(php_stream *stream TSRMLS_DC)
 		call_result = 0;
 	else
 		call_result = -1;
-	
+
 	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	return call_result;
 }
 
@@ -764,10 +830,10 @@ static int php_userstreamop_seek(php_stream *stream, off_t offset, int whence, o
 		/* stream_seek is not implemented, so disable seeks for this stream */
 		stream->flags |= PHP_STREAM_FLAG_NO_SEEK;
 		/* there should be no retval to clean up */
-		
-		if (retval) 
+
+		if (retval)
 			zval_ptr_dtor(&retval);
-		
+
 		return -1;
 	} else if (call_result == SUCCESS && retval != NULL && zval_is_true(retval)) {
 		ret = 0;
@@ -817,12 +883,14 @@ static int statbuf_from_array(zval *array, php_stream_statbuf *ssb TSRMLS_DC)
 
 #define STAT_PROP_ENTRY_EX(name, name2)                        \
 	if (SUCCESS == zend_hash_find(Z_ARRVAL_P(array), #name, sizeof(#name), (void**)&elem)) {     \
+		SEPARATE_ZVAL(elem);																	 \
 		convert_to_long(*elem);                                                                   \
 		ssb->sb.st_##name2 = Z_LVAL_PP(elem);                                                      \
 	}
 
 #define STAT_PROP_ENTRY(name) STAT_PROP_ENTRY_EX(name,name)
 
+	memset(ssb, 0, sizeof(php_stream_statbuf));
 	STAT_PROP_ENTRY(dev);
 	STAT_PROP_ENTRY(ino);
 	STAT_PROP_ENTRY(mode);
@@ -849,8 +917,8 @@ static int statbuf_from_array(zval *array, php_stream_statbuf *ssb TSRMLS_DC)
 	STAT_PROP_ENTRY(blocks);
 #endif
 
-#undef STAT_PROP_ENTRY	
-#undef STAT_PROP_ENTRY_EX	
+#undef STAT_PROP_ENTRY
+#undef STAT_PROP_ENTRY_EX
 	return SUCCESS;
 }
 
@@ -880,9 +948,9 @@ static int php_userstreamop_stat(php_stream *stream, php_stream_statbuf *ssb TSR
 		}
 	}
 
-	if (retval) 
+	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	return ret;
 }
 
@@ -892,9 +960,9 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 	zval *retval = NULL;
 	int call_result;
 	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
-	int ret = -1;
+	int ret = PHP_STREAM_OPTION_RETURN_NOTIMPL;
 	zval *zvalue = NULL;
-	zval **args[1];
+	zval **args[3];
 
 	switch (option) {
 	case PHP_STREAM_OPTION_CHECK_LIVENESS:
@@ -930,36 +998,150 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 		}
 
 		args[0] = &zvalue;
-		
+
 		/* TODO wouldblock */
 		ZVAL_STRINGL(&func_name, USERSTREAM_LOCK, sizeof(USERSTREAM_LOCK)-1, 0);
-		
+
 		call_result = call_user_function_ex(NULL,
 											&us->object,
 											&func_name,
 											&retval,
 											1, args, 0, NULL TSRMLS_CC);
-		
+
 		if (call_result == SUCCESS && retval != NULL && Z_TYPE_P(retval) == IS_BOOL) {
 			ret = !Z_LVAL_P(retval);
 		} else if (call_result == FAILURE) {
-			if (value == 0) { 
+			if (value == 0) {
 			   	/* lock support test (TODO: more check) */
-				ret = 0;
+				ret = PHP_STREAM_OPTION_RETURN_OK;
 			} else {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_LOCK " is not implemented!", 
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_LOCK " is not implemented!",
 								 us->wrapper->classname);
+				ret = PHP_STREAM_OPTION_RETURN_ERR;
 			}
 		}
 
 		break;
+
+	case PHP_STREAM_OPTION_TRUNCATE_API:
+		ZVAL_STRINGL(&func_name, USERSTREAM_TRUNCATE, sizeof(USERSTREAM_TRUNCATE)-1, 0);
+
+		switch (value) {
+		case PHP_STREAM_TRUNCATE_SUPPORTED:
+			if (zend_is_callable_ex(&func_name, us->object, IS_CALLABLE_CHECK_SILENT,
+					NULL, NULL, NULL, NULL TSRMLS_CC))
+				ret = PHP_STREAM_OPTION_RETURN_OK;
+			else
+				ret = PHP_STREAM_OPTION_RETURN_ERR;
+			break;
+
+		case PHP_STREAM_TRUNCATE_SET_SIZE: {
+			ptrdiff_t new_size = *(ptrdiff_t*) ptrparam;
+			if (new_size >= 0 && new_size <= (ptrdiff_t)LONG_MAX) {
+				MAKE_STD_ZVAL(zvalue);
+				ZVAL_LONG(zvalue, (long)new_size);
+				args[0] = &zvalue;
+				call_result = call_user_function_ex(NULL,
+													&us->object,
+													&func_name,
+													&retval,
+													1, args, 0, NULL TSRMLS_CC);
+				if (call_result == SUCCESS && retval != NULL) {
+					if (Z_TYPE_P(retval) == IS_BOOL) {
+						ret = Z_LVAL_P(retval) ? PHP_STREAM_OPTION_RETURN_OK :
+												 PHP_STREAM_OPTION_RETURN_ERR;
+					} else {
+						php_error_docref(NULL TSRMLS_CC, E_WARNING,
+								"%s::" USERSTREAM_TRUNCATE " did not return a boolean!",
+								us->wrapper->classname);
+					}
+				} else {
+					php_error_docref(NULL TSRMLS_CC, E_WARNING,
+							"%s::" USERSTREAM_TRUNCATE " is not implemented!",
+							us->wrapper->classname);
+				}
+			} else { /* bad new size */
+				ret = PHP_STREAM_OPTION_RETURN_ERR;
+			}
+			break;
+		}
+		}
+		break;
+
+	case PHP_STREAM_OPTION_READ_BUFFER:
+	case PHP_STREAM_OPTION_WRITE_BUFFER:
+	case PHP_STREAM_OPTION_READ_TIMEOUT:
+	case PHP_STREAM_OPTION_BLOCKING: {
+		zval *zoption = NULL;
+		zval *zptrparam = NULL;
+
+		ZVAL_STRINGL(&func_name, USERSTREAM_SET_OPTION, sizeof(USERSTREAM_SET_OPTION)-1, 0);
+
+		ALLOC_INIT_ZVAL(zoption);
+		ZVAL_LONG(zoption, option);
+
+		ALLOC_INIT_ZVAL(zvalue);
+		ALLOC_INIT_ZVAL(zptrparam);
+
+		args[0] = &zoption;
+		args[1] = &zvalue;
+		args[2] = &zptrparam;
+
+		switch(option) {
+		case PHP_STREAM_OPTION_READ_BUFFER:
+		case PHP_STREAM_OPTION_WRITE_BUFFER:
+			ZVAL_LONG(zvalue, value);
+			if (ptrparam) {
+				ZVAL_LONG(zptrparam, *(long *)ptrparam);
+			} else {
+				ZVAL_LONG(zptrparam, BUFSIZ);
+			}
+			break;
+		case PHP_STREAM_OPTION_READ_TIMEOUT: {
+			struct timeval tv = *(struct timeval*)ptrparam;
+			ZVAL_LONG(zvalue, tv.tv_sec);
+			ZVAL_LONG(zptrparam, tv.tv_usec);
+			break;
+			}
+		case PHP_STREAM_OPTION_BLOCKING:
+			ZVAL_LONG(zvalue, value);
+			break;
+		default:
+			break;
+		}
+
+		call_result = call_user_function_ex(NULL,
+			&us->object,
+			&func_name,
+			&retval,
+			3, args, 0, NULL TSRMLS_CC);
+
+		if (call_result == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_SET_OPTION " is not implemented!",
+					us->wrapper->classname);
+			ret = PHP_STREAM_OPTION_RETURN_ERR;
+		} else if (retval && zend_is_true(retval)) {
+			ret = PHP_STREAM_OPTION_RETURN_OK;
+		} else {
+			ret = PHP_STREAM_OPTION_RETURN_ERR;
+		}
+
+		if (zoption) {
+			zval_ptr_dtor(&zoption);
+		}
+		if (zptrparam) {
+			zval_ptr_dtor(&zptrparam);
+		}
+
+		break;
+		}
 	}
 
 	/* clean up */
 	if (retval) {
 		zval_ptr_dtor(&retval);
 	}
-  
+
 
 	if (zvalue) {
 		zval_ptr_dtor(&zvalue);
@@ -972,27 +1154,16 @@ static int php_userstreamop_set_option(php_stream *stream, int option, int value
 static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zfuncname, *zretval, *zcontext;
+	zval *zfilename, *zfuncname, *zretval;
 	zval **args[1];
 	int call_result;
 	zval *object;
 	int ret = 0;
 
 	/* create an instance of our class */
-	ALLOC_ZVAL(object);
-	object_init_ex(object, uwrap->ce);
-	ZVAL_REFCOUNT(object) = 1;
-	PZVAL_IS_REF(object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(object, "context");
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);
+	if(object == NULL) {
+		return ret;
 	}
 
 	/* call the unlink method */
@@ -1002,7 +1173,7 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_UNLINK, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&object,
 			zfuncname,
@@ -1020,7 +1191,7 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 	zval_ptr_dtor(&object);
 	if (zretval)
 		zval_ptr_dtor(&zretval);
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zfilename);
 
@@ -1030,27 +1201,16 @@ static int user_wrapper_unlink(php_stream_wrapper *wrapper, char *url, int optio
 static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char *url_to, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zold_name, *znew_name, *zfuncname, *zretval, *zcontext;
+	zval *zold_name, *znew_name, *zfuncname, *zretval;
 	zval **args[2];
 	int call_result;
 	zval *object;
 	int ret = 0;
 
 	/* create an instance of our class */
-	ALLOC_ZVAL(object);
-	object_init_ex(object, uwrap->ce);
-	ZVAL_REFCOUNT(object) = 1;
-	PZVAL_IS_REF(object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(object, "context");
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);	
+	if(object == NULL) {
+		return ret;
 	}
 
 	/* call the rename method */
@@ -1064,7 +1224,7 @@ static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_RENAME, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&object,
 			zfuncname,
@@ -1082,7 +1242,7 @@ static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char
 	zval_ptr_dtor(&object);
 	if (zretval)
 		zval_ptr_dtor(&zretval);
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zold_name);
 	zval_ptr_dtor(&znew_name);
@@ -1093,30 +1253,19 @@ static int user_wrapper_rename(php_stream_wrapper *wrapper, char *url_from, char
 static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zmode, *zoptions, *zfuncname, *zretval, *zcontext;
+	zval *zfilename, *zmode, *zoptions, *zfuncname, *zretval;
 	zval **args[3];
 	int call_result;
 	zval *object;
 	int ret = 0;
 
 	/* create an instance of our class */
-	ALLOC_ZVAL(object);
-	object_init_ex(object, uwrap->ce);
-	ZVAL_REFCOUNT(object) = 1;
-	PZVAL_IS_REF(object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(object, "context");
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);	
+	if(object == NULL) {
+		return ret;
 	}
 
-	/* call the unlink method */
+	/* call the mkdir method */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1131,7 +1280,7 @@ static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode,
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_MKDIR, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&object,
 			zfuncname,
@@ -1150,7 +1299,7 @@ static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode,
 	if (zretval) {
 		zval_ptr_dtor(&zretval);
 	}
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zfilename);
 	zval_ptr_dtor(&zmode);
@@ -1162,30 +1311,19 @@ static int user_wrapper_mkdir(php_stream_wrapper *wrapper, char *url, int mode,
 static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int options, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zoptions, *zfuncname, *zretval, *zcontext;
+	zval *zfilename, *zoptions, *zfuncname, *zretval;
 	zval **args[3];
 	int call_result;
 	zval *object;
 	int ret = 0;
 
 	/* create an instance of our class */
-	ALLOC_ZVAL(object);
-	object_init_ex(object, uwrap->ce);
-	ZVAL_REFCOUNT(object) = 1;
-	PZVAL_IS_REF(object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(object, "context");
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);	
+	if(object == NULL) {
+		return ret;
 	}
 
-	/* call the unlink method */
+	/* call the rmdir method */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1196,7 +1334,7 @@ static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int option
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_RMDIR, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&object,
 			zfuncname,
@@ -1215,7 +1353,7 @@ static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int option
 	if (zretval) {
 		zval_ptr_dtor(&zretval);
 	}
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zfilename);
 	zval_ptr_dtor(&zoptions);
@@ -1223,35 +1361,105 @@ static int user_wrapper_rmdir(php_stream_wrapper *wrapper, char *url, int option
 	return ret;
 }
 
+static int user_wrapper_metadata(php_stream_wrapper *wrapper, char *url, int option, void *value, php_stream_context *context TSRMLS_DC)
+{
+	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
+	zval *zfilename, *zoption, *zvalue, *zfuncname, *zretval;
+	zval **args[3];
+	int call_result;
+	zval *object;
+	int ret = 0;
+
+	MAKE_STD_ZVAL(zvalue);
+	switch(option) {
+		case PHP_STREAM_META_TOUCH:
+			array_init(zvalue);
+			if(value) {
+				struct utimbuf *newtime = (struct utimbuf *)value;
+				add_index_long(zvalue, 0, newtime->modtime);
+				add_index_long(zvalue, 1, newtime->actime);
+			}
+			break;
+		case PHP_STREAM_META_GROUP:
+		case PHP_STREAM_META_OWNER:
+		case PHP_STREAM_META_ACCESS:
+			ZVAL_LONG(zvalue, *(long *)value);
+			break;
+		case PHP_STREAM_META_GROUP_NAME:
+		case PHP_STREAM_META_OWNER_NAME:
+			ZVAL_STRING(zvalue, value, 1);
+			break;
+		default:
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown option %d for " USERSTREAM_METADATA, option);
+			zval_ptr_dtor(&zvalue);
+			return ret;
+	}
+
+	/* create an instance of our class */
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);	
+	if(object == NULL) {
+		zval_ptr_dtor(&zvalue);
+		return ret;
+	}
+
+	/* call the mkdir method */
+	MAKE_STD_ZVAL(zfilename);
+	ZVAL_STRING(zfilename, url, 1);
+	args[0] = &zfilename;
+
+	MAKE_STD_ZVAL(zoption);
+	ZVAL_LONG(zoption, option);
+	args[1] = &zoption;
+
+	args[2] = &zvalue;
+
+	MAKE_STD_ZVAL(zfuncname);
+	ZVAL_STRING(zfuncname, USERSTREAM_METADATA, 1);
+
+	call_result = call_user_function_ex(NULL,
+			&object,
+			zfuncname,
+			&zretval,
+			3, args,
+			0, NULL	TSRMLS_CC);
+
+	if (call_result == SUCCESS && zretval && Z_TYPE_P(zretval) == IS_BOOL) {
+		ret = Z_LVAL_P(zretval);
+	} else if (call_result == FAILURE) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_METADATA " is not implemented!", uwrap->classname);
+ 	}
+
+	/* clean up */
+	zval_ptr_dtor(&object);
+	if (zretval) {
+		zval_ptr_dtor(&zretval);
+	}
+
+	zval_ptr_dtor(&zfuncname);
+	zval_ptr_dtor(&zfilename);
+	zval_ptr_dtor(&zoption);
+	zval_ptr_dtor(&zvalue);
+
+	return ret;
+}
+
+
 static int user_wrapper_stat_url(php_stream_wrapper *wrapper, char *url, int flags, php_stream_statbuf *ssb, php_stream_context *context TSRMLS_DC)
 {
 	struct php_user_stream_wrapper *uwrap = (struct php_user_stream_wrapper*)wrapper->abstract;
-	zval *zfilename, *zfuncname, *zretval, *zflags, *zcontext;
-	zval **args[2];	
+	zval *zfilename, *zfuncname, *zretval, *zflags;
+	zval **args[2];
 	int call_result;
 	zval *object;
 	int ret = -1;
 
 	/* create an instance of our class */
-	ALLOC_ZVAL(object);
-	object_init_ex(object, uwrap->ce);
-	ZVAL_REFCOUNT(object) = 1;
-	PZVAL_IS_REF(object) = 1;
-
-	if (context) {
-		MAKE_STD_ZVAL(zcontext);
-		php_stream_context_to_zval(context, zcontext);
-		add_property_zval(object, "context", zcontext);
-		/* The object property should be the only reference,
-		   'get rid' of our local reference. */
-		zval_ptr_dtor(&zcontext);
-	} else {
-		add_property_null(object, "context");
+	object = user_stream_create_object(uwrap, context TSRMLS_CC);	
+	if(object == NULL) {
+		return ret;
 	}
 
-	/* call the stat_url method */
-	
-	/* call it's stream_open method - set up params first */
+	/* call it's stat_url method - set up params first */
 	MAKE_STD_ZVAL(zfilename);
 	ZVAL_STRING(zfilename, url, 1);
 	args[0] = &zfilename;
@@ -1262,14 +1470,14 @@ static int user_wrapper_stat_url(php_stream_wrapper *wrapper, char *url, int fla
 
 	MAKE_STD_ZVAL(zfuncname);
 	ZVAL_STRING(zfuncname, USERSTREAM_STATURL, 1);
-	
+
 	call_result = call_user_function_ex(NULL,
 			&object,
 			zfuncname,
 			&zretval,
 			2, args,
 			0, NULL	TSRMLS_CC);
-	
+
 	if (call_result == SUCCESS && zretval != NULL && Z_TYPE_P(zretval) == IS_ARRAY) {
 		/* We got the info we needed */
 		if (SUCCESS == statbuf_from_array(zretval, ssb TSRMLS_CC))
@@ -1280,16 +1488,16 @@ static int user_wrapper_stat_url(php_stream_wrapper *wrapper, char *url, int fla
 					uwrap->classname);
 		}
 	}
-	
+
 	/* clean up */
 	zval_ptr_dtor(&object);
 	if (zretval)
 		zval_ptr_dtor(&zretval);
-	
+
 	zval_ptr_dtor(&zfuncname);
 	zval_ptr_dtor(&zfilename);
 	zval_ptr_dtor(&zflags);
-		
+
 	return ret;
 
 }
@@ -1339,9 +1547,9 @@ static int php_userstreamop_closedir(php_stream *stream, int close_handle TSRMLS
 	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
 
 	assert(us != NULL);
-	
+
 	ZVAL_STRINGL(&func_name, USERSTREAM_DIR_CLOSE, sizeof(USERSTREAM_DIR_CLOSE)-1, 0);
-	
+
 	call_user_function_ex(NULL,
 			&us->object,
 			&func_name,
@@ -1350,11 +1558,11 @@ static int php_userstreamop_closedir(php_stream *stream, int close_handle TSRMLS
 
 	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	zval_ptr_dtor(&us->object);
 
 	efree(us);
-	
+
 	return 0;
 }
 
@@ -1365,7 +1573,7 @@ static int php_userstreamop_rewinddir(php_stream *stream, off_t offset, int when
 	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
 
 	ZVAL_STRINGL(&func_name, USERSTREAM_DIR_REWIND, sizeof(USERSTREAM_DIR_REWIND)-1, 0);
-	
+
 	call_user_function_ex(NULL,
 			&us->object,
 			&func_name,
@@ -1374,18 +1582,82 @@ static int php_userstreamop_rewinddir(php_stream *stream, off_t offset, int when
 
 	if (retval)
 		zval_ptr_dtor(&retval);
-	
+
 	return 0;
 
 }
 
+static int php_userstreamop_cast(php_stream *stream, int castas, void **retptr TSRMLS_DC)
+{
+	php_userstream_data_t *us = (php_userstream_data_t *)stream->abstract;
+	zval func_name;
+	zval *retval = NULL;
+	zval *zcastas = NULL;
+	zval **args[1];
+	php_stream * intstream = NULL;
+	int call_result;
+	int ret = FAILURE;
+
+	ZVAL_STRINGL(&func_name, USERSTREAM_CAST, sizeof(USERSTREAM_CAST)-1, 0);
+
+	ALLOC_INIT_ZVAL(zcastas);
+	switch(castas) {
+	case PHP_STREAM_AS_FD_FOR_SELECT:
+		ZVAL_LONG(zcastas, PHP_STREAM_AS_FD_FOR_SELECT);
+		break;
+	default:
+		ZVAL_LONG(zcastas, PHP_STREAM_AS_STDIO);
+		break;
+	}
+	args[0] = &zcastas;
+
+	call_result = call_user_function_ex(NULL,
+			&us->object,
+			&func_name,
+			&retval,
+			1, args, 0, NULL TSRMLS_CC);
+
+	do {
+		if (call_result == FAILURE) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " is not implemented!",
+					us->wrapper->classname);
+			break;
+		}
+		if (retval == NULL || !zend_is_true(retval)) {
+			break;
+		}
+		php_stream_from_zval_no_verify(intstream, &retval);
+		if (!intstream) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " must return a stream resource",
+					us->wrapper->classname);
+			break;
+		}
+		if (intstream == stream) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s::" USERSTREAM_CAST " must not return itself",
+					us->wrapper->classname);
+			intstream = NULL;
+			break;
+		}
+		ret = php_stream_cast(intstream, castas, retptr, 1);
+	} while (0);
+
+	if (retval) {
+		zval_ptr_dtor(&retval);
+	}
+	if (zcastas) {
+		zval_ptr_dtor(&zcastas);
+	}
+
+	return ret;
+}
+
 php_stream_ops php_stream_userspace_ops = {
 	php_userstreamop_write, php_userstreamop_read,
 	php_userstreamop_close, php_userstreamop_flush,
 	"user-space",
 	php_userstreamop_seek,
-	NULL, /* cast */
-	php_userstreamop_stat, 
+	php_userstreamop_cast,
+	php_userstreamop_stat,
 	php_userstreamop_set_option,
 };
 
