@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2013 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -400,10 +400,6 @@ static int php_sockop_set_option(php_stream *stream, int option, int value, void
 				}
 #endif
 				
-				case PHP_STREAM_OPTION_WRITE_BUFFER:
-					php_stream_set_chunk_size(stream, (ptrparam ? *(size_t *)ptrparam : PHP_SOCK_CHUNK_SIZE));
-					return PHP_STREAM_OPTION_RETURN_OK;
-
 				default:
 					return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 			}
