@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2013 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -110,7 +110,7 @@ PHP_MSHUTDOWN_FUNCTION(continuity)
 PHP_MINFO_FUNCTION(continuity)
 {
         php_info_print_table_start();
-        php_info_print_table_row(2, "Continuity Module Revision", "$Id: 38f4cba693f79d7f3814e872b1f1c73b19ce64cc $");
+        php_info_print_table_row(2, "Continuity Module Revision", "$Id: 96c39230afa6e73940a9011414341fe43d7566d9 $");
         php_info_print_table_row(2, "Server Version", conFget_build());
 #ifdef CONTINUITY_CDPEXT
 	php_info_print_table_row(2,"CDP Extensions", "enabled");
@@ -343,9 +343,8 @@ static void sapi_capi_register_server_variables(zval * track_vars_array TSRMLS_D
 
 }
 
-static void capi_log_message(char *message)
+static void capi_log_message(char *message TSRMLS_DC)
 {
-   TSRMLS_FETCH();
    capi_request_context *rc = (capi_request_context *) SG(server_context);
    logFmsg(0, "mod/php: %s", message);
 }
