@@ -540,17 +540,12 @@ static char *fpm_conf_set_array(zval *key, zval *value, void **config, int conve
 	kv->key = strdup(Z_STRVAL_P(key));
 
 	if (!kv->key) {
-		free(kv);
 		return "fpm_conf_set_array: strdup(key) failed";
 	}
 
 	if (convert_to_bool) {
 		char *err = fpm_conf_set_boolean(value, &subconf, 0);
-		if (err) {
-			free(kv->key);
-			free(kv);
-			return err;
-		}
+		if (err) return err;
 		kv->value = strdup(b ? "1" : "0");
 	} else {
 		kv->value = strdup(Z_STRVAL_P(value));
@@ -561,7 +556,6 @@ static char *fpm_conf_set_array(zval *key, zval *value, void **config, int conve
 
 	if (!kv->value) {
 		free(kv->key);
-		free(kv);
 		return "fpm_conf_set_array: strdup(value) failed";
 	}
 
@@ -584,7 +578,6 @@ static void *fpm_worker_pool_config_alloc() /* {{{ */
 	wp->config = malloc(sizeof(struct fpm_worker_pool_config_s));
 
 	if (!wp->config) { 
-		fpm_worker_pool_free(wp);
 		return 0;
 	}
 
@@ -1009,7 +1002,7 @@ static int fpm_conf_process_all_pools() /* {{{ */
 			nb_ext = 0;
 
 			/* find the number of extensions */
-			while (strtok(limit_extensions, " \t")) {
+			while ((ext = strtok(limit_extensions, " \t"))) {
 				limit_extensions = NULL;
 				nb_ext++;
 			}
@@ -1031,8 +1024,8 @@ static int fpm_conf_process_all_pools() /* {{{ */
 				nb_ext = 0;
 
 				/* parse the string and save the extension in the array */
-				while ((ext = strtok(limit_extensions, " \t"))) {
-					limit_extensions = NULL;
+				while ((ext = strtok(security_limit_extensions, " \t"))) {
+					security_limit_extensions = NULL;
 					wp->limit_extensions[nb_ext++] = strdup(ext);
 				}
 
@@ -1114,7 +1107,6 @@ int fpm_conf_write_pid() /* {{{ */
 
 		if (len != write(fd, buf, len)) {
 			zlog(ZLOG_SYSERROR, "Unable to write to the PID file.");
-			close(fd);
 			return -1;
 		}
 		close(fd);
@@ -1123,7 +1115,7 @@ int fpm_conf_write_pid() /* {{{ */
 }
 /* }}} */
 
-static int fpm_conf_post_process(int force_daemon TSRMLS_DC) /* {{{ */
+static int fpm_conf_post_process(TSRMLS_D) /* {{{ */
 {
 	struct fpm_worker_pool_s *wp;
 
@@ -1131,11 +1123,6 @@ static int fpm_conf_post_process(int force_daemon TSRMLS_DC) /* {{{ */
 		fpm_evaluate_full_path(&fpm_global_config.pid_file, NULL, PHP_LOCALSTATEDIR, 0);
 	}
 
-	if (force_daemon >= 0) {
-		/* forced from command line options */
-		fpm_global_config.daemonize = force_daemon;
-	}
-
 	fpm_globals.log_level = fpm_global_config.log_level;
 
 	if (fpm_global_config.process_max < 0) {
@@ -1468,7 +1455,6 @@ int fpm_conf_load_ini_file(char *filename TSRMLS_DC) /* {{{ */
 
 	if (ini_recursion++ > 4) {
 		zlog(ZLOG_ERROR, "failed to include more than 5 files recusively");
-		close(fd);
 		return -1;
 	}
 
@@ -1598,7 +1584,7 @@ static void fpm_conf_dump() /* {{{ */
 }
 /* }}} */
 
-int fpm_conf_init_main(int test_conf, int force_daemon) /* {{{ */
+int fpm_conf_init_main(int test_conf) /* {{{ */
 {
 	int ret;
 	TSRMLS_FETCH();
@@ -1644,7 +1630,7 @@ int fpm_conf_init_main(int test_conf, int force_daemon) /* {{{ */
 		return -1;
 	}
 
-	if (0 > fpm_conf_post_process(force_daemon TSRMLS_CC)) {
+	if (0 > fpm_conf_post_process(TSRMLS_C)) {
 		zlog(ZLOG_ERROR, "failed to post process the configuration");
 		return -1;
 	}
