@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2009 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -155,8 +155,6 @@ static const opt_struct OPTIONS[] = {
 	{'p', 1, "prefix"},
 	{'g', 1, "pid"},
 	{'R', 0, "allow-to-run-as-root"},
-	{'D', 0, "daemonize"},
-	{'F', 0, "nodaemonize"},
 	{'-', 0, NULL} /* end of args */
 };
 
@@ -561,7 +559,6 @@ void cgi_php_import_environment_variables(zval *array_ptr TSRMLS_DC)
 {
 	fcgi_request *request;
 	HashPosition pos;
-	int magic_quotes_gpc;;
 	char *var, **val;
 	uint var_len;
 	ulong idx;
@@ -594,13 +591,8 @@ void cgi_php_import_environment_variables(zval *array_ptr TSRMLS_DC)
 	php_php_import_environment_variables(array_ptr TSRMLS_CC);
 
 	request = (fcgi_request*) SG(server_context);
-	magic_quotes_gpc = PG(magic_quotes_gpc);
 	filter_arg = (array_ptr == PG(http_globals)[TRACK_VARS_ENV])?PARSE_ENV:PARSE_SERVER;
 
-	/* turn off magic_quotes while importing environment variables */
-	if (magic_quotes_gpc) {
-		zend_alter_ini_entry_ex("magic_quotes_gpc", sizeof("magic_quotes_gpc"), "0", 1, ZEND_INI_SYSTEM, ZEND_INI_STAGE_ACTIVATE, 1 TSRMLS_CC);
-	}
 	for (zend_hash_internal_pointer_reset_ex(request->env, &pos);
 	     zend_hash_get_current_key_ex(request->env, &var, &var_len, &idx, 0, &pos) == HASH_KEY_IS_STRING &&
 	     zend_hash_get_current_data_ex(request->env, (void **) &val, &pos) == SUCCESS;
@@ -612,9 +604,6 @@ void cgi_php_import_environment_variables(zval *array_ptr TSRMLS_DC)
 			php_register_variable_safe(var, *val, new_val_len, array_ptr TSRMLS_CC);
 		}
 	}
-	if (magic_quotes_gpc) {
-		zend_alter_ini_entry_ex("magic_quotes_gpc", sizeof("magic_quotes_gpc"), "1", 1, ZEND_INI_SYSTEM, ZEND_INI_STAGE_ACTIVATE, 1 TSRMLS_CC);
-	}
 }
 
 static void sapi_cgi_register_variables(zval *track_vars_array TSRMLS_DC)
@@ -923,7 +912,7 @@ static void php_cgi_usage(char *argv0)
 		prog = "php";
 	}
 
-	php_printf(	"Usage: %s [-n] [-e] [-h] [-i] [-m] [-v] [-t] [-p <prefix>] [-g <pid>] [-c <file>] [-d foo[=bar]] [-y <file>] [-D] [-F]\n"
+	php_printf(	"Usage: %s [-n] [-e] [-h] [-i] [-m] [-v] [-t] [-p <prefix>] [-g <pid>] [-c <file>] [-d foo[=bar]] [-y <file>]\n"
 				"  -c <path>|<file> Look for php.ini file in this directory\n"
 				"  -n               No php.ini file will be used\n"
 				"  -d foo[=bar]     Define INI entry foo with value 'bar'\n"
@@ -939,12 +928,8 @@ static void php_cgi_usage(char *argv0)
 				"  -y, --fpm-config <file>\n"
 				"                   Specify alternative path to FastCGI process manager config file.\n"
 				"  -t, --test       Test FPM configuration and exit\n"
-				"  -D, --daemonize  force to run in background, and ignore daemonize option from config file\n"
-				"  -F, --nodaemonize\n"
-				"                   force to stay in foreground, and ignore daemonize option from config file\n"
 				"  -R, --allow-to-run-as-root\n"
 				"                   Allow pool to run as root (disabled by default)\n",
-
 				prog, PHP_PREFIX);
 }
 /* }}} */
@@ -1179,123 +1164,119 @@ static void init_request_info(TSRMLS_D)
 				int len = script_path_translated_len;
 				char *ptr;
 
-				if (pt) {
-					while ((ptr = strrchr(pt, '/')) || (ptr = strrchr(pt, '\\'))) {
-						*ptr = 0;
-						if (stat(pt, &st) == 0 && S_ISREG(st.st_mode)) {
-							/*
-							 * okay, we found the base script!
-							 * work out how many chars we had to strip off;
-							 * then we can modify PATH_INFO
-							 * accordingly
-							 *
-							 * we now have the makings of
-							 * PATH_INFO=/test
-							 * SCRIPT_FILENAME=/docroot/info.php
-							 *
-							 * we now need to figure out what docroot is.
-							 * if DOCUMENT_ROOT is set, this is easy, otherwise,
-							 * we have to play the game of hide and seek to figure
-							 * out what SCRIPT_NAME should be
-							 */
-							int ptlen = strlen(pt);
-							int slen = len - ptlen;
-							int pilen = env_path_info ? strlen(env_path_info) : 0;
-							int tflag = 0;
-							char *path_info;
-							if (apache_was_here) {
-								/* recall that PATH_INFO won't exist */
-								path_info = script_path_translated + ptlen;
-								tflag = (slen != 0 && (!orig_path_info || strcmp(orig_path_info, path_info) != 0));
-							} else {
-								path_info = env_path_info ? env_path_info + pilen - slen : NULL;
-								tflag = (orig_path_info != path_info);
-							}
-
-							if (tflag) {
-								if (orig_path_info) {
-									char old;
-
-									_sapi_cgibin_putenv("ORIG_PATH_INFO", orig_path_info TSRMLS_CC);
-									old = path_info[0];
-									path_info[0] = 0;
-									if (!orig_script_name ||
-										strcmp(orig_script_name, env_path_info) != 0) {
-										if (orig_script_name) {
-											_sapi_cgibin_putenv("ORIG_SCRIPT_NAME", orig_script_name TSRMLS_CC);
-										}
-										SG(request_info).request_uri = _sapi_cgibin_putenv("SCRIPT_NAME", env_path_info TSRMLS_CC);
-									} else {
-										SG(request_info).request_uri = orig_script_name;
-									}
-									path_info[0] = old;
-								}
-								env_path_info = _sapi_cgibin_putenv("PATH_INFO", path_info TSRMLS_CC);
-							}
-							if (!orig_script_filename ||
-								strcmp(orig_script_filename, pt) != 0) {
-								if (orig_script_filename) {
-									_sapi_cgibin_putenv("ORIG_SCRIPT_FILENAME", orig_script_filename TSRMLS_CC);
-								}
-								script_path_translated = _sapi_cgibin_putenv("SCRIPT_FILENAME", pt TSRMLS_CC);
-							}
-							TRANSLATE_SLASHES(pt);
-
-							/* figure out docroot
-							 * SCRIPT_FILENAME minus SCRIPT_NAME
-							 */
-							if (env_document_root) {
-								int l = strlen(env_document_root);
-								int path_translated_len = 0;
-								char *path_translated = NULL;
-
-								if (l && env_document_root[l - 1] == '/') {
-									--l;
-								}
-
-								/* we have docroot, so we should have:
-								 * DOCUMENT_ROOT=/docroot
-								 * SCRIPT_FILENAME=/docroot/info.php
-								 */
-
-								/* PATH_TRANSLATED = DOCUMENT_ROOT + PATH_INFO */
-								path_translated_len = l + (env_path_info ? strlen(env_path_info) : 0);
-								path_translated = (char *) emalloc(path_translated_len + 1);
-								memcpy(path_translated, env_document_root, l);
-								if (env_path_info) {
-									memcpy(path_translated + l, env_path_info, (path_translated_len - l));
-								}
-								path_translated[path_translated_len] = '\0';
-								if (orig_path_translated) {
-									_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
-								}
-								env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
-								efree(path_translated);
-							} else if (	env_script_name &&
-										strstr(pt, env_script_name)
-							) {
-								/* PATH_TRANSLATED = PATH_TRANSLATED - SCRIPT_NAME + PATH_INFO */
-								int ptlen = strlen(pt) - strlen(env_script_name);
-								int path_translated_len = ptlen + (env_path_info ? strlen(env_path_info) : 0);
-								char *path_translated = NULL;
-
-								path_translated = (char *) emalloc(path_translated_len + 1);
-								memcpy(path_translated, pt, ptlen);
-								if (env_path_info) {
-									memcpy(path_translated + ptlen, env_path_info, path_translated_len - ptlen);
-								}
-								path_translated[path_translated_len] = '\0';
-								if (orig_path_translated) {
-									_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
-								}
-								env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
-								efree(path_translated);
-							}
-							break;
+				while ((ptr = strrchr(pt, '/')) || (ptr = strrchr(pt, '\\'))) {
+					*ptr = 0;
+					if (stat(pt, &st) == 0 && S_ISREG(st.st_mode)) {
+						/*
+						 * okay, we found the base script!
+						 * work out how many chars we had to strip off;
+						 * then we can modify PATH_INFO
+						 * accordingly
+						 *
+						 * we now have the makings of
+						 * PATH_INFO=/test
+						 * SCRIPT_FILENAME=/docroot/info.php
+						 *
+						 * we now need to figure out what docroot is.
+						 * if DOCUMENT_ROOT is set, this is easy, otherwise,
+						 * we have to play the game of hide and seek to figure
+						 * out what SCRIPT_NAME should be
+						 */
+						int ptlen = strlen(pt);
+						int slen = len - ptlen;
+						int pilen = env_path_info ? strlen(env_path_info) : 0;
+						int tflag = 0;
+						char *path_info;
+						if (apache_was_here) {
+							/* recall that PATH_INFO won't exist */
+							path_info = script_path_translated + ptlen;
+							tflag = (slen != 0 && (!orig_path_info || strcmp(orig_path_info, path_info) != 0));
+						} else {
+							path_info = env_path_info ? env_path_info + pilen - slen : NULL;
+							tflag = (orig_path_info != path_info);
 						}
+
+						if (tflag) {
+							if (orig_path_info) {
+								char old;
+
+								_sapi_cgibin_putenv("ORIG_PATH_INFO", orig_path_info TSRMLS_CC);
+								old = path_info[0];
+								path_info[0] = 0;
+								if (!orig_script_name ||
+									strcmp(orig_script_name, env_path_info) != 0) {
+									if (orig_script_name) {
+										_sapi_cgibin_putenv("ORIG_SCRIPT_NAME", orig_script_name TSRMLS_CC);
+									}
+									SG(request_info).request_uri = _sapi_cgibin_putenv("SCRIPT_NAME", env_path_info TSRMLS_CC);
+								} else {
+									SG(request_info).request_uri = orig_script_name;
+								}
+								path_info[0] = old;
+							}
+							env_path_info = _sapi_cgibin_putenv("PATH_INFO", path_info TSRMLS_CC);
+						}
+						if (!orig_script_filename ||
+							strcmp(orig_script_filename, pt) != 0) {
+							if (orig_script_filename) {
+								_sapi_cgibin_putenv("ORIG_SCRIPT_FILENAME", orig_script_filename TSRMLS_CC);
+							}
+							script_path_translated = _sapi_cgibin_putenv("SCRIPT_FILENAME", pt TSRMLS_CC);
+						}
+						TRANSLATE_SLASHES(pt);
+
+						/* figure out docroot
+						 * SCRIPT_FILENAME minus SCRIPT_NAME
+						 */
+						if (env_document_root) {
+							int l = strlen(env_document_root);
+							int path_translated_len = 0;
+							char *path_translated = NULL;
+
+							if (l && env_document_root[l - 1] == '/') {
+								--l;
+							}
+
+							/* we have docroot, so we should have:
+							 * DOCUMENT_ROOT=/docroot
+							 * SCRIPT_FILENAME=/docroot/info.php
+							 */
+
+							/* PATH_TRANSLATED = DOCUMENT_ROOT + PATH_INFO */
+							path_translated_len = l + (env_path_info ? strlen(env_path_info) : 0);
+							path_translated = (char *) emalloc(path_translated_len + 1);
+							memcpy(path_translated, env_document_root, l);
+							if (env_path_info) {
+								memcpy(path_translated + l, env_path_info, (path_translated_len - l));
+							}
+							path_translated[path_translated_len] = '\0';
+							if (orig_path_translated) {
+								_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
+							}
+							env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
+							efree(path_translated);
+						} else if (	env_script_name &&
+									strstr(pt, env_script_name)
+						) {
+							/* PATH_TRANSLATED = PATH_TRANSLATED - SCRIPT_NAME + PATH_INFO */
+							int ptlen = strlen(pt) - strlen(env_script_name);
+							int path_translated_len = ptlen + (env_path_info ? strlen(env_path_info) : 0);
+							char *path_translated = NULL;
+
+							path_translated = (char *) emalloc(path_translated_len + 1);
+							memcpy(path_translated, pt, ptlen);
+							if (env_path_info) {
+								memcpy(path_translated + ptlen, env_path_info, path_translated_len - ptlen);
+							}
+							path_translated[path_translated_len] = '\0';
+							if (orig_path_translated) {
+								_sapi_cgibin_putenv("ORIG_PATH_TRANSLATED", orig_path_translated TSRMLS_CC);
+							}
+							env_path_translated = _sapi_cgibin_putenv("PATH_TRANSLATED", path_translated TSRMLS_CC);
+							efree(path_translated);
+						}
+						break;
 					}
-				} else {
-					ptr = NULL;
 				}
 				if (!ptr) {
 					/*
@@ -1512,7 +1493,7 @@ PHP_FUNCTION(fastcgi_finish_request) /* {{{ */
 
 	if (request->fd >= 0) {
 
-		php_end_ob_buffers(1 TSRMLS_CC);
+		php_output_end_all(TSRMLS_C);
 		php_header(TSRMLS_C);
 
 		fcgi_flush(request, 1);
@@ -1548,7 +1529,7 @@ static zend_module_entry cgi_module_entry = {
 int main(int argc, char *argv[])
 {
 	int exit_status = FPM_EXIT_OK;
-	int cgi = 0, c;
+	int cgi = 0, c, use_extended_info = 0;
 	zend_file_handle file_handle;
 
 	/* temporary locals */
@@ -1569,12 +1550,9 @@ int main(int argc, char *argv[])
 	char *fpm_prefix = NULL;
 	char *fpm_pid = NULL;
 	int test_conf = 0;
-	int force_daemon = -1;
 	int php_information = 0;
 	int php_allow_to_run_as_root = 0;
 
-	fcgi_init();
-
 #ifdef HAVE_SIGNAL_H
 #if defined(SIGPIPE) && defined(SIG_IGN)
 	signal(SIGPIPE, SIG_IGN); /* ignore SIGPIPE in standalone mode so
@@ -1593,6 +1571,9 @@ int main(int argc, char *argv[])
 
 	sapi_startup(&cgi_sapi_module);
 	cgi_sapi_module.php_ini_path_override = NULL;
+	cgi_sapi_module.php_ini_ignore_cwd = 1;
+	
+	fcgi_init();
 
 #ifdef PHP_WIN32
 	_fmode = _O_BINARY; /* sets default for file streams to binary */
@@ -1659,7 +1640,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'e': /* enable extended info output */
-				CG(compiler_options) |= ZEND_COMPILE_EXTENDED_INFO;
+				use_extended_info = 1;
 				break;
 
 			case 't': 
@@ -1668,7 +1649,6 @@ int main(int argc, char *argv[])
 
 			case 'm': /* list compiled in modules */
 				cgi_sapi_module.startup(&cgi_sapi_module);
-				php_output_startup();
 				php_output_activate(TSRMLS_C);
 				SG(headers_sent) = 1;
 				php_printf("[PHP Modules]\n");
@@ -1676,7 +1656,8 @@ int main(int argc, char *argv[])
 				php_printf("\n[Zend Modules]\n");
 				print_extensions(TSRMLS_C);
 				php_printf("\n");
-				php_end_ob_buffers(1 TSRMLS_CC);
+				php_output_end_all(TSRMLS_C);
+				php_output_deactivate(TSRMLS_C);
 				fcgi_shutdown();
 				exit_status = FPM_EXIT_OK;
 				goto out;
@@ -1689,23 +1670,15 @@ int main(int argc, char *argv[])
 				php_allow_to_run_as_root = 1;
 				break;
 
-			case 'D': /* daemonize */
-				force_daemon = 1;
-				break;
-
-			case 'F': /* nodaemonize */
-				force_daemon = 0;
-				break;
-
 			default:
 			case 'h':
 			case '?':
 				cgi_sapi_module.startup(&cgi_sapi_module);
-				php_output_startup();
 				php_output_activate(TSRMLS_C);
 				SG(headers_sent) = 1;
 				php_cgi_usage(argv[0]);
-				php_end_ob_buffers(1 TSRMLS_CC);
+				php_output_end_all(TSRMLS_C);
+				php_output_deactivate(TSRMLS_C);
 				fcgi_shutdown();
 				exit_status = (c == 'h') ? FPM_EXIT_OK : FPM_EXIT_USAGE;
 				goto out;
@@ -1721,9 +1694,9 @@ int main(int argc, char *argv[])
 				SG(request_info).no_headers = 1;
 
 #if ZEND_DEBUG
-				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2014 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__,        __TIME__, get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2009 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__,        __TIME__, get_zend_version());
 #else
-				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2014 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__,      get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2009 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__,      get_zend_version());
 #endif
 				php_request_shutdown((void *) 0);
 				fcgi_shutdown();
@@ -1749,16 +1722,16 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	/* No other args are permitted here as there is not interactive mode */
+	/* No other args are permitted here as there is no interactive mode */
 	if (argc != php_optind) {
 		cgi_sapi_module.startup(&cgi_sapi_module);
-		php_output_startup();
 		php_output_activate(TSRMLS_C);
 		SG(headers_sent) = 1;
 		php_cgi_usage(argv[0]);
-		php_end_ob_buffers(1 TSRMLS_CC);
-		exit_status = FPM_EXIT_USAGE;
+		php_output_end_all(TSRMLS_C);
+		php_output_deactivate(TSRMLS_C);
 		fcgi_shutdown();
+		exit_status = FPM_EXIT_USAGE;
 		goto out;
 	}
 
@@ -1779,6 +1752,10 @@ int main(int argc, char *argv[])
 #endif
 		return FPM_EXIT_SOFTWARE;
 	}
+	
+	if (use_extended_info) {
+		CG(compiler_options) |= ZEND_COMPILE_EXTENDED_INFO;
+	}
 
 	/* check force_cgi after startup, so we have proper output */
 	if (cgi && CGIG(force_redirect)) {
@@ -1820,22 +1797,18 @@ consult the installation file that came with this distribution, or visit \n\
 		}
 	}
 
-	if (0 > fpm_init(argc, argv, fpm_config ? fpm_config : CGIG(fpm_config), fpm_prefix, fpm_pid, test_conf, php_allow_to_run_as_root, force_daemon)) {
+	if (0 > fpm_init(argc, argv, fpm_config ? fpm_config : CGIG(fpm_config), fpm_prefix, fpm_pid, test_conf, php_allow_to_run_as_root)) {
 
-		if (fpm_globals.send_config_pipe[1]) {
-			int writeval = 0;
-			zlog(ZLOG_DEBUG, "Sending \"0\" (error) to parent via fd=%d", fpm_globals.send_config_pipe[1]);
-			write(fpm_globals.send_config_pipe[1], &writeval, sizeof(writeval));
-			close(fpm_globals.send_config_pipe[1]);
+		if (fpm_globals.send_config_signal) {
+			zlog(ZLOG_DEBUG, "Sending SIGUSR2 (error) to parent %d", getppid());
+			kill(getppid(), SIGUSR2);
 		}
 		return FPM_EXIT_CONFIG;
 	}
 
-	if (fpm_globals.send_config_pipe[1]) {
-		int writeval = 1;
-		zlog(ZLOG_DEBUG, "Sending \"1\" (OK) to parent via fd=%d", fpm_globals.send_config_pipe[1]);
-		write(fpm_globals.send_config_pipe[1], &writeval, sizeof(writeval));
-		close(fpm_globals.send_config_pipe[1]);
+	if (fpm_globals.send_config_signal) {
+		zlog(ZLOG_DEBUG, "Sending SIGUSR1 (OK) to parent %d", getppid());
+		kill(getppid(), SIGUSR1);
 	}
 	fpm_is_running = 1;
 
