@@ -167,7 +167,7 @@ static int fpm_sockets_new_listening_socket(struct fpm_worker_pool_s *wp, struct
 {
 	int flags = 1;
 	int sock;
-	mode_t saved_umask = 0;
+	mode_t saved_umask;
 
 	sock = socket(sa->sa_family, SOCK_STREAM, 0);
 
@@ -176,14 +176,11 @@ static int fpm_sockets_new_listening_socket(struct fpm_worker_pool_s *wp, struct
 		return -1;
 	}
 
-	if (0 > setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &flags, sizeof(flags))) {
-		zlog(ZLOG_WARNING, "failed to change socket attribute");
-	}
+	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &flags, sizeof(flags));
 
 	if (wp->listen_address_domain == FPM_AF_UNIX) {
 		if (fpm_socket_unix_test_connect((struct sockaddr_un *)sa, socklen) == 0) {
 			zlog(ZLOG_ERROR, "An another FPM instance seems to already listen on %s", ((struct sockaddr_un *) sa)->sun_path);
-			close(sock);
 			return -1;
 		}
 		unlink( ((struct sockaddr_un *) sa)->sun_path);
@@ -195,7 +192,6 @@ static int fpm_sockets_new_listening_socket(struct fpm_worker_pool_s *wp, struct
 		if (wp->listen_address_domain == FPM_AF_UNIX) {
 			umask(saved_umask);
 		}
-		close(sock);
 		return -1;
 	}
 
@@ -207,7 +203,6 @@ static int fpm_sockets_new_listening_socket(struct fpm_worker_pool_s *wp, struct
 		if (wp->socket_uid != -1 || wp->socket_gid != -1) {
 			if (0 > chown(path, wp->socket_uid, wp->socket_gid)) {
 				zlog(ZLOG_SYSERROR, "failed to chown() the socket '%s'", wp->config->listen_address);
-				close(sock);
 				return -1;
 			}
 		}
@@ -215,7 +210,6 @@ static int fpm_sockets_new_listening_socket(struct fpm_worker_pool_s *wp, struct
 
 	if (0 > listen(sock, wp->config->listen_backlog)) {
 		zlog(ZLOG_SYSERROR, "failed to listen to address '%s'", wp->config->listen_address);
-		close(sock);
 		return -1;
 	}
 
@@ -362,7 +356,7 @@ int fpm_sockets_init_main() /* {{{ */
 		}
 
 	if (wp->listen_address_domain == FPM_AF_INET && fpm_socket_get_listening_queue(wp->listening_socket, NULL, &lq_len) >= 0) {
-			fpm_scoreboard_update(-1, -1, -1, (int)lq_len, -1, -1, FPM_SCOREBOARD_ACTION_SET, wp->scoreboard);
+			fpm_scoreboard_update(-1, -1, -1, (int)lq_len, -1, -1, 0, FPM_SCOREBOARD_ACTION_SET, wp->scoreboard);
 		}
 	}
 
