@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -1040,11 +1040,10 @@ int main(int argc, char *argv[])
 		while ((c=ap_php_getopt(argc, argv, OPTSTRING))!=-1) {
 			switch (c) {
 			case '?':
-				php_output_startup();
-				php_output_activate(TSRMLS_C);
+				php_output_tearup();
 				SG(headers_sent) = 1;
 				php_milter_usage(argv[0]);
-				php_end_ob_buffers(1 TSRMLS_CC);
+				php_output_teardown();
 				exit(1);
 				break;
 			}
@@ -1088,11 +1087,10 @@ int main(int argc, char *argv[])
 
 			case 'h': /* help & quit */
 			case '?':
-				php_output_startup();
-				php_output_activate(TSRMLS_C);
+				php_output_tearup();
 				SG(headers_sent) = 1;
 				php_milter_usage(argv[0]);
-				php_end_ob_buffers(1 TSRMLS_CC);
+				php_output_teardown();
 				exit(1);
 				break;
 
@@ -1111,8 +1109,8 @@ int main(int argc, char *argv[])
 				}
 				SG(headers_sent) = 1;
 				SG(request_info).no_headers = 1;
-				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2014 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
-				php_end_ob_buffers(1 TSRMLS_CC);
+				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2012 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
+				php_output_teardown();
 				exit(1);
 				break;
 
