@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2011 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,12 +17,12 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: winutil.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id$ */
 
 #include "php.h"
 #include <wincrypt.h>
 
-PHPAPI char *php_win_err(int error)
+PHPAPI char *php_win32_error_to_msg(int error)
 {
 	char *buf = NULL;
 
@@ -49,26 +49,74 @@ int php_win32_check_trailing_space(const char * path, const int path_len) {
 	}
 }
 
+HCRYPTPROV   hCryptProv;
+unsigned int has_crypto_ctx = 0;
+
+#ifdef ZTS
+MUTEX_T php_lock_win32_cryptoctx;
+void php_win32_init_rng_lock()
+{
+	php_lock_win32_cryptoctx = tsrm_mutex_alloc();
+}
+
+void php_win32_free_rng_lock()
+{
+	tsrm_mutex_lock(php_lock_win32_cryptoctx);
+	if (has_crypto_ctx == 1) {
+		CryptReleaseContext(hCryptProv, 0);
+		has_crypto_ctx = 0;
+	}
+	tsrm_mutex_unlock(php_lock_win32_cryptoctx);
+	tsrm_mutex_free(php_lock_win32_cryptoctx);
+
+}
+#else
+#define php_win32_init_rng_lock();
+#define php_win32_free_rng_lock();
+#endif
+
+
+
 PHPAPI int php_win32_get_random_bytes(unsigned char *buf, size_t size) {  /* {{{ */
-	HCRYPTPROV   hCryptProv;
-	int has_context = 0;
+
+	unsigned int has_contextg = 0;
+
 	BOOL ret;
 	size_t i = 0;
 
-	if (!CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, 0)) {
-		/* Could mean that the key container does not exist, let try 
-		   again by asking for a new one */
-		if (GetLastError() == NTE_BAD_KEYSET) {
-			if (CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET)) {
-				has_context = 1;
-			} else {
-				return FAILURE;
+#ifdef ZTS
+	tsrm_mutex_lock(php_lock_win32_cryptoctx);
+#endif
+
+	if (has_crypto_ctx == 0) {
+		/* CRYPT_VERIFYCONTEXT > only hashing&co-like use, no need to acces prv keys */
+		if (!CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_MACHINE_KEYSET|CRYPT_VERIFYCONTEXT )) {
+			/* Could mean that the key container does not exist, let try 
+			   again by asking for a new one. If it fails here, it surely means that the user running 
+               this process does not have the permission(s) to use this container.
+             */
+			if (GetLastError() == NTE_BAD_KEYSET) {
+				if (CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_NEWKEYSET | CRYPT_MACHINE_KEYSET | CRYPT_VERIFYCONTEXT )) {
+					has_crypto_ctx = 1;
+				} else {
+					has_crypto_ctx = 0;
+				}
 			}
+		} else {
+			has_crypto_ctx = 1;
 		}
 	}
 
+#ifdef ZTS
+	tsrm_mutex_unlock(php_lock_win32_cryptoctx);
+#endif
+
+	if (has_crypto_ctx == 0) {
+		return FAILURE;
+	}
+
 	ret = CryptGenRandom(hCryptProv, size, buf);
-	CryptReleaseContext(hCryptProv, 0);
+
 	if (ret) {
 		return SUCCESS;
 	} else {
